program CpGriTy
!
! Determines critical points (CP) of an electron density given on a
! regular (but not necessary orthogonal) grid.
! --> search for CP by looking for minimum of the gradient
!     obtained by interpolating the density with a
!     Tricubic Lagrange Method
! --> determines the eigenvectors and eigenvalues of the hessian
!     matrix ==> CP curvatures lamdda1, lambda2 and lambda3
!
! --> creates tables with sorted CP"s
!     selected CP cases:
!          1)  (3,-1) saddle points low density  :: inter-molecular CP ?
!          2)  (3,-1)    "      "   high   "     :: intra-molecular CP ?
!          3)  (3,+1) ring CP
!          4)  (3,+3) cage CP
!          5)  (3,-3) attractor
!          6)  other ?
!
!
! cc stands for cartesian coordinates
!
! June.2003      C. Katan and P. Rabiller GMCM Rennes
!                and M. Souhassou LCM3B Nancy
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
use bondpath_parameters
use constants_module
use cpgrity_dx_mod
use cpgrity_mod
use cp_module
use filename_module
use grid_module
use kind_numbers
use latt_module
use math_module
use old_module
use rho_module
!
implicit none
!
intrinsic :: cpu_time,date_and_time,min
!
integer(kind=int16) :: icount,iginc !!! step count
!
integer(kind=int32) :: il,jl,kl     !!! integer for loops
integer(kind=int32), dimension(8) :: datetime,datetime_old
!
real :: cpu_begin,cpu_end
real(kind=double) :: dens      !!! interpolated density
real(kind=double) :: omega,sigma !!! rank and signature of a CP
real(kind=double), dimension(3) :: ri,gradi !!! values of position, gradient
real(kind=double), dimension(3) :: r,rcp    !!! position in cartesian coordinates (cc)
real(kind=double), dimension(3) :: grad     !!! interpolated gradient in cc
real(kind=double), dimension(3) :: lambda   !!! curvatures
real(kind=double), dimension(3) :: rifirst  !!! first ri
real(kind=double), dimension(3) :: deltari  !!! |rifirst - ri|
!
real(kind=double), dimension(3,3) :: hess    !!! interpolated hessian  in cc
real(kind=double), dimension(3,3) :: hessi   !!! and hessian in grid coordinates
real(kind=double), dimension(3,3) :: invhess !!! invert of hess
real(kind=double), dimension(3,3) :: mat,eigenvec !!! dummy matrix
!
logical :: cr_flag        !!! flag for "carriage return on display output
!
character(len=*), parameter :: fmt0="(tr2,a,3f12.6,tr5,3f12.6)"
character(len=*), parameter :: fmt1="(a,2(i2,tr1),i4,a4,2(i2,a1),i2)"
character(len=*), parameter :: fmt2="(a,es10.2)"
character(len=*), parameter :: fmt3="(a,i10)"
character(len=*), parameter :: fmt4="(a,i6)"
character(len=*), parameter :: fmt5="(2a)"
character(len=*), parameter :: fmt6="(a,3f10.5,a,f10.5)"
character(len=*), parameter :: fmt7="(a1,i3,a1,i3,a2)"
character(len=*), parameter :: fmt8="(a3,i2,a1,i2,a4,3f6.1,tr3,3f6.1,2(a10,es9.2))"
!
!===============================================================================
!
call cpu_time(cpu_begin)
write(unit=*,fmt=fmta) "  Program CpGriTy running ... "
call date_and_time(values=datetime)
call date_and_time(values=datetime_old)
write(unit=*,fmt=fmta) ""
write(unit=*,fmt=fmt1) "CP search started on ",datetime(3),datetime(2),datetime(1)," at ",datetime(5),":",datetime(6),":", &
datetime(7)
write(unit=*,fmt=fmta) "Initialize parameters for cpgrity"
write(unit=*,fmt=fmta) ""
!
!print *,"bondpath_parameters: n_max,max_bond_length,dr_cur,d_max,d_write,do_at,c_o:",n_max,max_bond_length,dr_cur,d_max,d_write, &
!do_at,c_o
!print *,"cp_module: ncmax:",ncmax
!print *,"math_module: EpsOff:",EpsOff
!print *,"cpgrity_mod: nmaxgstep,maxat,invfric,dgradmin,rioff,densmaxcp,densoffcp,xmingrad:",nmaxgstep,maxat,invfric,dgradmin, &
!rioff,densmaxcp,densoffcp,xmingrad
!
call cpgrity_initialize()
!
!print *,"bondpath_parameters: n_max,max_bond_length,dr_cur,d_max,d_write,do_at,c_o:",n_max,max_bond_length,dr_cur,d_max,d_write, &
!do_at,c_o
!print *,"cp_module: ncmax:",ncmax
!print *,"math_module: EpsOff:",EpsOff
!print *,"cpgrity_mod: nmaxgstep,maxat,invfric,dgradmin,rioff,densmaxcp,densoffcp,xmingrad:",nmaxgstep,maxat,invfric,dgradmin, &
!rioff,densmaxcp,densoffcp,xmingrad
!
write(unit=*,fmt=fmta) ""
write(unit=*,fmt=fmt5) "    file_path= ",trim(file_path)
write(unit=*,fmt=fmt5) "   atoms file= ",trim(filename_1)
!
nfil=trim(file_path)//trim(filename_1)//".in"
write(unit=*,fmt=fmt5) "   nfil= ",trim(nfil)
open(unit=11,file=trim(nfil),status="old",action="read",position="rewind")
read(unit=11,fmt=fmta) filename_2
write(unit=*,fmt=fmt5) "    data file= ",trim(filename_2)
write(unit=*,fmt=fmta) ""
rewind(unit=11)
!
nfil=trim(file_path)//trim(filename_2)//trim(extension_data)
write(unit=*,fmt=fmta) ""
write(unit=*,fmt=fmt5) " - Get electron density in ",trim(nfil)
open(unit=10,file=nfil,status="old",action="read",form="unformatted",position="rewind")
!
!      open main output file containing all parameters and results
extension=".report"
nfil=trim(file_path)//trim(filename_2)//trim(extension)
open(unit=13,file=nfil,status="new",action="write")
!
write(unit=13,fmt=fmta) ""
write(unit=13,fmt=fmt1) "CP search started on ",datetime(3),datetime(2),datetime(1)," at ",datetime(5),":",datetime(6)
write(unit=13,fmt=*)
!
!      open files to be used with OpenDx routines
!      writes atomic periodic images
!       extension="-per.dx"
!
write(unit=*,fmt=fmt2) " - InvFric       =",invfric
write(unit=*,fmt=fmt2) " - DgradMin      =",dgradmin
write(unit=*,fmt=fmt2) " - RiOff         =",riOff
write(unit=*,fmt="(a,f10.2)") " - NmaxGsteps =",real(nmaxgstep,double)
write(unit=*,fmt=fmt2) " - Dens Low/High =",densmaxcp
write(unit=*,fmt=fmt2) " - DensOff       =",densoffcp
write(unit=*,fmt=*)
!
write(unit=13,fmt=fmta) "Informations concerning constants used for cutoff,..."
write(unit=13,fmt=*)
write(unit=13,fmt=fmt2) " - EpsOff        =",epsoff
write(unit=13,fmt=fmt2) " - InvFric       =",invfric
write(unit=13,fmt=fmt2) " - DgradMin      =",dgradmin
write(unit=13,fmt=fmt2) " - RiOff         =",riOff
write(unit=13,fmt="(a,i5)") " - NmaxGsteps =",nmaxgstep
write(unit=13,fmt=fmt2) " - Dens Low/High =",densmaxcp
write(unit=13,fmt=fmt2) " - DensOff       =",densoffcp
write(unit=13,fmt=fmt3) " - Ncmax         =",ncmax
write(unit=13,fmt=fmt4) " - MaxAt         =",maxat
write(unit=13,fmt=*)
!
write(unit=13,fmt=*)
write(unit=13,fmt=fmt2) " - max_bond_length =",max_bond_length !! maximum "straight" bond length for output
write(unit=13,fmt=*)
write(unit=13,fmt=fmt2) " - bond_path_increment =",dr_cur  !! increment for Runge Kutta bond path search
write(unit=13,fmt=fmt2) " - bond_path_xdo       =",c_o !!  "xdo x increment = distance" from (3,-1) CP for starting bond path search
write(unit=13,fmt=fmt2) " - bond_path_dmax      =",d_max !! maximum run out distance for bond path search termination
write(unit=13,fmt="(a,i8)") " - bond_path_nmax      =",n_max !! maximum number of bond path search steps before termination
write(unit=13,fmt=fmt2) " - bond_path_d2at      =",do_at !! distance from running point to atom for bond path search termination
write(unit=13,fmt=fmt2) " - bond_path_dwrite    =",d_write!! increment for bond path output writing
write(unit=13,fmt=*)
!
write(unit=*,fmt=*)
write(unit=*,fmt=fmt2) " - max_bond_length =",max_bond_length !! maximum "straight" bond length for output
write(unit=*,fmt=*)
write(unit=*,fmt=fmt2) " - bond_path_increment =",dr_cur  !! increment for Runge Kutta bond path search
write(unit=*,fmt=fmt2) " - bond_path_xdo       =",c_o !!  "xdo x increment = distance" from (3,-1) CP for starting bond path search
write(unit=*,fmt=fmt2) " - bond_path_dmax      =",d_max !! maximum run out distance for bond path search termination
write(unit=*,fmt="(a,i8)") " - bond_path_nmax      =",n_max !! maximum number of bond path search steps before termination
write(unit=*,fmt=fmt2) " - bond_path_d2at      =",do_at !! distance from running point to atom for bond path search termination
write(unit=*,fmt=fmt2) " - bond_path_dwrite    =",d_write!! increment for bond path output writing
write(unit=*,fmt=*)
!
call grid_initialize()
write(unit=*,fmt=*)
write(unit=*,fmt=*) " ------ "
write(unit=*,fmt=*) "| GRID |"
write(unit=*,fmt=*) " ------ "
write(unit=*,fmt="(a,3i4)") " points #  ",np1,np2,np3
write(unit=*,fmt=fmt6) " origin = |",origin,"| "
write(unit=*,fmt=fmt6) "   del1 = |", del1(:),"| ",inc1
write(unit=*,fmt=fmt6) "   del2 = |", del2(:),"| ",inc2
write(unit=*,fmt=fmt6) "   del3 = |", del3(:),"| ",inc3
write(unit=*,fmt=fmt6) "      a = |",del1(:)*real(np1,double),"| ",inc1*real(np1,double)
write(unit=*,fmt=fmt6) "      b = |",del2(:)*real(np2,double),"| ",inc2*real(np2,double)
write(unit=*,fmt=fmt6) "      c = |",del3(:)*real(np3,double),"| ",inc3*real(np3,double)
write(unit=*,fmt="(3(a,f8.2))") "  alpha =",deg*alpha,"  beta=",deg*beta,"  gamma=",deg*gamma
write(unit=*,fmt=*)
write(unit=13,fmt=*)
write(unit=13,fmt=*) " ------ "
write(unit=13,fmt=*) "| GRID |"
write(unit=13,fmt=*) " ------ "
write(unit=13,fmt="(a,3i4)") " points #  ",np1,np2,np3
write(unit=13,fmt=fmt6) " origin = |",origin,"| "
write(unit=13,fmt=fmt6) "   del1 = |", del1(:),"| ",inc1
write(unit=13,fmt=fmt6) "   del2 = |", del2(:),"| ",inc2
write(unit=13,fmt=fmt6) "   del3 = |", del3(:),"| ",inc3
write(unit=13,fmt=fmt6) "      a = |",del1(:)*real(np1,double),"| ",inc1*real(np1,double)
write(unit=13,fmt=fmt6) "      b = |",del2(:)*real(np2,double),"| ",inc2*real(np2,double)
write(unit=13,fmt=fmt6) "      c = |",del3(:)*real(np3,double),"| ",inc3*real(np3,double)
write(unit=13,fmt="(3(a,f8.2))") "  alpha =",deg*alpha,"   beta=",deg*beta,"   gamma=",deg*gamma
write(unit=13,fmt=*)
!
if(iso_test) then
  r_box(1:3,1)=5
  r_box(1,2)=np1-4
  r_box(2,2)=np2-4
  r_box(3,2)=np3-4
!
  write(unit=*,fmt=fmta) ""
  write(unit=*,fmt=fmta) "  Non periodic conditions applied !"
  write(unit=*,fmt="(a,i3,a1,i3,a2)",advance="no") "   box restricted to [",r_box(1,1),":",r_box(1,2),"]"
  write(unit=*,fmt=fmt7,advance="no") "[",r_box(2,1),":",r_box(2,2),"]"
  write(unit=*,fmt=fmt7) "[",r_box(3,1),":",r_box(3,2),"]"
  write(unit=*,fmt=fmta) ""
  write(unit=13,fmt=fmta) ""
  write(unit=13,fmt=fmta) "  Non periodic conditions applied"
  write(unit=13,fmt="(a,i3,a1,i3,a2)",advance="no") "   box restricted to [",r_box(1,1),":",r_box(1,2),"]"
  write(unit=13,fmt=fmt7,advance="no") "[",r_box(2,1),":",r_box(2,2),"]"
  write(unit=13,fmt=fmt7) "[",r_box(3,1),":",r_box(3,2),"]"
  write(unit=13,fmt=fmta) ""
else
  write(unit=*,fmt=fmta) ""
  write(unit=*,fmt=fmta) "  Periodic conditions applied"
  r_box(1:3,1)=1
  r_box(1,2)=np1
  r_box(2,2)=np2
  r_box(3,2)=np3
end if
!
!! MaxDens=maxval(rho(r_box(1,1):r_box(1,2), r_box(2,1):r_box(2,2),r_box(3,1):r_box(3,2)))
!
call strc_initialize()
allocate(rho(np1,np2,np3))
call rho_initialize()
write(*,*)'ncmax=',ncmax
allocate(cp(6,ncmax,24))
!
!!!!!!!!!!!!!!!
write(unit=*,fmt="(a,es10.3)") "  minimum density =",minval(rho(r_box(1,1):r_box(1,2),r_box(2,1):r_box(2,2),r_box(3,1):r_box(3,2)))
write(unit=*,fmt="(a,es10.3)") "  maximum density =",maxval(rho(r_box(1,1):r_box(1,2),r_box(2,1):r_box(2,2),r_box(3,1):r_box(3,2)))
write(unit=*,fmt=*)
write(unit=*,fmt=*)    " ------- "
write(unit=*,fmt=fmta) " | ATOMS |"
write(unit=*,fmt=*)    " ------- "
write(unit=*,fmt=fmta) "  name       X           Y           Z                I           J           K"
write(unit=13,fmt=*)    " ------- "
write(unit=13,fmt=fmta) " | ATOMS |"
write(unit=13,fmt=*)    " ------- "
write(unit=13,fmt=fmta) "  name       X           Y           Z                I           J           K"
do il=1,nat
  write(unit=13,fmt=fmt0) lat(il),rat(il,:),matmul(c2g,rat(il,:)-origin(:))
  write(unit=*,fmt=fmt0) lat(il),rat(il,:),matmul(c2g,rat(il,:)-origin(:))
end do
write(unit=13,fmt=fmta) ""
write(unit=13,fmt=fmta) "Atoms obtained from periodic images:  3 character label like ""C_1 +.-"""
write(unit=13,fmt=fmta) "  means that atom C_1 is given the translation  (+1 x a) + (0 x b) + (-1 x c)"
write(unit=13,fmt=fmta) ""
write(unit=*,fmt=*)
write(unit=13,fmt=*)
!
!!!!!!!!!!!!!!!
!     compute xrho=min(rho)-eps if logarithmic transformation is asked for CP finding
!     this means that mrho has been initialized to mrho=0
!     and in subroutine "interp", rho is replaced by log(rho - xrho)
drho=minval(rho)-1.0e-6_double
drho=drho-1.0e-6_double
!
! SEARCH CP
! loops on positions in grid coordinates
!
write(unit=*,fmt=*)    " ----------------------- "
write(unit=*,fmt=fmta) " | LOOP OVER GRID POINTS |"
write(unit=*,fmt=*)    " ----------------------- "
write(unit=*,fmt=fmta) "  ( w, s)       X     Y     Z        I     J     K             rho(CP)         grad_rho(CP)"
rcp(:)=500.0_double
nc(:)=0
iginc=0
xmingrad=1.0e10_double
cr_flag=.false.                        !<<<<<<<
do il=r_box(1,1),r_box(1,2)
  ri(1)=real(il-1,double)
  do jl=r_box(2,1),r_box(2,2)
    ri(2)=real(jl-1,double)
    do kl=r_box(3,1),r_box(3,2)
      ri(3)=real(kl-1,double)
      rifirst=ri
!
      call interp(ri,dens,gradi,hessi)
!
! transform gradient from grid to cartesian coordinates
!
      r=matmul(g2c,ri(:))
      grad=matmul(c2gt,gradi)
      mat=matmul(c2gt,hessi)
      hess=matmul(mat,c2g)
      dgrad=sqrt(dot_product(grad,grad))
      xmingrad=min(xmingrad,dgrad)
!
! stores actual values in old
!
      dgradold=dgrad
      rold=r
      densold=dens
      hessold=hess
      icount=0
      cp_found=.false.
!
! search extrema of CD by minimizing the gradient
!
      do
        icount=icount+1
        call matinv(hess,invhess)
        r=r-matmul(invhess,grad)*invfric
        ri=matmul(c2g,r)
        call interp(ri,dens,gradi,hessi)
        grad=matmul(c2gt,gradi)
        mat=matmul(c2gt,hessi)
        hess=matmul(mat,c2g)
        dgrad=sqrt(dot_product(grad,grad))
        deltari=abs(rifirst-ri)
        if(maxval(deltari) >= RiOff) then
          exit  !!! the distance from the present grid point is too large , so exit
        end if
        if(icount > nmaxgstep) then
!write(unit=*,fmt="(a,3i4,a)") "exit, too much steps for nothing at",il,jl,kl, " go to next grid point "
!         write(unit=*,fmt="(a1)",advance="no") "."
          cr_flag=.true.
          exit
        end if
        if(dgrad < dgradmin) then
          cp_found=.true.
          exit
        else
          if(dgrad > dgradold) then
!!!  the gradient modulus increases
            iginc=iginc+1
!           if(iginc >= 5) then         !<<<<<<<<  if(iginc >= 2) then
!Achtung: mit iginc >= 5 erhält man bei Wasser einen Cage CP !!!!!!!!!!!!!!!!!
            if(iginc >= 2) then
!if(dgrad < 1.0e-5_double) write(unit=*,fmt="(a,3f8.3,tr5,2es11.4)") "gradient increases ",ri(:),dens,dgrad
              exit
            end if
          else
            iginc=0
            rold=r
            densold=dens
            dgradold=dgrad
            hessold=hess
          end if
        end if
      end do
!!!!!
      if(cp_found .and. abs(dens) > densoffcp) then
        call eigen(hess,lambda,eigenvec)
        call ranksig(lambda,omega,sigma)
!       if(sqrt(dot_product(r-rcp,r-rcp)) >= 0.05) then   !<<<<<<<<   >= 0.01  !!!!
        if(sqrt(dot_product(r-rcp,r-rcp)) >= 0.01) then
          if(cr_flag) then
            write(unit=*,fmt=fmta) ""
            cr_flag=.false.
          end if
          write(unit=*,fmt=fmt8) "  (",int(omega),",",int(sigma),")   ",r(:)+origin(:),ri(:),"       ",dens,"        ",dgrad
          rcp=r
        end if
        if(omega < three) call testcp(6,lambda,eigenvec)
        if(omega == three .and. sigma == three) call testcp(4,lambda,eigenvec)
        if(omega == three .and. sigma == one) call testcp(3,lambda,eigenvec)
        if(omega == three .and. sigma == -one .and. abs(dens) <= densmaxcp) call testcp(1,lambda,eigenvec)
        if(omega == three .and. sigma == -one .and. abs(dens) > densmaxcp) call testcp(2,lambda,eigenvec)
        if(cp_attr .and. omega == three .and. sigma == -three) call testcp(5,lambda,eigenvec)
      end if
      ri=rifirst
    end do
  end do
end do
!
write(unit=*,fmt="(/,a,es10.3,//)") "minimum gradient modulus over grid points :",xmingrad
!
! HANDLES CP FOR PRINT OUT
!
! change origin to the one of atomic coordinates
!
do il=1,6
  do jl=1,nc(il)
    cp(il,jl,1:3)=cp(il,jl,1:3)+origin(:)
  end do
  call writeoutput(il)
end do
!
allocate(indx_valid(nat))
call bond_paths_graph()
!
deallocate(rho)
deallocate(cp)
!
call date_and_time(values=datetime)
write(unit=*,fmt=fmta) ""
write(unit=*,fmt=fmt1) "CP search started on ",datetime_old(3),datetime_old(2),datetime_old(1)," at ",datetime_old(5),":", &
datetime_old(6),":",datetime_old(7)
write(unit=*,fmt=fmt1) "CP search done on    ",datetime(3),datetime(2),datetime(1)," at ",datetime(5),":",datetime(6),":", &
datetime(7)
write(unit=13,fmt=fmta) ""
write(unit=13,fmt=fmt1) "CP search started on ",datetime_old(3),datetime_old(2),datetime_old(1)," at ",datetime_old(5),":", &
datetime_old(6),":",datetime_old(7)
write(unit=13,fmt=fmt1) "CP search done on    ",datetime(3),datetime(2),datetime(1)," at ",datetime(5),":",datetime(6),":", &
datetime(7)
!
close(10)
close(13)
call cpu_time(cpu_end)
write(unit=*,fmt="(tr3,a,f9.2,a)") "Total elapsed CPU time = ",cpu_end-cpu_begin," sec"
!
end program cpgrity
