module cpgrity_dx_mod

use bondpath_parameters         
use constants_module
use cp_module
use filename_module
use grid_module
use kind_numbers
use latt_module
use math_module, only: eigen
use rho_module
use strc_module

implicit none
private

public :: bond_paths_graph

private :: interp_b,valid_atom,half_bond_path,print_cp_for_dx,cp_groups_for_dx,CP_label_dx,Cp_path_dx

intrinsic :: abs,dot_product,index,int,len_trim,matmul,minval,modulo,real,sqrt,sum,trim

integer(kind=int32), dimension(:), allocatable, public :: indx_valid !!! indices of valid atoms

real(kind=single), dimension(3), public :: cp_pos,cp_lambda

real(kind=double), public :: cp_dens,cp_lap,cp_r,cp_e
real(kind=double), dimension(10000,7), public :: bd_path_up  !!! array to temporary store bond-path
real(kind=double), dimension(10000,7), public :: bd_path_down  !!! array to temporary store bond-path
logical, public :: test_at
character(len=6), public :: checked_atom
character(len=10), public :: tcp

character(len=*), parameter, private :: gmt0="(tr3,a)"

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine interp_b(r,dens,grad,hess)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
real(kind=double), intent(out) :: dens      !!! output density
real(kind=double), dimension(3), intent(in) :: r      !!! input position
real(kind=double), dimension(3), intent(out) :: grad   !!! output gradient
real(kind=double), dimension(3,3), intent(out) :: hess !!! output hessian matrix
!
integer(kind=int32) :: l,m,n,ll,mm,nn
integer(kind=int32), dimension(3) :: is
real(kind=double) :: tmp
real(kind=double), dimension(3) :: ss
real(kind=double), dimension(4,3) :: a,ap,app   
!
!  initialization of data
!
dens=zero
grad(:)=zero
hess(:,:)=zero
do l=1,3
  tmp=r(l)
  is(l)=int(tmp)+1
  ss(l)=tmp+one-real(is(l),double)
end do
!
!  if new position is out of box, shift of i-1, i, i+1 and i+2
!
do l=1,3
  if(ss(l) < zero) then
    is(l)=is(l)-1
    ss(l)=ss(l)+one
  end if
  if(ss(l) >= one) then
    is(l)=is(l)+1
    ss(l)= ss(l)-one
  end if
end do
!
!  coefficient for the interpolation
!
do l=1,3
  tmp=ss(l)
  a(1,l)=((-tmp*over6+half)*tmp-over3)*tmp
  a(2,l)=((tmp*half-one)*tmp-half)*tmp+one
  a(3,l)=((-tmp*half+half)*tmp+one)*tmp
  a(4,l)=(tmp*tmp-one)*tmp*over6
!
  ap(1,l)=(-tmp*half+one)*tmp-over3
  ap(2,l)=(tmp*t3over2-two)*tmp-half
  ap(3,l)=(-tmp*t3over2+one)*tmp+one
  ap(4,l)=tmp*tmp*half-over6
!
  app(1,l)=-tmp+one
  app(2,l)=tmp*three-two
  app(3,l)=-tmp*three+one
  app(4,l)=tmp
end do
!
!  interpolation of density, gradient and hessian 
!
do n=1,4
  nn=(n-2)+is(3)
  nn=modulo(nn,np3)
  if(nn <= 0) nn=nn+np3
  do m=1,4
    mm=(m-2)+is(2)         
    mm=modulo(mm,np2)
    if(mm <= 0) mm=mm+np2
    do l=1,4
      ll=(l-2)+is(1)
      ll=modulo(ll,np1)
      if(ll <= 0) ll=ll+np1
!
      tmp=rho(ll,mm,nn)             
      dens=dens+tmp*a(l,1)*a(m,2)*a(n,3)
!
      grad(1)=grad(1)+tmp*ap(l,1)*a(m,2)*a(n,3)
      grad(2)=grad(2)+tmp*a(l,1)*ap(m,2)*a(n,3)
      grad(3)=grad(3)+tmp*a(l,1)*a(m,2)*ap(n,3)
!
      hess(1,1)=hess(1,1)+tmp*app(l,1)*a(m,2)*a(n,3)
      hess(1,2)=hess(1,2)+tmp*ap(l,1)*ap(m,2)*a(n,3)
      hess(1,3)=hess(1,3)+tmp*ap(l,1)*a(m,2)*ap(n,3)
      hess(2,2)=hess(2,2)+tmp*a(l,1)*app(m,2)*a(n,3)
      hess(2,3)=hess(2,3)+tmp*a(l,1)*ap(m,2)*ap(n,3)
      hess(3,3)=hess(3,3)+tmp*a(l,1)*a(m,2)*app(n,3)
    end do
  end do
end do
hess(2,1)=hess(1,2)
hess(3,1)=hess(1,3)
hess(3,2)=hess(2,3)
!
end subroutine interp_b

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine valid_atom(at_name,at_flag)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
character(len=*), intent(in) :: at_name
logical, intent(out) :: at_flag
!
integer(kind=int32) :: i
!
at_flag=.true.
do i=1,9
  if(index(at_name(1:2),trim(at_check_list(i))) /= 0) then
    at_flag=.false.
    return
  end if
end do
!
return
end subroutine valid_atom

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine bond_paths_graph()
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int16), parameter :: un=34,uf=35,ul=36
integer(kind=int32) :: i1,i2,i3,j1,j2   !!! N of the 3 closest atoms
integer(kind=int32) :: n_up,n_down      !!! number of points in half-bond paths
!integer(kind=int32) :: n_vat    !<<<<<<<<<<<<<<<<<<
integer(kind=int32) :: i,j,k,num          !!! counter for loop
integer(kind=int32) :: ih1,ih2          !!! flags to test H---H occuence in determining bond length.
!real(kind=double) :: d_o !<<<<<<<<<<<
real(kind=double) :: d_12           !!! distance to the closest atom
real(kind=double) :: la,lb,lc           !!! cell vectors length
real(kind=double), dimension(3) :: r_start,uo      !!! coordinates of the CP and direction of lambda_3
real(kind=double), dimension(3) :: a,b,c
real(kind=double), dimension(3) :: ta,tb,tc        !!! translation vectors 
real(kind=double), dimension(3) :: r_1,r_2,r_2t,r_12    !!! vectors 
real(kind=double), dimension(3,3) :: u_nghb        !!! atomic positions of the 3 neighbours
character(len=7) :: label_cp        !!! CP label
character(len=100) :: nfil_dx,nfil_dx2   
!
character(len=*), parameter :: fmt0="(a,3f10.5)"
character(len=*), parameter :: fmt1="(i3,a3,i3,2(tr5,3f12.6),tr5,f9.5)"
character(len=*), parameter :: fmt2="(a,es12.5,a,3f12.5)"
character(len=*), parameter :: fmt3="(a,3f12.5,i6,tr2,5es12.5)"
character(len=*), parameter :: fmt4="(7es13.5)"
!
extension=".path"
nfil=trim(file_path)//trim(filename_2)//trim(extension)
open(unit=21,file=nfil,status="new",action="write")
write(unit=21,fmt=fmta) trim(filename_2) 
!!!!! output box informations
a(:)=real(np1,double)*del1(:)
b(:)=real(np2,double)*del2(:)
c(:)=real(np3,double)*del3(:)
la=sqrt(dot_product(a,a))/real(np1,double)
lb=sqrt(dot_product(b,b))/real(np2,double)
lc=sqrt(dot_product(c,c))/real(np3,double)
write(unit=21,fmt="(a,3(i4,a2),a,3f8.3,a)") "___grid:  ",np1," x",np2," x",np3," x","  (",la,lb,lc,")"
write(unit=21,fmt=fmt0) "___grid_origin@ ",origin(:)
write(unit=21,fmt=fmta) "___grid_vertices:"
write(unit=21,fmt=fmt0) "   (1)  ",zero,zero,zero
write(unit=21,fmt=fmt0) "   (2)  ",a(:)
write(unit=21,fmt=fmt0) "   (3)  ",a(:)+b(:)
write(unit=21,fmt=fmt0) "   (4)  ",b(:)
write(unit=21,fmt=fmt0) "   (5)  ",b(:)+c(:)
write(unit=21,fmt=fmt0) "   (6)  ",c(:)
write(unit=21,fmt=fmt0) "   (7)  ",a(:)+c(:)
write(unit=21,fmt=fmt0) "   (8)  ",a(:)+b(:)+c(:)
!!!!! output atoms informations
!!      first check valid atoms according to "at-check_list" 
!!      defined in "cpgrity_initialize" subroutine
j=0
do i=1,nat
  checked_atom=lat(i)
! is current atom true atom or dummy, or Critical Point ? 
  call valid_atom(checked_atom,test_at)
  if(test_at) then
    j=j+1
    indx_valid(j)=i
  end if
end do
!n_vat=j       !<<<<<<<<<<<<<<<<<<<<<<<
!!
write(unit=21,fmt="(a,i5)") "___number_of_input_atoms:",nat !n_vat
write(unit=21,fmt=fmta) "___atomic_positions:"
!!
do j=1,nat           ! n_vat
  i=j                  ! indx_valid(j)
  write(unit=21,fmt="(i3,tr2,a6,tr1,3f12.6)") j,trim(lat(i)),rat(i,:)
end do
!!!!! output conventional atomic bonds
!
write(unit=21,fmt=fmta) "___bonds:"
do i1=1,nat                ! n_vat
  i2=i1                    ! indx_valid(i1)
  r_1=rat(i2,:)
  ih1= index(lat(i2),"H")
  do j1=i1+1,nat           ! n_vat
    j2=j1                  ! indx_valid(j1)
    r_2=rat(j2,:)
    ih2= index(lat(j2),"H")
    if(ih1 /= 1 .or. ih2 /= 1) then     !!! no H---H bonds !
      if(iso_test) then
        r_12=r_2-r_1
        d_12=sqrt(dot_product(r_12,r_12))                
        if(d_12 > zero .and. d_12 <= max_bond_length) then
          write(unit=21,fmt=fmt1) i2," ---",j2,r_1,r_2,d_12
        end if
      else
        do i=-1,1
          ta=real(i,double)*a(:)
          do j=-1,1
            tb=real(j,double)*b(:)
            do k=-1,1
              tc=real(k,double)*c(:)
              r_2t=r_2+ta+tb+tc
              r_12=r_2t-r_1
              d_12=sqrt(dot_product(r_12,r_12))                
              if(d_12 > zero .and. d_12 <= max_bond_length) then
!!!write(unit=*,fmt="(i3,a3,i3,a,tr3,f8.4)") i2," ---",j2, " <==> "//lat(i2)//"---"//lat(j2)//"  --> ",d_12
                write(unit=21,fmt=fmt1) i2," ---",j2,r_1,r_2t,d_12
              end if
            end do
          end do
        end do
      end if
    end if
  end do  !! loop over 2nd atom "j1"
end do    !! loop over 1st atom "i1"
write(unit=21,fmt=fmta) "___end_of_bonds"
!
do i=1,2
  if(i == 1) then
    rsi="SL"
    write(unit=*,fmt=fmta) ""
    write(unit=*,fmt=fmta) "Process low density (3,-1) CP bond paths"
    tcp="lowsaddle"
    nfil_dx=trim(file_path)//"/dx/saddle_low.dx"  ! dx file for low density saddle point
    nfil_dx2=trim(file_path)//"/dx/paths_low.dx"  !! dx file for low density path
  else
    rsi="SH"
    tcp="highsaddle"
    write(unit=*,fmt=fmta) ""
    write(unit=*,fmt=fmta) "Process high density (3,-1) CP bond paths"
    nfil_dx=trim(file_path)//"/dx/saddle_high.dx"
    nfil_dx2=trim(file_path)//"/dx/paths_high.dx"  !! dx file for high density path
  end if
  open(unit=34,file=nfil_dx,status="new",action="write")
  open(unit=35,file=nfil_dx2,status="new",action="write")
  num=0        ! counts number of low or high saddles
!
  do j=1,nc(i)                   !! i ==> type of CP   
    if(cp(i,j,18) == one) then
!
!
      call CP_label(j,label_cp,rsi)
      r_start=cp(i,j,1:3)- origin(:)      !!! CP position
      uo=cp(i,j,9:11)               !!! Lambda_3 direction
!!
      cp_dens=cp(i,j,4)
      cp_lap=sum(cp(i,j,6:7))
      grc=gfactor*cp(i,j,4)**t5over3+over6*cp_lap
      vrc=cp_lap/four-two*grc
      vrc=vrc*kjpmol
      cp_r=abs(cp(i,j,8)/cp(i,j,6))
      cp_e=(cp(i,j,8)/cp(i,j,7))-one
      write(unit=*,fmt=*)
      write(unit=*,fmt="(a,i4,2a)") " CP #",j," : ",trim(label_cp)
      write(unit=*,fmt="(a,3f12.5,tr2,5(tr2,es12.5))") "___"//trim(label_cp(2:len_trim(label_cp)-1))//"_critical_point@", &
      cp(i,j,1:3),cp_dens,cp_lap,vrc,cp_r,cp_e
      write(unit=*,fmt=fmt2) "Lambda_3 ",cp(i,j,6)," --> uo = ",cp(i,j,9:11)
      write(unit=*,fmt=fmt2) "Lambda_2 ",cp(i,j,7)," -->      ",cp(i,j,12:14)
      write(unit=*,fmt=fmt2) "Lambda_1 ",cp(i,j,8)," -->      ",cp(i,j,15:17)
!!
      i1=int(cp(i,j,19))
      i2=int(cp(i,j,20))
      i3=int(cp(i,j,21))
!     d_o=0.8*minval(cp(i,j,22:24))    !<<<<<<<<<<<
      u_nghb(1,:)=ratoms(i1,:)-origin(:) 
      u_nghb(2,:)=ratoms(i2,:)-origin(:) 
      u_nghb(3,:)=ratoms(i3,:)-origin(:) 
      write(unit=*,fmt=fmta,advance="no") "."
      call half_bond_path(u_nghb,r_start,uo,n_down,"-")
      call half_bond_path(u_nghb,r_start,uo,n_up,"+")
      num=num+1
      cp_pos=cp(i,j,1:3)
      cp_lambda=cp(i,j,6:8)
      write(unit=21,fmt=fmt3) "___critical_point@"//trim(label_cp(2:len_trim(label_cp)-1)),cp(i,j,1:3),n_down+n_up,cp_dens,cp_lap, &
      vrc,cp_r,cp_e
      write(unit=*,fmt=fmt3) "___critical_point@"//trim(label_cp(2:len_trim(label_cp)-1)),cp(i,j,1:3),n_down+n_up,cp_dens,cp_lap, &
      vrc,cp_r,cp_e
      call print_cp_for_dx(34,num)!!!!print saddles file for dx
!!!
      call Cp_path_dx(35,k,1,num,n_down,n_up)
      do k=n_down,1,-1
        write(unit=21,fmt=fmt4) bd_path_down(k,:)
        call Cp_path_dx(35,k,2,num,n_down,n_up)
      end do
      do k=1,n_up
        call Cp_path_dx(35,k,21,num,n_down,n_up)
        write(unit=21,fmt=fmt4) bd_path_up(k,:)
      end do
      call Cp_path_dx(35,k,3,num,n_down,n_up)
    end if
  end do
  call Cp_path_dx(35,k,4,num,n_down,n_up)
  close(35)
  call cp_groups_for_dx(34,num)
  close(34)
end do
!
do i=3,5
  num=0  
  select case(i)
  case(3)
    write(unit=21,fmt=fmta) "___ring_critical_points:"
    rsi="RG"
    tcp="ring"        
    nfil_dx=trim(file_path)//"/dx/rings.dx"
  case(4)
    write(unit=21,fmt=fmta) "___cage_critical_points:"
    rsi="CG"
    tcp="cage"
    nfil_dx=trim(file_path)//"/dx/cages.dx"
  case(5)
    write(unit=21,fmt=fmta) "___maxima_critical_points:"
    rsi="AT"
    tcp="attractor"
    nfil_dx=trim(file_path)//"/dx/attractors.dx"
  end select
  open(unit=34,file=trim(nfil_dx),status="new",action="write")
  do j=1,nc(i)
    if(cp(i,j,18) == one) then
      num=num+1
      cp_pos=cp(i,j,1:3)
      cp_dens=cp(i,j,4)
      cp_lambda=cp(i,j,6:8) ! eMail 5.12.2006
      cp_lap=sum(cp_lambda) ! eMail 5.12.2006
      grc=gfactor*cp(i,j,4)**t5over3+over6*cp_lap
      vrc=cp_lap/four-two*grc
      vrc=vrc*kjpmol
      cp_r=abs(cp(i,j,8)/cp(i,j,6))
      cp_e=(cp(i,j,8)/cp(i,j,7))-one
      call print_cp_for_dx(34,num) !!!print cp datas in the required file for OpenDx
      call CP_label(j,label_cp,rsi)         
      write(unit=21,fmt="(a,3f12.5,2(tr2,es12.5))") trim(label_cp(2:len_trim(label_cp)-1)),cp(i,j,1:3),cp_dens,cp_lap,cp_r,cp_e
    end if 
  end do
  call cp_groups_for_dx(34,num) !!!!!print the end of Dx file (last Object " group") 
  close(34)
end do
!
close(21)
end subroutine bond_paths_graph

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine half_bond_path(r_nb,r_o,u_o,npt,flag)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(out) :: npt             !!! number of stored points
real(kind=double), dimension(3), intent(in) :: r_o,u_o         
real(kind=double), dimension(3,3), intent(in) :: r_nb  !!! coordinates of the closest atoms
character(len=1), intent(in) :: flag  !!! "+" or "-" along lambda3 curvature
!
integer(kind=int32) :: i,k
real(kind=double) :: mod_grad,dr_mid,rho_cur,d_tot
real(kind=double) :: lam1,lam2,lam3,lap,d_part
real(kind=double),dimension(3) :: r_cur,r_grid,del_r,grad_cur,grad_grid,grad_dir
real(kind=double),dimension(3) :: r_mid,d_at,u_at
real(kind=double),dimension(3) :: lambda
real(kind=double),dimension(3,3) :: hess_cur,hess_grid,mat_3x3,eigen_vec
!
dr_mid=half*dr_cur
!
del_r=dr_cur*u_o
if(index(flag,"-") /= 0) del_r=-del_r
r_cur=r_o+c_o*del_r
write(unit=*,fmt="(2a,2(tr2,3f10.5),tr5,f10.5)") " path ",flag,r_o,del_r,sqrt(dot_product(c_o*del_r,c_o*del_r))
!
npt=0
d_tot=zero
d_part=zero
k=0
do
  if(d_tot > d_max .or. npt > n_max) then
    exit
  else
! compute gradient and density at starting point
    r_cur(:)=r_cur(:)+del_r(:)
    k=k+1
    do i=1,3
      u_at=r_nb(i,:)-r_cur(:)
      d_at(i)=sqrt(dot_product(u_at,u_at))
    end do
    if(minval(d_at(:)) < do_at) exit
    r_grid=matmul(c2g,r_cur)
    call interp_b(r_grid,rho_cur,grad_grid,hess_grid)
    grad_cur=matmul(c2gt,grad_grid)
    mod_grad=sqrt(dot_product(grad_cur,grad_cur))
! print out density, gradient, ellipticity, laplacian
! (d_cur/d_write) points retained
!!!
!!!  ICI pour test sur sortie de boucle half_bond_path
!!!
    if(d_part > d_write) then
      d_part=zero
      mat_3x3=matmul(c2gt,hess_grid) 
      hess_cur=matmul(mat_3x3,c2g)
      call eigen(hess_cur,lambda,eigen_vec)
      lam3=lambda(1)
      lam2=lambda(2)
      lam1=lambda(3)
      lap=lam1+lam2+lam3
      ellip=lam1/lam2-one
!!! save data
      npt=npt+1
      if(index(flag,"-") /= 0) then
        bd_path_down(npt,1:3)=r_cur(:)+origin(:)
        bd_path_down(npt,4)=rho_cur
        bd_path_down(npt,5)=mod_grad
        bd_path_down(npt,6)=ellip
        bd_path_down(npt,7)=lap
      else
        bd_path_up(npt,1:3)=r_cur(:)+origin(:)
        bd_path_up(npt,4)=rho_cur
        bd_path_up(npt,5)=mod_grad
        bd_path_up(npt,6)=ellip
        bd_path_up(npt,7)=lap
      end if
    end if
!               
! use gradient at starting point to extrapolate mid-point
! and compute gradient at mid-point             
    grad_dir(:)=grad_cur(:)/mod_grad
    r_mid(:)=r_cur(:)+dr_mid*grad_dir(:)
    r_grid=matmul(c2g,r_mid)
    call interp_b(r_grid,rho_cur,grad_grid,hess_grid)
    grad_cur=matmul(c2gt,grad_grid)
    mod_grad=sqrt(dot_product(grad_cur,grad_cur))
    grad_dir(:)=grad_cur(:)/mod_grad
! extrapolate next point from gradient at mid-point
    del_r(:)=dr_cur*grad_dir(:)
    d_tot=d_tot+dr_cur
    d_part=d_part+dr_cur
  end if
end do
!
end subroutine half_bond_path

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine print_cp_for_dx(lu,k)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: k,lu
!
!real(kind=single) :: ratio_ !<<<<<<<<<<<<<<<<<<<<<<
character(len=7) :: cp_name
!
character(len=*), parameter :: fmt1="(tr3,a,es12.5,a)"
character(len=*), parameter :: fmt2="(tr3,3f9.4)"
character(len=*), parameter :: fmt3="(tr3,es12.5)"
!
!ratio_=abs(cp_lambda(1)/cp_lambda(3))   !<<<<<<<<<<<<<<<<<<<<<
!
call cp_label_dx(k,cp_name,rsi)
write(unit=lu,fmt="(3a)") "Object ""xyz_",trim(cp_name),""" class array type float rank 1 shape 3 items 1 data follows" 
write(unit=lu,fmt=fmt2) cp_pos! CP position
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""dens_"//trim(cp_name)//""" class array type double rank 0 items 1 data follows"
write(unit=lu,fmt=*) cp_dens! CP density
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""lapl_"//trim(cp_name)//""" class array type float rank 0 items 1 data follows"
write(unit=lu,fmt="(es11.4)") cp_lap! CP laplacian
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""ratio_"//trim(cp_name)//""" class array type double rank 0 items 1 data follows"
write(unit=lu,fmt=*) cp_r! CP ratio
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""ellip_"//trim(cp_name)//""" class array type double rank 0 items 1 data follows"
write(unit=lu,fmt=*) cp_e! CP ratio
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""kinetic_"//trim(cp_name)//""" class array type float rank 0 items 1 data follows"
write(unit=lu,fmt=fmt3) grc! CP kinetic energy
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 	
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""potential_"//trim(cp_name)//""" class array type float rank 0  items 1 data follows"
write(unit=lu,fmt="(tr3,es12.5)") vrc! CP potential
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
write(unit=lu,fmt=*)    	    	        
write(unit=lu,fmt=fmta) "Object ""type_"//trim(cp_name)//""" class array type string rank 1 shape 10 items 1 data follows"
write(unit=lu,fmt=gmt0) """"//trim(tcp)//""""! for color
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
!
write(unit=lu,fmt=fmta) "Object ""label_"//trim(cp_name)//""" class array type string rank 1 shape 10 items 1 data follows"
write(unit=lu,fmt=gmt0) """"//trim(cp_name)//""""	! label
write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions"""  
write(unit=lu,fmt=*)   
write(unit=lu,fmt=fmta) "Object ""field_"//trim(cp_name)//""" class field"
write(unit=lu,fmt=gmt0) "component ""positions""  ""xyz_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""data_string"//"""  """//"label_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""data_type""     ""type_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""density""     ""dens_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""laplacian""     ""lapl_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""kinetic""     ""kinetic_"//trim(cp_name)//""""
write(unit=lu,fmt=gmt0) "component ""potential""     ""potential_"//trim(cp_name)//"""" 
write(unit=lu,fmt=gmt0) "component ""ratio""     ""ratio_"//trim(cp_name)//""""    
write(unit=lu,fmt=gmt0) "component ""ellipticity""     ""ellip_"//trim(cp_name)//""""   	   	    	
write(unit=lu,fmt=fmt1) "attribute ""density"" string ""density:",cp_dens,""""!info about atom
write(unit=lu,fmt="(tr3,a,3es14.5,a)") "attribute ""lambda"" string ""lambda:",cp_lambda,""""
write(unit=lu,fmt=fmt1) "attribute ""laplacian"" string ""laplacian:",cp_lap,""""
write(unit=lu,fmt=fmt1) "attribute ""kinetic"" string ""kinetic energy:",grc,""""
write(unit=lu,fmt=fmt1) "attribute ""potential"" string ""potential:",vrc,""""
write(unit=lu,fmt=*) "#----------------------------------------------"
!
end subroutine print_cp_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    	
subroutine cp_groups_for_dx(lu,k)	
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
!
integer(kind=int32), intent(in) :: lu,k	
!
integer(kind=int32) :: icp
character(len=7) :: cp_name
!
write(unit=lu,fmt=fmta) "Object """//trim(rsi)//"_group"" class group"
do icp=1,k
  call cp_label_dx(icp,cp_name,rsi)
  write(unit=lu,fmt=gmt0) "member """//trim(cp_name)//"""  value ""field_"//trim(cp_name)//""""
  end do
write(unit=lu,fmt=*)
!
end subroutine cp_groups_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine CP_label_dx(nn,cc,rs)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
!
integer(kind=int32), intent(in) :: nn
character(len=2), intent(in) :: rs
character(len=7), intent(out) :: cc
!
integer(kind=int32) :: nm,nc,nd,nu,ns,ndm
!
ns=48  !!! shift to transform integer 0 to 9 to ASCII "0" to "9"
if(nn < 10) then
  cc=rs//char(ns+nn)
else if(nn < 100) then
  nd=nn/10
  nu=nn-10*nd 
  cc=rs//char(ns+nd)//char(ns+nu)
else if(nn <= 1000) then
  nc=nn/100
  nd=(nn-100*nc)/10
  nu=nn-100*nc-10*nd
  cc=rs//char(ns+nc)//char(ns+nd)//char(ns+nu)
else if(nn <= 10000) then
  nm=nn/1000
  nc=(nn-1000*nm)/100
  nd=(nn-1000*nm-100*nc)/10
  nu=nn-1000*nm-100*nc-10*nd
  cc=rs//char(ns+nm)//char(ns+nc)//char(ns+nd)//char(ns+nu)
else if(nn <= 100000) then
  ndm=nn/10000
  nm=(nn-10000*ndm)
  nc=(nn-10000*ndm-1000*nm)/100
  nd=(nn-10000*ndm-1000*nm-100*nc)/10
  nu=nn-10000*ndm-1000*nm-100*nc-10*nd
  cc=rs//char(ns+ndm)//char(ns+nm)//char(ns+nc)//char(ns+nd)//char(ns+nu)
else
  write(unit=13,fmt=fmta) "WARNING    number of CP  exceeding ncmax !"
  stop
end if
!
end subroutine CP_label_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine Cp_path_dx(lu,n,nboucle,j,nbas,nhaut)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu,n,j 
integer(kind=int32), intent(in) :: nbas,nhaut
integer, intent(in) :: nboucle
!
integer(kind=int32) :: i
character(len=7) :: cp_name  !!!name of saddle
!
call cp_label_dx(j,cp_name,rsi)
!
select case(nboucle)
case(1)
  write(unit=lu,fmt=*) "Object ""path_xyz_"//trim(cp_name)//""" class array type float rank 1 shape 3 items",nbas+nhaut
  write(unit=lu,fmt=*) "data follows"
case(2)
  cp_pos=bd_path_down(n,1:3)
  write(unit=lu,fmt="(3f9.4)") cp_pos
case(21)
  cp_pos=bd_path_up(n,1:3)
  write(unit=lu,fmt="(3f9.4)") cp_pos
case(3)
  write(unit=lu,fmt=*) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""dens_"//trim(cp_name)//""" class constantarray type float rank 0 items",nbas+nhaut 
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt=*) cp_dens! CP density
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""lapl_"//trim(cp_name)//""" class constantarray type float rank 0 items",nbas+nhaut 
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt="(es12.5)") cp_lap! CP laplacian
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""ratio_"//trim(cp_name)//""" class constantarray type float rank 0 items",nbas+nhaut 
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt=*) cp_r! CP ratio
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""ellip_"//trim(cp_name)//""" class constantarray type float rank 0 items",nbas+nhaut 
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt=*) cp_e! CP ratio
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""kinetic_"//trim(cp_name)//""" class constantarray type float rank 0 items",nbas+nhaut
  write(unit=lu,fmt=*) "data follow"
  write(unit=lu,fmt="(tr3,es12.5)") grc! CP kinetic energy
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 	
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""potential_"//trim(cp_name)//""" class constantarray type float rank 0  items",nbas+nhaut 
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt="(tr3,es12.5)") vrc! CP potential
  write(unit=lu,fmt=gmt0) "attribute ""dep"" string ""positions""" 
  write(unit=lu,fmt=*)
  write(unit=lu,fmt="(a,i4)") "Object ""color_path"//trim(cp_name)//""" class constantarray type float rank 1 shape 3 items", &
  nbas+nhaut
  write(unit=lu,fmt=*) "data follows"
  write(unit=lu,fmt=*) 1,0,half
  write(unit=lu,fmt=*) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*) "Object ""path_field"//trim(cp_name)//""" class field"
  write(unit=lu,fmt=gmt0) "component ""positions"" ""path_xyz_"//trim(cp_name)//""""
  write(unit=lu,fmt=gmt0) "component ""colors"" ""color_path"//trim(cp_name)//""""
  write(unit=lu,fmt=gmt0) "component ""density""     ""dens_"//trim(cp_name)//""""
  write(unit=lu,fmt=gmt0) "component ""laplacian""     ""lapl_"//trim(cp_name)//""""
  write(unit=lu,fmt=gmt0) "component ""kinetic""     ""kinetic_"//trim(cp_name)//""""
  write(unit=lu,fmt=gmt0) "component ""potential""     ""potential_"//trim(cp_name)//"""" 
  write(unit=lu,fmt=gmt0) "component ""ratio""     ""ratio_"//trim(cp_name)//""""    
  write(unit=lu,fmt=gmt0) "component ""ellipticity""     ""ellip_"//trim(cp_name)//""""   	   	    	
  write(unit=lu,fmt=*) "#----------------------------------------------------------------"
  write(unit=lu,fmt=*)
case(4)
  write(unit=lu,fmt=*) "Object ""path_group"" class group"
  do i=1,j
    call cp_label_dx(i,cp_name,rsi)
    write(unit=lu,fmt=*) "member ""path_"//trim(cp_name)//"""  ""path_field"//trim(cp_name)//""""
  end do
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=*)
end select
end subroutine Cp_path_dx

end module cpgrity_dx_mod
