module cpgrity_mod

use bondpath_parameters         
use constants_module
use cp_module
use filename_module
use grid_module
use kind_numbers
use latt_module
use math_module
use old_module
use rho_module
use strc_module
use test_module

implicit none
private

public :: cpgrity_initialize,grid_initialize,interp,ranksig,rho_initialize,strc_initialize,testcp,writeoutput

private :: add_lattice,get_atoms,get_one_par,indexx

intrinsic :: acos,aint,dot_product,index,len,len_trim,log,matmul,maxval,minval,modulo,nint,real,sum,transpose,trim

real(kind=double), parameter, private :: over64=one/64.0_double
real(kind=double), parameter, private :: a0=0.5291772   !!! a.u. -> angstrom
! constant used for estimating density in e-/A^3 ...
real(kind=double), parameter, private :: a03=one/a0**3  !!! for density in e-/A^3
real(kind=double), parameter, private :: a04=a03/a0
real(kind=double), parameter, private :: a05=a03/a0**2  !!! for gradient in e-/A^5

integer(kind=int32), public :: nmaxgstep=500       !!! max # of allowed steps following gradient path without reaching CP
integer(kind=int32), public :: maxat=52            !!! nat*maxat= max number of atoms
real(kind=double), public :: invfric=3.0e-1_double   !!! friction^-1 for min. of gradient
real(kind=double), public :: dgradmin=1.0e-9_double  !!! if dgrad lt, assumed to 0
real(kind=double), public :: rioff=1.5_double        !!! stops research if ri-rifirst gt
real(kind=double), public :: densmaxcp=half          !!! if dens ge, cp considered as saddle "high" density
real(kind=double), public :: densoffcp=1.0e-5_double !!! if dens lt, cp not considered
real(kind=double), private :: lam_0=1.0e-6_double    !!! if lambda < Lam_0 then =0
real(kind=double), public :: xmingrad     !!! minimum of gradient on grid points

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine get_one_par(i_unt,kwrd,c_io,i_io,x_io,cix_flag)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! enables initialization of parameters character, integer or real
! reading such "key_word = value !" expressions in "InteGriTy.in" file
!
integer(kind=int32), intent(in) :: i_unt
integer(kind=int32), intent(inout) :: i_io
real(kind=double), intent(inout) :: x_io
character(len=*), intent(inout) :: c_io
character(len=*), intent(in) :: cix_flag
character(len=*), intent(in) :: kwrd
!
integer(kind=int32) :: eol,jkw,i1,i2,io_err,i_line
!real(kind=double) :: xx
character(len=256) :: o_line
!
!print *,"get_one_par: i_unt,cix_flag,kwrd:",i_unt,cix_flag," ",kwrd
rewind(unit=i_unt)
i_line=0
do
  i_line=i_line+1
  read(unit=i_unt,fmt=fmta,iostat=io_err) o_line
  if(io_err == 0) then
    if(index(o_line,"EOB!") /= 0 .or. index(o_line,"END!") /= 0) exit
    if(index(o_line(1:2),"!!") /= 0) cycle
    jkw=index(o_line,trim(kwrd))
    if(jkw /= 0) then
      eol=len_trim(o_line)
      i1=index(o_line(jkw:eol),"=")+jkw
      if(i1 == jkw) then
        cycle        ! empty line
      else
        i2=index(o_line(jkw:eol),"!")
        if(i2 == 0) then
          i2=eol
        else
          i2=i2+jkw-2
        end if
!print *,"get_one_par: o_line(i1:i2):",o_line(i1:i2)
        select case(cix_flag)
        case("c")
          c_io=o_line(i1:i2)
        case("i")
          read(unit=o_line(i1:i2),fmt=*) i_io
        case("x")
          read(unit=o_line(i1:i2),fmt=*) x_io
        case default
          write(unit=6,fmt=*) "ERROR in get_one_par: wrong cix_flag => STOP"
          stop
        end select
!        if(verify("c",cix_flag) == 0) then
!          c_io=""     
!          read(unit=o_line(i1:i2),fmt=fmta) c_io
!        else
!          if(verify("i",cix_flag) == 0) then
!!           i_io=nint(xx)
!            read(unit=o_line(i1:i2),fmt=*) i_io
!          end if
!          if(verify("x",cix_flag) == 0) then
!!           x_io=xx
!            read(unit=o_line(i1:i2),fmt=*) x_io
!          end if
!        end if
        exit
      end if
    else
      c_io="no_occ"     
      cycle
    end if
  else
    c_io="error"
    write(unit=*,fmt="(a,i3,/,tr5,a)") " error reading o_line at line ",i_line,trim(o_line)
    exit
  end if
end do
!print *,"get_one_par: i_unt,cix_flag,kwrd:",i_unt,cix_flag,kwrd
!print *,"get_one_par: i_io,x_io,c_io:",i_io,x_io,c_io
return
!
end subroutine get_one_par

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine cpgrity_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   Initialisation of program parameters from "cpgrity.in" file
!
integer(kind=int32) :: io_error,i_get,ilb,ifb
real(kind=double) :: x_get
character(len=256) :: c_get
character(len=20) :: key_wrd
character(len=1) :: par_type
!
! constant used for estimating Kinetic and potential energies at CP
!
gfactor=0.3_double*(three*pi**2)**(two*over3)
mrho=one           !!! multiply by -1 for finding attractors if it can help 
!
!    open cpgrity file to re-initialize parameters
!
open(unit=10,file="cpgrity.in",iostat=io_error,status="old",action="read",position="rewind")
!
!!!  check if "cpgrity.in" file exists or can be opened
!
if(io_error /= 0) then
  close(unit=10)
  write(unit=*,fmt=fmta) " file cpgrity.in not found or unreadable - program stops"
  open(unit=10,file="cpgrity.error",status="new",action="write")
  write(unit=10,fmt=*) " file cpgrity.in not found or unreadable - program stops"
  close(unit=10)
  stop
else
!!!  check file extension for total and valence densities
!
  key_wrd="path"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  file_path=trim(c_get)
  if(index(file_path,"no_occ") /= 0) file_path=""
  key_wrd="file"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"no_occ") /= 0 .or. len_trim(c_get) <= 0) then
    close(unit=10)
    write(unit=*,fmt=fmta) " data and atoms files not found  - program stops"
    open(unit=10,file="cpgrity.error",status="new",action="write")
    write(unit=10) " data and atoms files not found - program stops"
    close(unit=10)
    stop
  else
    ifb=index(trim(c_get)," ")
    ilb=len_trim(c_get)
    if(ilb == 0 .or. ilb > len(c_get)) then
      close(unit=10)
      open(unit=10,file="cpgrity.error",status="new",action="write")
      write(unit=10,fmt=*) " data filename too long - program stops"
      write(unit=*,fmt=*) " data filename too long - program stops"
      close(unit=10)
      stop
    end if
    filename_1=c_get(ifb+1:ilb)
  end if
!!
  key_wrd="ext"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"no_occ") /= 0) then
    extension="-tot.data"
  else 
    if(len_trim(c_get) > 0) then
      extension_data=trim(c_get)
    else
      close(unit=10)
      open(unit=10,file="integrity.error",status="new",action="write")
      write(unit=10) " data file extension badly defined - program stops"
      write(unit=*,fmt=*) " data file extension badly defined - program stops"
      close(unit=10)
      stop
    end if
  end if
!
  key_wrd="cp_type"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"attractor") /= 0) then
    cp_attr=.true.
  else
    cp_attr=.false.
  end if
!!!
  key_wrd="cp_unit"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"ang") /= 0) then
    cp_unit=.true.
  else
    cp_unit=.false.
  end if
!!!
  key_wrd="epsoff"              !!! for subroutine eigen
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,epsoff,par_type)
  key_wrd="invfric"       !!! friction^-1 for min. of gradient
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,invfric,par_type)
  key_wrd="dgradmin"   !!! if dgrad lt, assumed to 0
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,dgradmin,par_type)
  key_wrd="rioff"    !!! stops research if ri-rifirst gt
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,rioff,par_type)
  key_wrd="densmaxcp"   !!! if dens ge,  cp considered
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,densmaxcp,par_type)
  key_wrd="densoffcp"    !!! if dens lt, cp not considered
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,densoffcp,par_type)
  key_wrd="lam_0"     !!! if lambda < Lam_0 then =0
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,lam_0,par_type)
  key_wrd="mrho"
  par_type="x"
  call get_one_par(10,key_wrd,c_get,i_get,mrho,par_type) !!! multiply rho by -1 for attractor finding
!!!
  key_wrd="nmaxgstep"        !!! maximum number of allowed steps following gradients without reaching CP
  par_type="i"
  call get_one_par(10,key_wrd,c_get,nmaxgstep,x_get,par_type)
  key_wrd="ncmax"        !!! maximum number of CP
  par_type="i"
  call get_one_par(10,key_wrd,c_get,ncmax,x_get,par_type)
  key_wrd="maxat"    !!! nat*maxat= max number of atoms, including periodic images
  par_type="i"
  call get_one_par(10,key_wrd,c_get,maxat,x_get,par_type)
!
!!!  check grid type  : periodic or isolated system 
!
  key_wrd="grid"
  par_type="c"
  call get_one_par(10,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"iso") /= 0) then
    iso_test=.true.
  else
    iso_test=.false.
  end if
!
  key_wrd="max_bond_length"
  call get_one_par(10,key_wrd,c_get,i_get,max_bond_length,"x")
  key_wrd="bond_path_increment"
  call get_one_par(10,key_wrd,c_get,i_get,dr_cur,"x")
  key_wrd="bond_path_dmax"
  call get_one_par(10,key_wrd,c_get,i_get,d_max,"x")
  key_wrd="bond_path_dwrite"
  call get_one_par(10,key_wrd,c_get,i_get,d_write,"x")
  key_wrd="bond_path_d2at"
  call get_one_par(10,key_wrd,c_get,i_get,do_at,"x")
  key_wrd="bond_path_xdo"
  call get_one_par(10,key_wrd,c_get,i_get,c_o,"x")
  key_wrd="bond_path_nmax"
  call get_one_par(10,key_wrd,c_get,n_max,x_get,"i")
!
  close(unit=10)
end if
!
end subroutine cpgrity_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine grid_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Search for all informations concerning the grid and determines
! transformations matrices (example: g2c= from grid to cartesian 
!
real(kind=double) :: d11,d12,d13,d21,d22,d23,d31,d32,d33,detg2c
!
!       version using unformatted binary file !!!!
read(unit=10) np1,np2,np3
read(unit=10) origin
read(unit=10) del1
read(unit=10) del2
read(unit=10) del3
!
write(unit=*,fmt=fmta) "Informations concerning grid"
write(unit=*,fmt="(a,3i4)") "number of points in each direction ",np1,np2,np3
write(unit=*,fmt="(3f12.5)") origin
write(unit=*,fmt="(3f12.5)") del1
write(unit=*,fmt="(3f12.5)") del2
write(unit=*,fmt="(3f12.5)") del3
!
inc1=sqrt(dot_product(del1,del1))
inc2=sqrt(dot_product(del2,del2))
inc3=sqrt(dot_product(del3,del3))
!
alpha=acos(dot_product(del2,del3)/(inc2*inc3))
beta =acos(dot_product(del1,del3)/(inc1*inc3))
gamma=acos(dot_product(del1,del2)/(inc1*inc2))
!
!  transformation matrix from grid to cartesian 
!
g2c(1,1)=del1(1)
g2c(2,1)=del1(2)
g2c(3,1)=del1(3)
g2c(1,2)=del2(1)
g2c(2,2)=del2(2)
g2c(3,2)=del2(3)
g2c(1,3)=del3(1)
g2c(2,3)=del3(2)
g2c(3,3)=del3(3)
!
!  transformation matrix from cartesian to grid :c2g
!  c2gt=Transpose(c2g)=Transpose(Inv(g2c))
!
d11= del2(2)*del3(3)-del2(3)*del3(2)
d12=-del1(2)*del3(3)+del1(3)*del3(2)
d13= del1(2)*del2(3)-del1(3)*del2(2)
d21=-del2(1)*del3(3)+del2(3)*del3(1)
d22= del1(1)*del3(3)-del1(3)*del3(1)
d23=-del1(1)*del2(3)+del1(3)*del2(1)
d31= del2(1)*del3(2)-del2(2)*del3(1)
d32=-del1(1)*del3(2)+del1(2)*del3(1)
d33= del1(1)*del2(2)-del1(2)*del2(1)
!
detg2c=del1(1)*d11+del2(1)*d12+del3(1)*d13
!
c2gt(1,1)= d11/detg2c
c2gt(2,1)= d21/detg2c
c2gt(3,1)= d31/detg2c
c2gt(1,2)= d12/detg2c
c2gt(2,2)= d22/detg2c
c2gt(3,2)= d32/detg2c
c2gt(1,3)= d13/detg2c
c2gt(2,3)= d23/detg2c
c2gt(3,3)= d33/detg2c
!
c2g=transpose(c2gt)
!
!   transformation matrix cartesian <-> fractional
!
npx(1:3,1:3)=zero
npx(1,1)=np1
npx(2,2)=np2
npx(3,3)=np3
f2c=matmul(g2c,npx)
call matinv(f2c,c2f)
!
!!!     write c2f matrix to file used in matlab visualisation programs
!!!     "old version" maybe not longer necessary  ????
!!
!!open(unit=20,file=trim(file_path)//"c2f.mtx",status="new",action="write")
!!write(unit=20,fmt="(3f15.10)") c2f(1,:)
!!write(unit=20,fmt="(3f15.10)") c2f(2,:)
!!write(unit=20,fmt="(3f15.10)") c2f(3,:)
!!close(unit=20)
!
end subroutine grid_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine strc_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Search for atomic coordinates and determines periodic images included
! in a sphere centered at the center of the initial box and of 
! radius = maxdis
! Makes also data explorer files for box and periodic atoms
!
integer(kind=int32) :: io,i,j,k,ikw
real(kind=double), dimension(3) :: a,b,c,abc
!real(kind=double), dimension(3) :: abmc,ambc,ambmc  !<<<<<<<<<<<<<<
!real(kind=double), dimension(3) :: ab,ac,bc,mab,mac,mbc  !<<<<<<<<<<<<<
!real(kind=double), dimension(4) :: modabc  !<<<<<<<<<<<<<
character(len=1) :: ta,tb,tc
character(len=6) :: ltempo,tlab
character(len=128) :: title     !!! name for filenames
character(len=256) :: linecar
!
! Determines cell parameters and corresponding translations
!
a=del1*np1
b=del2*np2
c=del3*np3
abc=a+b+c
!abmc=a+b-c  !<<<<<<<<<<<<<
!ambc=a-b+c  !<<<<<<<<<<<<<
!ambmc=a-b-c  !<<<<<<<<<<<<<
!ab=a+b  !<<<<<<<<<<<<<
!ac=a+c  !<<<<<<<<<<<<<
!bc=b+c  !<<<<<<<<<<<<<
!mab=-a+b  !<<<<<<<<<<<<<
!mac=-a+c  !<<<<<<<<<<<<<
!mbc=-b+c  !<<<<<<<<<<<<<
center=origin+abc*half
!
! Max distance from center for which periodic image of atom is considered
!
!modabc(1)=sqrt(dot_product(abc,abc))
!modabc(2)=sqrt(dot_product(abmc,abmc))
!modabc(3)=sqrt(dot_product(ambc,ambc))
!modabc(4)=sqrt(dot_product(ambmc,ambmc))
!
! First read of structure file to determine number of atoms in initial box
!
rewind(unit=11)
nat=0
read(unit=11,fmt=*) title
read(unit=11,fmt=*) lunit
io=0
do
  if(io /= 0) exit
  read(unit=11,fmt=fmta) linecar
  io=index(linecar,"!EOB")
  if(index(linecar,"ATOM") /= 0) nat=nat+1
end do
!
ntable=nat*maxat
!print *,"strc_initialize: nat,maxat,ntable:",nat,maxat,ntable
allocate(lat(nat))
allocate(latoms(ntable))
allocate(rat(nat,3))
allocate(ratoms(ntable,3))
!
! Read atomic label and coordinates
!
call get_atoms()
do ikw=1,nat
  rat(ikw,:)=rat(ikw,:)*lunit
end do
!
! Determines periodic images of atoms (+/- a,b,c ...)
!
natoms=0
do io=1,nat
  ltempo=lat(io)
  ltempo=ltempo(1:2)
  if(index(ltempo,"CP") == 0 .and. index(ltempo,"DO") == 0 .and. index(ltempo,"DU") == 0) then
    if(iso_test) then
      natoms=natoms+1
      ratoms(natoms,:)=rat(io,:)
      latoms(natoms)=trim(lat(io))
    else
      do k=-1,1       
        if(k == -1) tc="-"
        if(k == 0) tc="."
        if(k == 1) tc="+"
        do j=-1,1
          if(j == -1) tb="-"
          if(j == 0) tb="."
          if(j == 1) tb="+"
          do i=-1,1
            if(i == -1) ta="-"
            if(i == 0) ta="."
            if(i == 1) ta="+"
            tlab=ta//tb//tc
            natoms=natoms+1
            ratoms(natoms,:)=rat(io,:)+real(i,double)*a(:)+real(j,double)*b(:)+real(k,double)*c(:)
            latoms(natoms)=trim(lat(io))//" "//trim(tlab)
          end do
        end do
      end do
    end if  
  end if   
end do
!
close(unit=11)
!
end subroutine strc_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine add_lattice(i,vector,label)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
! Determines periodic images in each direction and check if
! the atom is at a distance less then maxdis from center
!
integer(kind=int32), intent(in) :: i
real(kind=double), dimension(3), intent(in) :: vector
character(len=7), intent(in) :: label
!
real(kind=double) :: dis
real(kind=double), dimension(3) :: rtempo
!
rtempo=rat(i,:)-center+vector
dis=sqrt(dot_product(rtempo,rtempo))
if(dis < maxdis) then
  natoms=natoms+1
  if(natoms > ntable) then
    write(unit=13,fmt=fmta) "WARNING   maximum number of atoms exeeded"
    stop
  end if
  ratoms(natoms,:)=rat(i,:)+vector
  latoms(natoms)=lat(i)//label 
end if
!
end subroutine add_lattice

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine rho_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! READS the density on the grid
!
integer(kind=int32) :: i,j,k
!
read(unit=10)(((rho(i,j,k),k=1,np3),j=1,np2),i=1,np1)
!
end subroutine rho_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine interp(r,dens,grad,hess)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! interpolates rho given on a regular grid defined in grid_module
! a(4,3),ap(4,3),app(4,3), are coefficient for the interpolation
! s(3) and ss(3) are fractions of the incrementation in the interval ] 0,1[
! the interpolation is based on a tricubic Lagrange method;
! if x C [i,i+1] then the terms included are
! i-1,i,i+1,i+2 corresponding to 1,2,3,4 in the table a
!
!!   Oct. 1999  
!
real(kind=double), intent(out) :: dens       !!! output density
real(kind=double), dimension(3), intent(in) :: r        !!! input position
real(kind=double), dimension(3), intent(out) :: grad    !!! output gradient
real(kind=double), dimension(3,3), intent(out) :: hess  !!! output hessian matrix
!
integer(kind=int32) :: l,m,n,ll,mm,nn
integer(kind=int32), dimension(3) :: is
real(kind=double) :: dens0
real(kind=double), dimension(3) :: ss
real(kind=double), dimension(4,3) :: a,ap,app   
!
!  initialization of data
!
dens=zero
grad(:)=zero
hess(:,:)=zero
do l=1,3
  dens0=r(l)
  m=nint(dens0)
  ss(l)=dens0-real(m,double)
  is(l)=m+1
end do
!
!  if new position is out of box, shift of i-1, i, i+1 and i+2
!
!   should not appear with use of "floor" instead of "int"
do l=1,3
  dens0=ss(l)
  a(1,l)=((-dens0*over6+half)*dens0-over3)*dens0
  a(2,l)=((dens0*half-one)*dens0-half)*dens0+one
  a(3,l)=((-dens0*half+half)*dens0+one)*dens0
  a(4,l)=(dens0*dens0-one)*dens0*over6
!
  ap(1,l)=(-dens0*half+one)*dens0-over3
  ap(2,l)=(dens0*t3over2-two)*dens0-half
  ap(3,l)=(-dens0*t3over2+one)*dens0+one
  ap(4,l)=dens0*dens0*half-over6
!
  app(1,l)=-dens0+one
  app(2,l)=dens0*three-two
  app(3,l)=-dens0*three+one
  app(4,l)=dens0
end do
!
!  interpolation of density, gradient and hessian 
!
do n=1,4
  nn=(n-2)+is(3)
  nn=modulo(nn,np3)
  if(nn < 1) nn=nn+np3
  do m=1,4
    mm=(m-2)+is(2)         
    mm=modulo(mm,np2)
    if(mm < 1) mm=mm+np2
    do l=1,4
      ll=(l-2)+is(1)
      ll=modulo(ll,np1)
      if(ll < 1) ll=ll+np1
!
      dens0=rho(ll,mm,nn)
!
      if(mrho == -one) then
        dens0=-dens0
      else if(mrho == zero) then 
        dens0=log(dens0-drho)
      end if
!            density
      dens=dens+dens0*a(l,1)*a(m,2)*a(n,3)
!            gradient
      grad(1)=grad(1)+dens0*ap(l,1)*a(m,2)*a(n,3)
      grad(2)=grad(2)+dens0*a(l,1)*ap(m,2)*a(n,3)
      grad(3)=grad(3)+dens0*a(l,1)*a(m,2)*ap(n,3)
!            hessian
      hess(1,1)=hess(1,1)+dens0*app(l,1)*a(m,2)*a(n,3)
      hess(2,2)=hess(2,2)+dens0*a(l,1)*app(m,2)*a(n,3)
      hess(3,3)=hess(3,3)+dens0*a(l,1)*a(m,2)*app(n,3)
      hess(1,2)=hess(1,2)+dens0*ap(l,1)*ap(m,2)*a(n,3)
      hess(2,3)=hess(2,3)+dens0*a(l,1)*ap(m,2)*ap(n,3)
      hess(3,1)=hess(3,1)+dens0*ap(l,1)*a(m,2)*ap(n,3)
    end do
  end do
end do
hess(2,1)=hess(1,2)
hess(1,3)=hess(3,1)
hess(3,2)=hess(2,3)
!
end subroutine interp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine ranksig(eigval,ome,sig)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!    Determines rank and signature of a CP from eigenvalues 
!
real(kind=double), intent(out) :: ome,sig
real(kind=double), dimension(3), intent(in) :: eigval
!
integer(kind=int32) :: i
!
ome=three
sig=zero
do i=1,3
  if(abs(eigval(i)) < Lam_0) ome=ome-one
  sig=sig+real(sign(one,eigval(i)),double)
end do
!
end subroutine ranksig

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine testcp(icase,lambda,eigenvec)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
!!   STORES THE PROPERTIES OF THIS Critical POINT
!!   IN A TABLE OF THE CORRESPONDING CLASS :  
!!    attractor, saddle, ring, cage, strange 
!!
integer(kind=int32), intent(in) :: icase !!! type of CP
real(kind=double), dimension(3), intent(in) :: lambda    !!! eigenvalues of hess
real(kind=double), dimension(3,3), intent(in) :: eigenvec  !!! eigenvectors of hess
!
integer(kind=int32) :: inc   !!! counter for loop
integer(kind=int32) :: ncl   !!! nc for this icase
!
ncl=nc(icase)
lnc=.true.
!
! if first CP                 
!
if(ncl == 0) then
  ncl=ncl+1
  cp(icase,1,1:3)=rold(:)
  cp(icase,1,4)=densold
  cp(icase,1,5)=dgradold
  cp(icase,1,6:8)=lambda(:)
  cp(icase,1,9:11)=eigenvec(:,1)
  cp(icase,1,12:14)=eigenvec(:,2)
  cp(icase,1,15:17)=eigenvec(:,3)
  lnc=.false.
end if
!
! if not first CP  => compares to others
!
inc=0
do
  if(.not.(lnc .and. inc < ncl)) exit
  inc=inc+1
! dis(:)=abs(cp(icase,inc,1:3)-rold(:))
  dis(:)=0.666_double*abs(cp(icase,inc,1:3)-rold(:))
!
!Achtung: falls der Faktor 0.666 fehlt werden für Wasser 11 statt 7 (3,-1) Sattelpunkte HD gefunden !!!!!!!!!!!!!!!!!!!!
!
  if(dis(1) <= inc1 .and. dis(2) <= inc2 .and. dis(3) <= inc3) then
    if(dgradold < cp(icase,inc,5)) then
      cp(icase,inc,1:3)=rold(:)
      cp(icase,inc,4)=densold
      cp(icase,inc,5)=dgradold
      cp(icase,inc,6:8)=lambda(:)
      cp(icase,inc,9:11)=eigenvec(:,1)
      cp(icase,inc,12:14)=eigenvec(:,2)
      cp(icase,inc,15:17)=eigenvec(:,3)
    end if
    lnc=.false.
  end if
end do
!
! if new CP                       
!
if(lnc) then
  ncl=ncl+1
  if(ncl > ncmax) then
    write(unit=13,fmt="(a,i5)") "WARNING   number of CP greater than ncmax=",ncmax
    write(unit=*,fmt="(a,i5)") "WARNING   number of CP greater than ncmax=",ncmax
    stop
  end if
  cp(icase,ncl,1:3)=rold(:)
  cp(icase,ncl,4)=densold
  cp(icase,ncl,5)=dgradold
  cp(icase,ncl,6:8)=lambda(:)
  cp(icase,ncl,9:11)=eigenvec(:,1)
  cp(icase,ncl,12:14)=eigenvec(:,2)
  cp(icase,ncl,15:17)=eigenvec(:,3)
end if
!
nc(icase)=ncl
!
end subroutine testcp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine indexx(n,arr,indx)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   indexing subroutine to sort CP by decreasing density value
!
integer(kind=int32), intent(in) ::  n
integer(kind=int32), dimension(n), intent(out) :: indx
real(kind=double), dimension(n), intent(in) :: arr
!
integer(kind=int32) :: i,j,k,l
integer(kind=int32) :: indxt,ir,itemp,jstack
integer(kind=int32), parameter :: m=7
integer(kind=int32), parameter :: nstack=50
integer(kind=int32), dimension(nstack) :: istack
real(kind=double) :: a
logical :: test,flag
!
do j=1,n
  indx(j)=j
end do   
jstack=0
l=1
ir=n
do
  if(ir-l < m) then
    do j=l+1,ir
      indxt=indx(j)
      a=arr(indxt)
      test=.true.
      do i=j-1,1,-1
        if(arr(indx(i)) <= a) then
          test=.false.
          exit
        end if
        indx(i+1)=indx(i)
      end do
      if(test) i=0
      indx(i+1)=indxt
    end do
    if(jstack == 0) exit
    ir=istack(jstack)
    l=istack(jstack-1)
    jstack=jstack-2
  else
    k=(l+ir)/2
    itemp=indx(k)
    indx(k)=indx(l+1)
    indx(l+1)=itemp
    if(arr(indx(l+1)) > arr(indx(ir))) then
      itemp=indx(l+1)
      indx(l+1)=indx(ir)
      indx(ir)=itemp
    end if
    if(arr(indx(l)) > arr(indx(ir))) then
      itemp=indx(l)
      indx(l)=indx(ir)
      indx(ir)=itemp
    end if
    if(arr(indx(l+1)) > arr(indx(l))) then
      itemp=indx(l+1)
      indx(l+1)=indx(l)
      indx(l)=itemp
    end if
    i=l+1
    j=ir
    indxt=indx(l)
    a=arr(indxt)
    flag=.true.
    do
      if(.not.flag) exit
      test=.true.
      do
        if(.not.test) exit
        i=i+1
        if(arr(indx(i)) >= a) test=.false.
      end do
      test=.true.
      do
        if(.not.test) exit
        j=j-1
        if(arr(indx(j)) <= a) test=.false.
      end do
      if(j <= i) then
        flag=.false.
      else
        flag=.true.
        itemp=indx(i)
        indx(i)=indx(j)
        indx(j)=itemp
      end if
    end do
    indx(l)=indx(j)
    indx(j)=indxt
    jstack=jstack+2
    if(jstack > nstack) then
! pause
      print *,"NSTACK too small in indexx"
    end if
    if(ir-i+1 >= j-l) then
      istack(jstack)=ir
      istack(jstack-1)=i
      ir=j-1
    else
      istack(jstack)=j-1
      istack(jstack-1)=l
      l=i
    end if
  end if
end do
!
return
end subroutine indexx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine writeoutput(icase)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
!!   FINDS NEIGHBOURING ATOMS FOR EACH CP AND
!!
!!   WRITES ALL OUTPUT FILES FOR EACH TYPE OF CP
!!
integer(kind=int32), intent(in) :: icase          !!! type of CP
!
integer(kind=int32) :: ncl            !!! nc for this icase
integer(kind=int32) :: inc,i,j        !!! counter for loop
integer(kind=int32) :: i1,i2,i3,ix       !!! indices for closest atoms  
integer(kind=int32), dimension(3) :: indxa,indx_at
integer(kind=int32), dimension(:), allocatable :: cpindx
real(kind=double) :: dismin1,dismin2,dismin3,dismin !!! dist. atoms-CP
real(kind=double) :: a12,a23,a31     !!! angles CP with the closest atoms
real(kind=double), dimension(3) :: rf,rc,ar3,ad3
real(kind=double), dimension(3) :: r1,r2,r3     !!! coordinates of the closest atoms
real(kind=double), dimension(3,3) :: ar33
real(kind=double), dimension(:), allocatable :: cprho
logical :: ex,offen
character(len=1) :: flg     !!! used to mark doubtfull CP"s
character(len=3) :: lt1,lt2,lt3
character(len=6) :: nb1,nb2,nb3
character(len=7) :: label_cp
character(len=8), dimension(3) :: ar3c
character(len=10) :: lat1="          ",lat2="          ",lat3="          " !!! labels for atoms near CP
character(len=256) :: title_line
!
character(len=*), parameter :: fmt0="(a,3f10.5)"
character(len=*), parameter :: fmt1="(a,2(f8.5,3a),f8.5,2a)"
character(len=*), parameter :: fmt2="(a,es12.4,2(a,es9.2))"
character(len=*), parameter :: fmt3="(a,3es12.4,2(a,es9.2))"
character(len=*), parameter :: fmt4="(2(a,es9.2))"
character(len=*), parameter :: fmt5="(tr1,a5,tr4,a5,a7,tr1,a2,3f10.5,tr2,a6,f10.5,tr2,a4)"
character(len=*), parameter :: fmt6="(a,i4,a,f8.5,2(3a,f8.5),2a)"
character(len=*), parameter :: fmt7="(2(a,es12.4))"
character(len=*), parameter :: fmt9="(a,3es12.4)"
character(len=*), parameter :: fmt10="(i3,6es10.2,2f6.2,2es9.1,tr2,3(tr1,a6,tr1,a3),3f6.3,3f7.1,tr1,a1,tr1,f7.1)"
character(len=*), parameter :: fmt11="(tr14,a,3f10.5)"
!
ncl=nc(icase)
!
allocate(cpindx(ncl))
allocate(cprho(ncl))
do i=1,ncl
  cprho(i)=-cp(icase,i,4)
end do
call indexx(ncl,cprho,cpindx)
do i=1,17
  do j=1,ncl
    cprho(j)=cp(icase,j,i)
  end do
  do j=1,ncl
    cp(icase,j,i)=cprho(cpindx(j))
  end do
end do
!
!     selected CP cases:
!          1)  (3,-1) saddle points low density :: inter-molecular CP ?
!          2)  (3,-1)    "      "   high   " :: intra-molecular CP ?
!          3)  (3,+1) ring CP
!          4)  (3,+3) cage CP
!          5)  (3,-3) attractor
!          6)  other ?
!
select case(icase)
case(1)
  title_line="==================     (3,-1) SADDLE POINTS LOW DENSITY - INTERMOLECULAR     ===================="
  extension="-saddle-ld.results"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  write(unit=*,fmt=fmta) nfil
  open(unit=12,file=nfil,status="new",action="write")
  extension="-saddle-ld-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="SL"
case(2)
  title_line="==================     (3,-1) SADDLE POINTS HIGH DENSITY - INTRAMOLECULAR     ===================="
  extension="-saddle-hd.results"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=12,file=nfil,status="new",action="write")
  extension="-saddle-hd-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="SH"
case(3)
  title_line="==================     (3,+1) RING CRITICAL POINTS     ==========================================="
  extension="-ring.results"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=12,file=nfil,status="new",action="write")
  extension="-ring-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="RG"
case(4)
  title_line="==================     (3,+3) CAGE CRITICAL POINTS     ==========================================="
  extension="-cage.results" 
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=12,file=nfil,status="new",action="write")
  extension="-cage-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="CG"
case(5)
  title_line="==================     (3,-3) ATTRACTORS                ==========================================="
  extension="-attractor.results" 
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=12,file=nfil,status="new",action="write")
  extension="-attractor-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="AR"
case(6)
  title_line="? ? ? ? ? ? ? ? ?      OTHERS                        ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?"
  extension="-other.results" 
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=12,file=nfil,status="new",action="write")
  extension="-other-cp.list"
  nfil=trim(file_path)//trim(filename_2)//trim(extension)
  open(unit=17,file=nfil,status="new",action="write")
  rsi="OT"
end select
!
write(unit=17,fmt=fmta) trim(filename_2)
write(unit=12,fmt=fmta) trim(filename_2)
write(unit=17,fmt=*) lunit
write(unit=12,fmt="(/,a,i4,2/)") trim(title_line),nc(icase)
write(unit=13,fmt="(/,a,i4,2/)") trim(title_line),nc(icase)
write(unit=*,fmt="(/,a,i4,2/)") trim(title_line),nc(icase)
!
!      put there the header line to get the results in the shape of arrays
if(ncl >= 1) then
  if(cp_unit) then
    write(unit=*,fmt="(2a)") "  #  gradient  rho                 lambda               Lap     ratio ellip.", &
    "   G       V        A1         A2         A3         d1    d2    d3       a12    a23    a31"
    write(unit=*,fmt="(2a)") "     e-/A4     e-/A3                                                          kJ/mol   kJ/mol"
    write(unit=13,fmt="(2a)") "  #  gradient  rho                 lambda               Lap     ratio ellip.", &
    "    G       V        A1         A2         A3         d1    d2    d3       a12    a23    a31"
    write(unit=13,fmt="(2a)") "     e-/A4     e-/A3                                                          kJ/mol   kJ/mol"
  else   
    write(unit=*,fmt="(2a)") "  #  gradient  rho                 lambda               Lap     ratio ellip.", &
    "   G       V        A1         A2         A3         d1    d2    d3       a12    a23    a31"
    write(unit=*,fmt="(2a)") "     e-/au4    e-/au3                                                        eV/au3   eV/au3"
    write(unit=13,fmt="(2a)") "  #  gradient  rho                 lambda               Lap     ratio ellip.", &
    "    G       V        A1         A2         A3         d1    d2    d3       a12    a23    a31"
    write(unit=13,fmt="(2a)") "     e-/au4    e-/au3                                                        eV/au3   eV/au3"
  end if
end if
!
  inquire(file="debug.log",exist=ex,opened=offen)
  if(ex) then
    if(.not.offen) open(unit=40,file="debug.log",status="old",action="write",position="append")
  else
    open(unit=40,file="debug.log",status="new",action="write")
  end if
!
do inc=1,ncl
 !!  dismin1=MaxDens+50.0_double
  !  dismin2=dismin1+one
  ! dismin3=dismin2+one
  !! needs MaxDens=maxval(rho(r_box(1,1):r_box(1,2), r_box(2,1):r_box(2,2),r_box(3,1):r_box(3,2)))
  dismin1=50.0_double
  dismin2=51.0_double
  dismin3=52.0_double
  r1(:)=zero
  r2(:)=zero
  r3(:)=zero
  i1=0
  i2=0
  i3=0
  lat1="no atom" 
  lat2="no atom" 
  lat3="no atom" 
!
  do i=1,natoms
    dismin=sqrt((ratoms(i,1)-cp(icase,inc,1))**2+(ratoms(i,2)-cp(icase,inc,2))**2+(ratoms(i,3)-cp(icase,inc,3))**2)
    if(dismin < dismin3) then
      dismin3=dismin
      i3=i
      if(dismin3 < dismin2) then
        dismin=dismin2
        ix=i2
        dismin2=dismin3
        i2=i3
        dismin3=dismin
        i3=ix
        if(dismin2 < dismin1) then
          dismin=dismin1
          ix=i1
          dismin1=dismin2
          i1=i2
          dismin2=dismin
          i2=ix
        end if
      end if
    end if
  end do
!
  indx_at(1)=i1
  indx_at(2)=i2
  indx_at(3)=i3
  lat1=latoms(i1)
  r1(:)=ratoms(i1,:)-cp(icase,inc,1:3)
  lat2=latoms(i2)
  r2(:)=ratoms(i2,:)-cp(icase,inc,1:3)
  lat3=latoms(i3)
  r3(:)=ratoms(i3,:)-cp(icase,inc,1:3)
  a12=deg*acos(dot_product(r1,r2)/(sqrt(dot_product(r1,r1))*sqrt(dot_product(r2,r2))))
  a23=deg*acos(dot_product(r2,r3)/(sqrt(dot_product(r2,r2))*sqrt(dot_product(r3,r3))))
  a31=deg*acos(dot_product(r3,r1)/(sqrt(dot_product(r3,r3))*sqrt(dot_product(r1,r1))))
!
  ar3(1)=a12
  ar3(2)=a23
  ar3(3)=a31
  ad3(1)=dismin1
  ad3(2)=dismin2
  ad3(3)=dismin3
  ar3c(1)=lat1
  ar3c(2)=lat2
  ar3c(3)=lat3
  ar33(1,:)=r1(:)
  ar33(2,:)=r2(:)
  ar33(3,:)=r3(:)
!
  if(icase <= 3) then      !!! sorting by decreasing angles for bond CP"s
    call indexx(3,ar3,indxa)
    i3=indxa(1)
    i2=indxa(2)
    i1=indxa(3)
    a12=ar3(i1)
    a23=ar3(i2)
    a31=ar3(i3)
    dismin1=ad3(i1)
    dismin2=ad3(i2)
    dismin3=ad3(i3)
    r1(:)=ar33(i1,:)
    r2(:)=ar33(i2,:)
    r3(:)=ar33(i3,:)
    lat1=ar3c(i1)
    lat2=ar3c(i2)
    lat3=ar3c(i3)
  end if
!
!<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
! cp(icase,inc,19)=real(indx_at(i1),double)
! cp(icase,inc,20)=real(indx_at(i2),double)
! cp(icase,inc,21)=real(indx_at(i3),double)
  cp(icase,inc,19)=real(indx_at(1),double)
  cp(icase,inc,20)=real(indx_at(2),double)
  cp(icase,inc,21)=real(indx_at(3),double)
!<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  cp(icase,inc,22)=dismin1
  cp(icase,inc,23)=dismin2
  cp(icase,inc,24)=dismin3
!
  flg=" "        
  cp(icase,inc,18)=one
!
  select case(icase)
  case(1,2)
    if(sum(ar3(:)) < 330 .or. minval(ad3(:)) < 0.3) then
      flg="*"
      cp(icase,inc,18)=zero
    end if
  case(3,4)
    if(maxval(ar3(:)) < 60 .or. minval(ad3(:)) < 0.3) flg="*"
  case(5)
    if(minval(ad3(:)) >= 0.2) flg="*"
  case default
    flg=" "        
  end select
!
  laplacian=cp(icase,inc,6)+cp(icase,inc,7)+cp(icase,inc,8)
  ratio=abs(cp(icase,inc,8))/cp(icase,inc,6)
  ellip=(cp(icase,inc,8)/cp(icase,inc,7))-one
  grc=gfactor*cp(icase,inc,4)**t5over3+over6*laplacian
  vrc=laplacian/four-two*grc
  call CP_label(inc,label_cp,rsi)
!
  write(unit=40,fmt=*) trim(label_cp),"-->",lat1,"|",lat2,"|",lat3
!
  write(unit=12,fmt=*)
  write(unit=12,fmt=fmta) "------------------------------------------------------------"
  write(unit=12,fmt=fmt6) "Critical point number ",inc
  write(unit=12,fmt=fmta) "------------------------------------------------------------"
  write(unit=12,fmt=fmta) " # Properties in atomic units:"
  write(unit=12,fmt=fmt0) "   R=",cp(icase,inc,1:3)
  write(unit=12,fmt=fmt1) "     at ",dismin1," from ",trim(lat1),", ",dismin2," from ",trim(lat2), &
  ", and ",dismin3," from ",trim(lat3)
  write(unit=12,fmt=fmt2) "   Density ",cp(icase,inc,4),"   Gradient ",cp(icase,inc,5),"   Laplacian ",laplacian
  write(unit=12,fmt=fmt3) "   Lambdas ",cp(icase,inc,8:6:-1),"   Ellipticity ", &
  cp(icase,inc,8)/cp(icase,inc,7)-one,"   l1/l3 ",ratio
  write(unit=12,fmt=fmta) "    Eigenvectors :"
  write(unit=12,fmt=fmt11) "e1: ",cp(icase,inc,15:17)
  write(unit=12,fmt=fmt11) "e2: ",cp(icase,inc,12:14)
  write(unit=12,fmt=fmt11) "e3: ",cp(icase,inc,9:11)
  write(unit=12,fmt=fmt4) "   Gcp ",grc,"   Vcp ",vrc
!
!           put CP back to unit cell for listing files
!
  rf=matmul(c2f,cp(icase,inc,1:3))
  do i=1,3
    rf(i)=rf(i)-aint(rf(i))
  end do
  rc=matmul(f2c,rf)
  rc(:)=rc(:)/lunit
!
  write(unit=17,fmt=fmt5) "!CRPT","NAME=",label_cp,"R=",rc,"DENS=",cp(icase,inc,4),"!END"             
!
! WRITE THE SAME IN UNITS angstroms and kJ/mol
!
  write(unit=12,fmt=*)
  write(unit=12,fmt=fmta) " # Properties in Angstroms and KJ/mol"
  write(unit=12,fmt=fmt6) "   CP",inc," at ",a0*dismin1," from ",lat1,", ",a0*dismin2," from ",lat2, &
  ", and ",a0*dismin3," from ",lat3
  write(unit=12,fmt=fmt7) "   Density ",a03*cp(icase,inc,4),"  Laplacian ",laplacian*a05
  write(unit=12,fmt=fmt9) "   Lambdas ",a05*cp(icase,inc,8:6:-1)
  write(unit=12,fmt="(2(a,es11.4))") "   Gcp ",grc*kjpmol,"  Vcp ",vrc*kjpmol
!
  i1=index(lat1," ")
  nb1=lat1(1:i1-1)
  lt1=lat1(i1+1:len_trim(lat1))        
print *,"writeoutput: lat1,i1,nb1,i1+1-len_trim(lat1):",lat1,i1,nb1,i1+1-len_trim(lat1)
print *,"writeoutput: lt1:",lt1
  i2=index(lat2," ")
  nb2=lat2(1:i2-1)
  lt2=lat2(i2+1:len_trim(lat2))        
print *,"writeoutput: lat2,i2,nb2,i2+1-len_trim(lat2):",lat2,i2,nb2,i2+1-len_trim(lat2)
print *,"writeoutput: lt2:",lt2
  i3=index(lat3," ")
  nb3=lat3(1:i3-1)
  lt3=lat3(i3+1:len_trim(lat3))   
print *,"writeoutput: lat3,i3,nb3,i3+1-len_trim(lat3):",lat3,i3,nb3,i3+1-len_trim(lat3)
print *,"writeoutput: lt3:",lt3
!
!       write tables
!
  if(cp_unit) then
    write(unit=*,fmt=fmt10) inc,a04*cp(icase,inc,5),a03*cp(icase,inc,4),a05*cp(icase,inc,8:6:-1),a05*laplacian,ratio,ellip, &
    grc*kjpmol,vrc*kjpmol,nb1,lt1,nb2,lt2,nb3,lt3,a0*dismin1,a0*dismin2,a0*dismin3,a12,a23,a31,flg,a12+a23+a31
    write(unit=13,fmt=fmt10) inc,a04*cp(icase,inc,5),a03*cp(icase,inc,4),a05*cp(icase,inc,8:6:-1),a05*laplacian,ratio,ellip, &
    grc*kjpmol,vrc*kjpmol,nb1,lt1,nb2,lt2,nb3,lt3,a0*dismin1,a0*dismin2,a0*dismin3,a12,a23,a31,flg,a12+a23+a31
  else
    write(unit=*,fmt=fmt10) inc,cp(icase,inc,5),cp(icase,inc,4),cp(icase,inc,8:6:-1),laplacian,ratio,ellip,grc,vrc,nb1,lt1,nb2, &
    lt2,nb3,lt3,dismin1,dismin2,dismin3,a12,a23,a31,flg,a12+a23+a31         
    write(unit=13,fmt=fmt10) inc,cp(icase,inc,5),cp(icase,inc,4),cp(icase,inc,8:6:-1),laplacian,ratio,ellip,grc,vrc,nb1,lt1,nb2, &
    lt2,nb3,lt3,dismin1,dismin2,dismin3,a12,a23,a31,flg,a12+a23+a31
  end if
end do
!
close(unit=40)
!
write(unit=17,fmt=fmta) "!EOB"
!
close(unit=12)
close(unit=17)
!
end subroutine writeoutput

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine get_atoms()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! read ATOM PROPERTIES
!
integer(kind=int32) :: i1,i2,i
character(len=256) :: linecar
!
i=0
!
rewind(unit=11)
do
  if(i >= nat) exit
  read(unit=11,fmt=fmta) linecar
  if(index(linecar,"!ATOM") /= 0) then
    i1=5+index(linecar,"NAME=")
    if(i1 <= 5) then
      read(unit=11,fmt=fmta) linecar
      i1=5+index(linecar,"NAME=")
    end if
    if(i1 <= 5) then
      write(unit=13,fmt=*) "NAME= not found immediately after ATOM, program stopped"
      stop
    else
      i2=i1+index(linecar(i1+1:)," ")-2
      i=i+1
      lat(i)=linecar(i1+1:i2)
    end if
  end if
end do
!
i=0
rewind(unit=11)
do
  if(i >= nat) exit
  read(unit=11,fmt=fmta) linecar
  if(index(linecar,"!ATOM") /= 0) then
    i1=2+index(linecar,"R=")
    if(i1 <= 2) then
      read(unit=11,fmt=fmta) linecar
      i1=2+index(linecar,"R=")
    end if
    if(i1 <= 2) then
      write(unit=13,fmt=*) "R= not found immediately after ATOM, program stopped"
      stop
    else
      i=i+1
      read(unit=linecar(i1:),fmt=*) rat(i,:)
    end if
  end if
end do
!
return
end subroutine get_atoms

end module cpgrity_mod
