module constants_module

use kind_numbers
implicit none
private

real(kind=double), parameter, public :: zero=0.0_double
real(kind=double), parameter, public :: half=0.5_double
real(kind=double), parameter, public :: one=1.0_double
real(kind=double), parameter, public :: two=2.0_double
real(kind=double), parameter, public :: three=3.0_double
real(kind=double), parameter, public :: four=4.0_double
real(kind=double), parameter, public :: five=5.0_double
real(kind=double), parameter, public :: six=6.0_double

real(kind=double), parameter, public :: over3=one/three
real(kind=double), parameter, public :: over6=one/six
real(kind=double), parameter, public :: t3over2=three*half
real(kind=double), parameter, public :: t5over3=five*over3
real(kind=double), parameter, public :: pi=3.14159265358979323846_double
real(kind=double), parameter, public :: dpi=pi*two
real(kind=double), parameter, public :: deg=180.0_double/pi
real(kind=double), parameter, public :: kjpmol=one/3.8088e-4_double !!! kJ/mol -> a.u.

real(kind=double), parameter, public :: ModGradMin=1.0e-30_double  !!! if modgrad lt, assumed to 0  ==> WARNING

! list of non atomic labels, any name(*2) different from the following ones
! will be considered as density point attractors to be integrated
!   at_check_list(1)="DU"     !! dummy atom, generally unit cell vortices
!   at_check_list(2)="DO"     !! dipole origin (molecular)
!   at_check_list(3)="CP"     !! generic critical point
!   at_check_list(4)="SH"     !! saddle point (high density)
!   at_check_list(5)="SL"     !! saddle point (low density)
!   at_check_list(6)="CG"     !! cage CP
!   at_check_list(7)="RG"     !! ring CP
!   at_check_list(8)="DB"     !! unit cell vortices
!   at_check_list(9)="__"     !! not yet defined (user choice)
character(len=*), dimension(9), parameter, public :: at_check_list=(/ "DU","DO","CP","SH","SL","CG","RG","DB","__" /)

character(len=*), parameter, public :: fmta="(a)"

end module constants_module 
