module cp_module

use kind_numbers
use constants_module
implicit none
private

public :: CP_label
intrinsic :: char

integer(kind=int32), public :: ncmax=5000          !!! maximum number of CP
integer(kind=int32), dimension(6), public :: nc !!! total number of CP

real(kind=double), public :: laplacian,ratio,ellip,grc,vrc         
real(kind=double), public :: gfactor     !!! for kinetic energy at CP
real(kind=double), dimension(3), public :: dis  !!! distance between this CP and previous
real(kind=double), dimension(:,:,:), allocatable, public :: cp  !!! properties of cp 
logical, public :: cp_found,cp_attr   !!! flag
logical, public :: cp_unit   !!! flag

character(len=2), public :: rsi             !!! CP type 'SL' or 'SH'

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine CP_label(nn,cc,rs)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: nn
character(len=2), intent(in) :: rs
character(len=7), intent(out) :: cc
!
integer(kind=int32) :: nc,nd,nu,ns
!
ns=48  !!! shift to transform integer 0 to 9 to ASCII "0" to "9"
if(nn < 10) then
  cc=char(39)//rs//char(ns+nn)//char(39)
else if(nn < 100) then
  nd=nn/10
  nu=nn-10*nd
  cc=char(39)//rs//char(ns+nd)//char(ns+nu)//char(39)
else if(nn <= ncmax) then
  nc=nn/100
  nd=(nn-100*nc)/10
  nu=nn-100*nc-10*nd
  cc=char(39)//rs//char(ns+nc)//char(ns+nd)//char(ns+nu)//char(39)
else
  write(unit=13,fmt="(a)")"WARNING    number of CP  exceeding ncmax !"
  stop
end if
!
end subroutine CP_label

end module cp_module
