!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!   Determination of atomic basin limits and simultaneous integration   !!
!!      of total and valence electron densities within the basins        !!
!!         integrated volume and Laplacian are also returned             !!
!!                                                                       !!
!!         use of romberg method for integration convergence             !!
!!             in both  radial and angular coordinates                   !!
!!                                                                       !!
!!   P.Rabiller, C.Katan and M.Guezo        M.Souhassou                  !!
!!   GMCM UMR6626 Univ. Rennes1             LCM3B UMR 7036 Univ. Nancy 1 !!
!!   6 APRIL 2005                                                        !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   modified stuff is quoted with ##
!
!##  attention modifier la notice en consquence !!!
!##  out_romberg n'existe plus
!##  sortie des basins: sortie "long" n'existe plus !!!
!##  modes "interactifs", ie. "manual", "plane" et "line"  ne sont pas disponibles en "parallel"
!##  redfinir les entres pour le mode "line" (doit etre plus court que pour plane)
!##
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
Program main
!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
use atom_module
use basin_module
use constants_module
use filename_module
use file_units_module
use fordx
use grid_module
use integrity_mod
use integrity_dx_mod
use interpolation
use kind_numbers
use math_module
use plane_module
use results_module
use rho_module
use romberg_module
implicit none
!
intrinsic :: cpu_time,maxval,minval,nint,sum
!
integer(kind=int32), dimension(5), parameter :: k_prop=(/24,25,12,13,26/)
integer(kind=int32), dimension(3), parameter :: k_dip=(/29,30,31/)
!
integer(kind=int32) :: i,j,k,imol,i1,i2,j1,j2,k1,k2,ih1,ih2,ii
integer(kind=int32) :: start_t,end_t,nbt,nbp
integer(kind=int32), dimension(8) :: run_time
integer(kind=int32), dimension(:), allocatable :: indx_valid
real :: cpu_begin,cpu_end
real(kind=double) :: gfactor,grc,vrc           !!! for kinetic energy at CP
real(kind=double) :: rr2,dip,d_12,la,lb,lc,th_min,th_max,del_th,ph_min,ph_max,del_ph
real(kind=double), dimension(3) :: rx,rj,ta,tb,tc,r_1,r_2,r_2t,r_12,a,b,c
real(kind=double), dimension(3,3) :: hessj
real(kind=double) :: surf_dens,surf_lapl,surf_grad2
real(kind=double), dimension(3) :: surf_grad !!!!data at basin surface points positions for Dx
logical :: test_at,warning,scalar_run=.true.
character(len=256) :: test_kind
!
character(len=*), parameter :: fmt0="(tr8,a,/)"
!
!===============================================================================
!
!##   scalar_run should be set to ".false." if parallel run
!##   default value is ".true."
!
call cpu_time(cpu_begin)
gfactor=0.3_double*(three*pi**2)**(two/three)
!
if(scalar_run) then
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta) " InteGriTy_for_DX in progress ..."
  write(unit=SOU,fmt=fmta) ""
end if
call InteGriTy_initialize()
call ini_convergence_test()
!
if(scalar_run) then
  write(unit=SOU,fmt=fmta) "    Grid  & atoms properties reading ..."
  write(unit=SOU,fmt=fmta) ""
end if
if(index(filename_2,".in") == 0) then
  nfil=trim(filename_2)//".in"
else
  nfil=trim(filename_2)
end if
if(scalar_run) write(unit=SOU,fmt=fmt0) trim(nfil)
open(unit=MIU,file=trim(nfil),status="old",action="read",position="rewind")
call struc_initialize()
close(MIU)
!
extension=".integration"
nfil=trim(file_path)//trim(filename_1)//trim(extension)
if(scalar_run) write(unit=SOU,fmt=fmt0) trim(nfil)
open(unit=MOU,file=nfil,status="new",action="write")
!
call get_atom_par()
!
if(scalar_run) write(unit=SOU,fmt=fmta) "    Electron density grids reading ..."
if(scalar_run) write(unit=SOU,fmt=fmta) ""
!      extension="-tot.data"
nfil=trim(file_path)//trim(filename_1)//trim(extension_tot)
if(scalar_run) write(unit=SOU,fmt=fmt0) trim(nfil)
open(unit=MIU,file=nfil,status="old",action="read",form="unformatted",position="rewind")
call grid_initialize()
!
allocate(rho(np1,np2,np3))
call rho_initialize()
print *,"main: NACH rho_initialize"
call print_rhointerp_for_dx()
print *,"main: NACH print_rhointerp_for_dx"
close(MIU)
!
!      extension="-val.data"
nfil=trim(file_path)//trim(filename_1)//trim(extension_val)
if(scalar_run) write(unit=SOU,fmt=fmt0) trim(nfil)
open(unit=MIU,file=nfil,status="old",action="read",form="unformatted",position="rewind")
call rhov_initialize()
print *,"main: NACH rhov_initialize"
close(MIU)
!
call date_and_time(values=datetime)
!
!    Compute direct integration over the box, including non periodicity restriction if present
!
i1=int(r_box(1,1))
i2=int(r_box(1,2))
j1=int(r_box(2,1))
j2=int(r_box(2,2))
k1=int(r_box(3,1))
k2=int(r_box(3,2))
tot_grid_charge=sum(rho(i1:i2,j1:j2,k1:k2))*delv
val_grid_charge=sum(rhov(i1:i2,j1:j2,k1:k2))*delv
grid_volume=delv*real(i2-i1+1,double)*real(j2-j1+1,double)*real(k2-k1+1,double)
rho_tot_min=minval(rho(i1:i2,j1:j2,k1:k2))
rho_tot_max=maxval(rho(i1:i2,j1:j2,k1:k2))
rho_val_min=minval(rhov(i1:i2,j1:j2,k1:k2))
rho_val_max=maxval(rhov(i1:i2,j1:j2,k1:k2))
!
!    check number of valid attractors
allocate(indx_valid(nat))
nbas=0
j=0
do i=1,nat
  checked_atom=lat(i)
! is current atom true atom or dummy, or Critical Point ?
  call valid_atom(checked_atom,test_at)
  if(test_at .and. (rmin_atom(i) > zero) ) then   !! not retained if rmin declared <0
    j=j+1
    indx_valid(j)=i
    nbas=nbas+1
  end if
end do
allocate(indx_atoms(nbas))
indx_atoms(1:nbas)=indx_valid(1:nbas)
!
!##   allocate array for storing the basins" boundaries
allocate(rlim_thph(nbas,nth+1,nphi+1))
!
!     initialization of unit cell quantities
tot_cell_charge=zero
val_cell_charge=zero
cell_volume=zero
cell_laplacian=zero
tot_vol_err=zero
nb_e_tot=zero
nb_e_val=zero
do i=1,nbas
  iat=indx_atoms(i)
  nb_e_tot=nb_e_tot+real(nb_e_atom(iat),double)*mult_factor(iat)
  nb_e_val=nb_e_val+real(nb_eval_atom(iat),double)*mult_factor(iat)
end do
!
!     initializes molecular properties
!
mol_number=maxval(mol_index(:))
allocate(atom_prop(nbas,24))
allocate(integ_prop(nbas,60))
allocate(dip_mol(mol_number,3))
allocate(ori_mol(mol_number,3))
allocate(tot_mol_charge(mol_number))
allocate(val_mol_charge(mol_number))
allocate(mol_volume(mol_number))
allocate(mol_laplacian(mol_number))
allocate(nb_e_mol(mol_number))
allocate(nb_eval_mol(mol_number))
!
if(mol_number >= 1) then
  do imol=1,mol_number
    tot_mol_charge(imol)=zero
    val_mol_charge(imol)=zero
    mol_volume(imol)=zero
    mol_laplacian(imol)=zero
    nb_e_mol(imol)=0
    nb_eval_mol(imol)=0
    dip_mol(imol,:)=zero
    ori_mol(imol,:)=zero
    j=0
    do i=1,nbas
      iat=indx_atoms(i)
      if(mol_index(iat) == imol) then
        j=j+1
        ori_mol(imol,:)=ori_mol(imol,:)+rat(iat,:)
        nb_e_mol(imol)=nb_e_mol(imol)+real(nb_e_atom(iat),double)*mult_factor(iat)
        nb_eval_mol(imol)=nb_eval_mol(imol)+real(nb_eval_atom(iat),double)*mult_factor(iat)
      end if
    end do
    ori_mol(imol,:)=ori_mol(imol,:)/real(j,double)
  end do
  do i=1,nat
    iat=i
    checked_atom=lat(iat)
    if(index(checked_atom(1:2),"DO") /= 0) then
      read(unit=checked_atom(3:),fmt=*) imol
      write(unit=MOU,fmt="(/,a,3f9.5,a,i2,a,3f9.5,/))") "   !!! barycentric origin ",ori_mol(imol,:)," of molecule ",imol, &
         " replaced by ",rat(iat,:)
      ori_mol(imol,:)=rat(iat,:)
    end if
  end do
end if
!
!    prints out basin search and integration parameters,
!    atomic and grid characteristics
!
!    on output file
call print_parameters(MOU)
call print_atoms(MOU)
call print_grid(MOU)
!
!    on screen
if(scalar_run) then
  call print_parameters(SOU)
  call print_atoms(SOU)
  call print_grid(SOU)
end if
!
if(scalar_run .and. (index(running_mode,"plane") /= 0)) then
  write(unit=*,fmt=*) "call plane_rho"
  call plane_rho()
end if
if(scalar_run .and. index(running_mode,"line") /= 0) then
  write(unit=*,fmt=*) "call line_rho"
  call line_rho()
  stop "  line profile ended "
end if
!
!  LOOP OVER ALL ATOMS IN THE CELL
!  ===============================
!
do ii=1,nbas
  ibas=ii
  warning=.false.  ! at begining minimum basin radius is a priori correct
  iat=indx_atoms(ibas)
  count_radial=0
  count_phi=0  !! initialize file counters for manual "radial,phi" investigation
!
! takes current atom position, translates from absolute origin
! and computes grid coordinates
  ra(:)=rat(iat,:)
  rag(:)=ra(:)-origin(:)
  rgg(:)=matmul(c2g,rag(:))
!
  call date_and_time(values=datetime)
  run_time=datetime
  integ_prop(iat,48:50)=datetime(3:1:-1)
  integ_prop(iat,51:53)=datetime(5:7)
!
!       COARSE LIMITS SEARCH
!       ====================
!
! initializes starting distance for "coarse" basin boundary search
  r_start=rstart_atom(iat)
  dmin=rmin_atom(iat)
  if(scalar_run) call print_atom_header(SOU)
!
  out_basin=read_out_basin
!       initializes basin search parameters to "coarse" search
  call init_basin_search("coarse")
!
!       initialization of statistics purpose quantities
  rlim_max=zero
  rlim_mean=zero
  rr2=zero
  rlim_min=9.0_double
  max_rho=zero
  min_rho=1.0e10_double
  max_dist=zero
  min_dist=10.0_double
  max_grad=zero
  min_grad=1.0e10_double
  atom_surface=zero
!
!       perform theta & phi loops to compute basin boundary
  boundary_check=.true.
  dtheta=pi/real(nth,double)
  dphi=dpi/real(nphi,double)
  call basin_theta_phi_loops(rr2)
! computes rlim statistics
  rlim_mean=rlim_mean/real((nth+1)*(nphi+1),double)
  rr2=rr2/real((nth+1)*(nphi+1),double)
  rlim_sig=sqrt(rr2-rlim_mean*rlim_mean)
  atom_surface=atom_surface*dtheta*dphi
!
!       reset minimum radius bellow which considered as belonging to atomic basin
  dmin=coef_dmin*rlim_min
  if(dmin < rmin_atom(iat)) then
    warning=.true.
  end if
!
!!      integ_prop(ibas,1:9) contains results from coarse basin search
  integ_prop(ibas,1)=min_dist    ! explored minimum distance to the attractor
  integ_prop(ibas,2)=max_dist    ! explored maximum distance from the attractor
  integ_prop(ibas,3)=rlim_min    ! minimum basin radius
  integ_prop(ibas,4)=rlim_max    ! maximum basin radius
  integ_prop(ibas,5)=dmin        ! starting minimum basin radius for fine search
  integ_prop(ibas,6)=min_rho     ! minimum found rho value
  integ_prop(ibas,7)=max_rho     ! maximum found rho value
  integ_prop(ibas,8)=min_grad    ! minimum found grad(rho) value
  integ_prop(ibas,9)=max_grad    ! maximum found grad(rho) value
!
! prints out "coarse" atomic basin properties
  if(scalar_run) call print_coarse_basin_stat(SOU)
!
! initializes basin search parameters to "fine" search
  call init_basin_search("fine")
!
  atom_vol_err=atom_surface*dtol
  out_basin=.false.
!
!       initialization of loops numbers and statistics
  nloop_theta=0
  nloop_tot=0
  nloop_thph=0
  nloop_phi=0
  nloop_phi_min=jmax
  nloop_r_min=jmax
  nloop_phi_max=0
  nloop_r_max=0
!       initialization of atomic quantities
  tot_atom_charge=zero
  val_atom_charge=zero
  atom_volume=zero
  atom_laplacian=zero
!
  do i=1,24
    if(i <= 6) then
      rad_prop(i)=zero
      rad_dprop(i)=zero
    end if
    atom_prop(iat,i)=zero
    theta_prop(i)=zero
    phi_prop(i)=zero
    theta_dprop(i)=zero
    phi_dprop(i)=zero
  end do
!!
!!!!
!       check for "auto", "grid"  or "manual" running mode in case of scalar run
!       for parallel run "qromb_theta" is called
!
!
!         "basin"  ==> runs through all atoms only with initial 3D basin search
!                      for graphics output, "nth" and "nphi" should be increased
!                      as necessary in the "InteGriTy.in" file
!
!         "plane"  ==> runs through all atoms to get their basin intersection with
!                      a user defined plane. Density and its gradient are also
!                      interpolated on a plane regular grid. Output are writen
!                      on "filename.plane" file (UNIT=BOU)
!
!         "plane"  ==> search density, gradient, laplacian and ellipticity along
!                      a user defined line.Output are writen on "filename.line"
!                      file (UNIT=BOU)
!
!
!         "auto"   ==> runs through all atoms with "qromb_theta"
!
!         "grid"   ==> runs integration with regular angular grid
!
!         "manual" ==> output files to follow convergence according to
!
!          (r)  radial study at given theta & phi through all r
!               files : rho_tot_study.txt, idem for "_val", "_lap"
!                       radial_info.txt containing rho(r)
!
!          (p)  phi study at given theta through all r & phi
!               files : phi_tot_study.txt, idem for "_val", "_vol" & "_lap"
!
!          (t)  theta study through all r & phi & theta
!               files : theta_tot_study.txt, idem for "_val", "_vol" & "_lap"
!
!!
  if(scalar_run) then
    select case (running_mode)
    case("basin")
      write(unit=SOU,fmt="(/,a,/)") "   Quick Basin Search ..." ! no more integration !!!
!
    case("plane")
      write(unit=SOU,fmt="(/,a,/)") "   Plane Basin Search ..."
      call plane_basin()
!
    case ("auto")
      call qromb_theta()
!
    case ("manual")
      do
        write(unit=SOU,fmt=fmta,advance="no") " radial, phi or theta investigation ? "
        read(unit=KIU,fmt=fmta) test_kind
        if(len_trim(test_kind) == 0) then
          exit
        else
          rh_study=index(trim(test_kind),"r") /= 0
          ph_study=index(trim(test_kind),"p") /= 0
          th_study=index(trim(test_kind),"t") /= 0
          if(rh_study) then
            write(unit=SOU,fmt=fmta,advance="no") " Full or Single "
            read(unit=KIU,fmt=fmta) test_kind
            if(index(test_kind,"S") /= 0 .or. index(test_kind,"s") /= 0) then
              write(unit=SOU,fmt=fmta,advance="no") " input theta & phi : "
              read(unit=KIU,fmt=*) theta,phi
              nbt=1
              nbp=1
              test_kind="single"
            else
              test_kind="full"
              do
                write(unit=SOU,fmt=fmta,advance="no") " input theta  MIN  INC  MAX : "
                read(unit=KIU,fmt=*) th_min,del_th,th_max
                nbt=1+int((th_max-th_min)/del_th)
                write(unit=SOU,fmt=fmta,advance="no") " input phi    MIN  INC  MAX : "
                read(unit=KIU,fmt=*) ph_min,del_ph,ph_max
                nbp=1+int((ph_max-ph_min)/del_ph)
                if(nbt*nbp < 100) then
                  exit
                else
                write(unit=SOU,fmt=fmta) " Number of ""Theta*Phi"" must be less than 100,  enter new values !!!"
                end if
              end do
            end if
            do i=1,nbt
              do j=1,nbp
                count_radial=count_radial+1
                if(count_radial > 100) then
                  write(unit=SOU,fmt=fmta) " File counter > 100    ==> Starts again at #1 !!!"
                  count_radial=1
                end if
                if(index(test_kind,"full") /= 0) then
                  theta=th_min+real(i-1,double)*del_th
                  phi=ph_min+real(j-1,double)*del_ph
                end if
                sinth=sin(theta)
                costh=cos(theta)
                v_unit(1)=sinth*cos(phi)
                v_unit(2)=sinth*sin(phi)
                v_unit(3)=costh
                call rlim_and_dmin_adjust(warning)
                if(warning) write(unit=MOU,fmt="(a,f8.3,a,a)") "WARNING !!!   dmin=",dmin," too high for atom ",lat(iat)
                call qromb_rho()
                write(unit=SOU,fmt="(6f16.7)") rad_prop
                write(unit=SOU,fmt="(6f16.7)") rad_dprop
              end do
            end do
          end if
          if(ph_study) then
            write(unit=SOU,fmt=fmta,advance="no") " input theta : "
            count_phi=count_phi+1
            if(count_phi > 100) then
              write(unit=SOU,fmt=fmta) " File counter > 100    ==> Starts again at #1 !!!"
              count_phi=1
            end if
            read(unit=KIU,fmt=*) theta
            sinth=sin(theta)
            costh=cos(theta)
            v_unit(3)=costh
            call qromb_phi()
            write(unit=SOU,fmt="(3f16.7)") phi_prop(6:8)
            write(unit=SOU,fmt="(3f16.7)") phi_dprop(6:8)
          end if
          if(th_study) then
            call qromb_theta()
            write(unit=SOU,fmt="(3f16.7)") theta_prop(6:8)
            write(unit=SOU,fmt="(3f16.7)") theta_dprop(6:8)
          end if
        end if
      end do
    end select
  else
    call qromb_theta()
  end if
!
!!
!!!!
  atom_prop(iat,1:24)=theta_prop(1:24)
  atom_prop(iat,1)=atom_prop(iat,1)*over3
!!
!!!!
!!
  integ_prop(ibas,10)=rlim_min       !  fine minimum basin radius
  integ_prop(ibas,11)=rlim_max       !  fine maximum basin radius
  integ_prop(ibas,12)=rlim_mean      !  fine mean basin radius
  integ_prop(ibas,13)=rlim_sig       !  fine basin radius standard deviation
!
  if(romb_stat) then
    nloop_thph=nloop_thph+2
    nloop_tot=nloop_tot+2
    integ_prop(ibas,14)=real(nloop_tot,double) ! # of integration loops
    integ_prop(ibas,15)=real(2**(nloop_r_min-1)+1,double) ! min  # of radial loops
    integ_prop(ibas,16)=nint(real(nloop_tot,double)/real(nloop_thph),double) ! mean
    integ_prop(ibas,17)=real(2**(nloop_r_max-1)+1,double)                     ! max
    integ_prop(ibas,18)=real(2**(nloop_phi_min-1)+1,double)                   ! min  # of phi loops
    integ_prop(ibas,19)=nint(real(nloop_thph,double)/(integ_prop(ibas,18)-2)) ! mean
    integ_prop(ibas,20)=real(2**(nloop_phi_max-1)+1,double)                   ! max
    integ_prop(ibas,21)=real(2**(nloop_theta-1)+1,double)                     ! # of theta loops
    integ_prop(ibas,22)=real(nloop_thph,double)                               ! # of theta&phi loops
  end if
!
  if(boundary_check) then
    integ_prop(ibas,23)=one  ! =1. no box boundary encountered
  else
    integ_prop(ibas,23)=zero  ! =0. if box boundary encountered
  end if
!
  integ_prop(ibas,24)=theta_prop(2)              ! basin surface
  integ_prop(ibas,25)=theta_prop(1)*over3        ! basin volume
  integ_prop(ibas,26)=theta_prop(3)              ! basin total charge
  integ_prop(ibas,27)=theta_prop(4)              ! basin valence charge
  integ_prop(ibas,28)=theta_prop(5)              ! basin Laplacian sum
  integ_prop(ibas,29:31)=theta_prop(6:8)         ! Dipole
  integ_prop(ibas,32:37)=theta_prop(9:14)*half   ! Quadrupole
  integ_prop(ibas,38:47)=theta_prop(15:24)       ! Octupole
!
  call date_and_time(values=datetime)
  call date_time_to_sec(datetime,end_t)
  call date_time_to_sec(run_time,start_t)
  call integration_time(start_t,end_t,j_time,h_time,m_time,s_time)
  integ_prop(ibas,54)=real(j_time,double)
  integ_prop(ibas,55)=real(h_time,double)
  integ_prop(ibas,56)=real(m_time,double)
  integ_prop(ibas,57)=real(s_time,double)
  if(warning) then
    integ_prop(ibas,58)=one
  else
    integ_prop(ibas,58)=zero
  end if
end do ! "ibas" basins (valid atoms only)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!     OUTPUTS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! creates  a file containing integrated basins positions and if asked boundary surfaces
!
open(unit=BLU,file=trim(file_path)//trim(filename_1)//".basin",status="new",action="write")
write(unit=BLU,fmt=fmta) trim(filename_1)
!!!!! output box informations
ta(:)=real(np1,double)*del1(:)
tb(:)=real(np2,double)*del2(:)
tc(:)=real(np3,double)*del3(:)
la=sqrt(dot_product(ta,ta))/real(np1,double)
lb=sqrt(dot_product(tb,tb))/real(np2,double)
lc=sqrt(dot_product(tc,tc))/real(np3,double)
write(unit=BLU,fmt="(a,3(i4,a2),a,3f8.3,a)") "___grid:  ",np1," x",np2," x",np3," x","  (",la,lb,lc,")"
write(unit=BLU,fmt="(a,3f10.5)") "___grid_origin@ ",origin(:)
write(unit=BLU,fmt=fmta) "___grid_vertices:"
write(unit=BLU,fmt="(a,3f10.5)") "   (1)  ",zero,zero,zero
write(unit=BLU,fmt="(a,3f10.5)") "   (2)  ",ta(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (3)  ",ta(:)+tb(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (4)  ",tb(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (5)  ",tb(:)+tc(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (6)  ",tc(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (7)  ",ta(:)+tc(:)
write(unit=BLU,fmt="(a,3f10.5)") "   (8)  ",ta(:)+tb(:)+tc(:)
call print_box_for_dx(ta,tb,tc)
write(unit=BLU,fmt="(a,i5)") "___number_of_input_atoms:",nat !nbas
write(unit=BLU,fmt=fmta) "___atomic_positions:"
do ii=1,nat ! nbas  !! nat substitu  nbas
  ibas=ii
  iat=ibas
  write(unit=BLU,fmt="(i3,tr2,a6,tr1,3f12.6)") ibas,trim(lat(iat)),rat(iat,:)
end do
!!
a=del1*real(np1,double)
b=del2*real(np2,double)
c=del3*real(np3,double)
allocate(bonds_name(20000))
open(unit=35,file=trim(file_path)//"dx/bonds.dx",status="new",action="write")
ibond=0
write(unit=BLU,fmt=fmta) "___bonds:"
do i1=1,nat                  !!! nat substitu ici  nbas
  i2=i1
  r_1=rat(i2,:)
  ih1=index(lat(i2),"H")
  do j1=i1+1,nat                !!! nat substitu ici  nbas
    j2=j1
    r_2=rat(j2,:)
    ih2=index(lat(j2),"H")
    if(ih1 /= 1 .or. ih2 /= 1) then     !!! no H---H bonds !
      if(iso_test) then
        r_12=r_2-r_1
        d_12=sqrt(dot_product(r_12,r_12))
        if(d_12 > zero .and. d_12 <= max_bond_length) then
          ibond=ibond+1
          write(unit=21,fmt="(i3,a3,i3,2(tr5,3f12.6),tr5,f9.5)") i2," ---",j2,r_1,r_2,d_12
          call print_bonds_for_dx(35,r_1,r_2,d_12,i2,j2)
          bonds_name(ibond)=trim(lat(i2))//"---"//trim(lat(j2))
        end if
      else
        do i=-1,1
          ta=real(i,double)*a(:)
          do j=-1,1
            tb=real(j,double)*b(:)
            do k=-1,1
              tc=real(k,double)*c(:)
              r_2t=r_2+ta+tb+tc
              r_12=r_2t-r_1
              d_12=sqrt(dot_product(r_12,r_12))
              if(d_12 > zero .and. d_12 <= max_bond_length) then
                ibond=ibond+1
                write(unit=21,fmt="(i3,a3,i3,2(tr5,3f12.6),tr5,f9.5)") i2," ---",j2,r_1,r_2t,d_12
                call print_bonds_for_dx(35,r_1,r_2t,d_12,i2,j2)
                bonds_name(ibond)=trim(lat(i2))//"---"//trim(lat(j2))
              end if
            end do
          end do
        end do
      end if
    end if
  end do  !! loop over 2nd atom "j1"
end do    !! loop over 1st atom "i1"
write(unit=BLU,fmt=fmta) "___end_of_bonds"
call print_bonds_group_for_dx(35,ibond)
close(35)
deallocate(bonds_name)
!!!
write(unit=BLU,fmt="(a,2i5)") "___number_of_theta_&_phi_steps:",nth,nphi
dtheta=pi/real(nth,double)
dphi=dpi/real(nphi,double)
do ii=1,nbas
  ibas=ii
  iat=indx_atoms(ibas)
!! open files ".data" for Dx: 1 file for each basin
  open(unit=36,file=trim(file_path)//"dx/"//trim(lat(iat))//".bxyz",status="new",action="write")
  open(unit=35,file=trim(file_path)//"dx/"//trim(lat(iat))//".prop",status="new",action="write")
!!! modifier ici pour ajouter S,V,Rmean, sig(R), q dip, ex,ey,ez
!!!!!!!
  dip=sqrt(dot_product(integ_prop(ibas,29:31),integ_prop(ibas,29:31)))
  write(unit=BLU,fmt="(a,tr2,9f9.3)") "___atomic_surface@"//trim(lat(iat)),integ_prop(ibas,k_prop),dip,integ_prop(ibas,k_dip)/dip
  theta=zero
!
  do i=1,nth+1
    v_unit(3)=cos(theta)
    sinth=sin(theta)
    phi=zero
    do j=1,nphi+1
      v_unit(1)=sinth*cos(phi)
      v_unit(2)=sinth*sin(phi)
      rx(:)=rat(iat,:)+rlim_thph(ibas,i,j)*v_unit(:)
      write(unit=BLU,fmt=*) rx
      write(unit=36,fmt=*) rx
      rj=matmul(c2g,rx)
!!!!!compute rho,laplacian,G and V at rlim pos...
      call interp(rj,surf_dens,surf_grad,.true.,hessj)
      hessj=matmul(c2gt,hessj)
      hessj=matmul(hessj,c2g)
      surf_lapl=hessj(1,1)+hessj(2,2)+hessj(3,3)
      surf_grad=matmul(c2gt,surf_grad)
      surf_grad=matmul(surf_grad,c2g)
      surf_grad2=surf_grad(1)**2+surf_grad(2)**2+surf_grad(3)**2
      if(surf_dens < 1.0e-5) surf_dens=1.0e-5_double
      grc=gfactor*surf_dens**t5over3+over6*surf_lapl+(one/72.0_double)*surf_grad2/surf_dens
!!...kinetic energy in case grad is not null
      vrc=surf_lapl/four-two*grc
      write(unit=35,fmt=*) surf_dens,surf_lapl,grc,vrc
      phi=phi+dphi
    end do
    theta=theta+dtheta
  end do
  close(36)!".xyz" file
  close(35)!".prop"
end do
!
write(unit=BLU,fmt=fmta) "___end_of_file"
close(BLU)
write(unit=*,fmt=*) trim(file_path)//"dx/basin.dx"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
open(unit=35,file=trim(file_path)//"dx/basin.dx",status="new",action="write")  !modify:should use defined filepath
call print_basin_result_for_dx(35)
close(35)
open(unit=35,file=trim(file_path)//"dx/atoms.dx",status="new",action="write")
call print_atom_results_for_dx(35)
close(35)
!
if(index(running_mode,"plane") == 0) then
  do ii=1,nbas
    ibas=ii
    iat=indx_atoms(ibas)
    call print_atom_header(MOU)
    if(scalar_run) call print_atom_header(SOU)
!!
    call print_atom_results(MOU)
    if(scalar_run) call print_atom_results(SOU)
!
! takes into account atom multiplicity
! and adds up atomic quantities to "cell" ones
    tot_atom_charge=integ_prop(ibas,26)*mult_factor(iat)
    val_atom_charge=integ_prop(ibas,27)*mult_factor(iat)
    atom_volume=integ_prop(ibas,25)*mult_factor(iat)
    atom_laplacian=integ_prop(ibas,28)*mult_factor(iat)
    atom_vol_err=integ_prop(ibas,24)*dtol*mult_factor(iat)
!
    tot_cell_charge=tot_cell_charge+tot_atom_charge
    val_cell_charge=val_cell_charge+val_atom_charge
    cell_volume=cell_volume+atom_volume
    tot_vol_err=tot_vol_err+atom_vol_err
    cell_laplacian=cell_laplacian+atom_laplacian
!
!       adds up atomic quantities to "molecular" ones
    do imol=1,mol_number
      if(mol_index(iat) == imol) then
        tot_mol_charge(imol)=tot_mol_charge(imol)+tot_atom_charge
        val_mol_charge(imol)=val_mol_charge(imol)+val_atom_charge
        mol_volume(imol)=mol_volume(imol)+atom_volume
        mol_laplacian(imol)=mol_laplacian(imol)+atom_laplacian
        do j=1,3
          if(tot_use) then
            dip_mol(imol,j)=dip_mol(imol,j)-(tot_atom_diff*(rat(iat,j)-ori_mol(imol,j))+integ_prop(ibas,j+28))
          else
            dip_mol(imol,j)=dip_mol(imol,j)-(val_atom_diff*(rat(iat,j)-ori_mol(imol,j))+integ_prop(ibas,j+28))
          end if
        end do
      end if
    end do  ! imol
  end do
end if    !! if(index(running_mode,"plane") /= 0)
!
if(index(running_mode,"plane") /= 0) close(BOU)
!
call date_and_time(values=datetime)
end_time(:)=datetime(:)
!
! prints out cell and molecular integration results
if(index(running_mode,"plane") == 0) then
  call print_cell_results(MOU)
  if(scalar_run) call print_cell_results(SOU)
end if
!
write(unit=MOU,fmt="(2/,a)") " Job done ..."
!
deallocate(rlim_thph)
deallocate(rho)
deallocate(rhov)
deallocate(lat)
deallocate(rat)
deallocate(rstart_atom)
deallocate(rmin_atom)
deallocate(nb_e_atom)
deallocate(nb_eval_atom)
deallocate(atom_size)
deallocate(atom_color)
close(MOU)
call cpu_time(cpu_end)
write(unit=*,fmt="(tr3,a,f9.2,a)") "Total elapsed CPU time = ",cpu_end-cpu_begin," sec"
!
end program main
