module integrity_dx_mod

use atom_module
use basin_module
use constants_module
use filename_module
use file_units_module
use fordx
use grid_module
use interpolation
use kind_numbers
use math_module, only: eigen
use plane_module
use romberg_module

implicit none
private

public :: line_rho,print_box_for_dx,print_bonds_for_dx,print_bonds_group_for_dx,print_basin_result_for_dx, &
print_atom_results_for_dx,print_rhointerp_for_dx

private :: num2ascii

intrinsic :: char,dot_product,index,len_trim,matmul,sqrt,trim

integer(kind=int32), public :: ibond
character(len=20), dimension(:), allocatable, public :: bonds_name !! name of bondx for dx

character(len=*), parameter, private :: gmta="(tr3,a)"

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine line_rho()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   subroutine to initialize a line with regular  spacing
!   and to compute the density and its gradient on that meshgrid
!
integer :: i,j,k
!integer :: imolp   !<<<<<<<<<<<<<<<<<
real(kind=double) :: lam1,lam2,lam3,lap,ellip
real(kind=double) :: densj
real(kind=double) :: gx
!real(kind=double) :: xrho,prho,xlap    !<<<<<<<<<<<<<<<<<
real(kind=double), dimension(3) :: rx,rj,gradx,gradj
real(kind=double), dimension(3) :: lambda
real(kind=double), dimension(3,3) :: hessj,eigen_vec
character(len=6) :: lbl
!
!xrho=0.3_double*(three*pi**2)**(two/three)  !<<<<<<<<<<<<<<<<<<
!prho=(5.0_double/three)   !<<<<<<<<<<<<<<<<<<<
!xlap=(one/six)      !<<<<<<<<<<<<<<<
!!
if(index(plane_def,"interactively") /= 0) then
!!! plane specifications are not given in the integrity.in file, ask from user
  write(unit=SOU,fmt=fmta) "  INTERACTIVE DEFINITION FOR SEARCH ON A LINE"
  write(unit=SOU,fmt=fmta) "  OUTPUT GIVES x, rho(x), |grad(rho(x))|, lapl., ellip."
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta) "   # Line definition"
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta) "      grid orientation matrix : "
  do i=1,3
    write(unit=SOU,fmt="(tr1,3f12.6)") real(np1,double)*del1(i),real(np2,double)*del2(i),real(np3,double)*del3(i)
  end do
  write(unit=SOU,fmt="(/,a,3f10.4,/)") "      grid origin : ",origin
!
!     origin atom , 2nd  and 3rd atoms defining Oxy plane
!
  write(unit=SOU,fmt=fmta) ">>> atoms list : "
  do i=1,mol_number
    write(unit=SOU,fmt="(2/,tr2,a,i2)") " molecule ",i
    k=0
    do j=1,nat
      if(mol_index(j) == i) then
        k=k+1
        write(unit=SOU,fmt="(tr1,a6)",advance="no") trim(lat(j))
        if(modulo(k,10) == 0) then
          write(unit=SOU,fmt=fmta) " ..."
          write(unit=SOU,fmt=fmta) ""
        end if
      end if
    end do
  end do
!
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta,advance="no") "      enter atom #1 defining line origin  : "
  read(unit=SOU,fmt=fmta) lbl1
  write(unit=SOU,fmt=fmta,advance="no") "      enter atom #2 defining line end     : "
  read(unit=SOU,fmt=fmta) lbl2
  write(unit=SOU,fmt=fmta,advance="no") "      enter number of points : "
  read(unit=SOU,fmt=*) nx
end if
!
extension=".line"
nfil=trim(file_path)//trim(filename_1)//trim(extension)
open(unit=BOU,file=nfil,status="new",action="write")
!
if(len_trim(lbl1) < 1 .or. len_trim(lbl2) < 1) then
  lbl1="O"
  lbl2="k"
  p1(:)=zero
  p2=real(np3,double)*del3
else
  do i=1,nat
    lbl=lat(i)
    if(lbl1 == lbl) then
      p1(:)=rat(i,:)
!     imolp=mol_index(i)  !<<<<<<<<<<<<<<<<<<
    end if
    if(lbl2 == lbl) then
      p2(:)=rat(i,:)
    end if
  end do
end if
!
ra1(:)=p1(:)-origin(:)
ra2(:)=p2(:)-p1(:)
!
v1(:)=p2(:)-p1(:)
u1=v1/real(nx,double)
dxo=sqrt(dot_product(u1,u1))
!
write(unit=BOU,fmt=fmta) filename_1
write(unit=BOU,fmt="(4(a,tr3))") lbl1,lbl2
write(unit=BOU,fmt="(i4,tr2,f13.7)") nx,dxo
!
write(unit=SOU,fmt=fmta) " Line profile "
write(unit=SOU,fmt="(4(a,tr3))") lbl1,lbl2
write(unit=SOU,fmt="(i4,tr2,f13.7)") nx,dxo
!
!     GRADIENT INTERPOLATION IN THE LINE
!     ===================================
!
do i=0,nx
  rx(:)=real(i,double)*u1(:)
  dyo=real(i,double)*dxo
  rj=matmul(c2g,rx)
  call interp(rj,densj,gradj,.true.)
  call interp(rj,densj,gradj,.true.,hessj)
  hessj=matmul(c2gt,hessj)
  hessj=matmul(hessj,c2g)
  gradx=matmul(c2gt,gradj)
  gx=sqrt(dot_product(gradx,gradx))
  call eigen(hessj,lambda,eigen_vec)
  lam3=lambda(1)
  lam2=lambda(2)
  lam1=lambda(3)
  ellip=lam1/lam2-one
  lap=lam1+lam2+lam3
!
!   rajouter ici la densite d'energie cinetique
!
  write(unit=BOU,fmt=*) dyo,densj,gx,lap,ellip
end do
return
!
end subroutine line_rho

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!!!!OUTPUTS FOR OPEN DX
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_basin_result_for_dx(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!output file for OpenDX
!informations about basins:surfaces,color,atom position and label
!!!!!!!a color file should be reading!!!!!!
!
integer, intent(in) :: lu
!
integer(kind=int32) :: nitem,ibas
!
nitem=(nphi+1)*(nth+1)
!
write(unit=lu,fmt="(a,2i4)") "Object ""connec_"" class gridconnections counts",nth+1,nphi+1 !component ""connection"", don"t change
write(unit=lu,fmt=gmta) "attribute ""element type"" string ""quads"""
write(unit=lu,fmt=gmta) "attribute ""ref"" string ""positions"""
write(unit=lu,fmt=*)
!
do ibas=1,nbas
  iat=indx_atoms(ibas)
!write one field for each basin
  write(unit=lu,fmt="(a,i8,a)") "Object ""xyz_"//trim(lat(iat))//""" class array type float rank 1 shape 3 items ",nitem, &
    " data file"
  write(unit=lu,fmt=fmta) trim(lat(iat))//".bxyz"
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
!!
  write(unit=lu,fmt="(a,i8,a)") "Object ""properties_"//trim(lat(iat))//""" class array type float rank 1 shape 4 items ",nitem
  write(unit=lu,fmt=fmta) " data file "//trim(lat(iat))//".prop"
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) "Object ""color_"//trim(lat(iat))//""" class constantarray type float rank 1 shape 3 items "
  write(unit=lu,fmt="(i5,a)") nitem," data follows"
  write(unit=lu,fmt="(tr3,3(f4.2,tr1))") atom_color(ibas,:)!atom color
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
!
!!!!!integred data to select basin in dx
  write(unit=lu,fmt=fmta) "Object ""DataVolume_"//trim(lat(iat))//""" class constantarray type float rank 1 shape 1 items"
  write(unit=lu,fmt="(i5,a)") nitem," data follows"
  write(unit=lu,fmt="(tr3,f10.3)") integ_prop(ibas,25)
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
!!!!!!
  write(unit=lu,fmt=fmta) "Object ""DataCharge_"//trim(lat(iat))//""" class constantarray type float rank 1 shape 1 items"
  write(unit=lu,fmt="(i5,a)") nitem," data follows"
  write(unit=lu,fmt="(tr3,f10.3)") integ_prop(ibas,26)
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
!!!!!!
  write(unit=lu,fmt=fmta) "Object ""DataLaplacian_"//trim(lat(iat))//""" class constantarray type float rank 1 shape 1 items"
  write(unit=lu,fmt="(i5,a)") nitem," data follows"
  write(unit=lu,fmt="(tr3,f10.3)") integ_prop(ibas,28)
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
!!!!!!
  write(unit=lu,fmt=fmta) "Object ""basin_field_"//trim(lat(iat))//""" class field"
  write(unit=lu,fmt=gmta) "component ""positions"" "//"""xyz_"//trim(lat(iat))//""""
  write(unit=lu,fmt="(tr3,a,tr3,a)") "component ""connections""","""connec_"""
  write(unit=lu,fmt=gmta) "component ""colors"" ""color_"//trim(lat(iat))//""""!only two first letters of lat..
  write(unit=lu,fmt=gmta) "component ""Volume"" ""DataVolume_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""Charge"" ""DataCharge_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""Properties"" ""properties_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""Laplacian"" ""DataLaplacian_"//trim(lat(iat))//""""
  write(unit=lu,fmt="(tr3,a,f10.3,a)") "attribute ""Surf"" string ""Surface:",integ_prop(ibas,24),""""
  write(unit=lu,fmt="(tr3,a,f10.3,a)") "attribute ""Vol"" string ""Volume:",integ_prop(ibas,25),""""
  write(unit=lu,fmt="(tr3,a,f10.3,a)") "attribute ""Charge"" string ""Total charge:",integ_prop(ibas,26),""""
  write(unit=lu,fmt="(tr3,a,f10.3,a)") "attribute ""Lapl"" string ""Laplacian:",integ_prop(ibas,28),""""!!!!modify:print there usefull informations
  write(unit=lu,fmt=*) "#----------------------------------------------"
end do
!
!basin names together
write(unit=lu,fmt="(a,i3,a)") "Object ""basin_name"" class array type string rank 1 shape 12 items ",nbas," data follows"
do ibas=1,nbas
  iat=indx_atoms(ibas)
  write(unit=lu,fmt="(tr3,a1,a6,a,a1)") """",ba,trim(lat(iat)),""""
end do
write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=lu,fmt=*)
!basin positions together(atom positions are used)
write(unit=lu,fmt="(a,i8,a)") "Object ""xyz_basin"" class array type float rank 1 shape 3 items",nbas," data follows"
do ibas=1,nbas
  iat=indx_atoms(ibas)
  write(unit=lu,fmt="(tr2,3(tr1,f9.4))") rat(iat,:)
end do
write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
!
write(unit=lu,fmt=*)
write(unit=lu,fmt=*) "Object ""basin_text"" class field"
write(unit=lu,fmt="(tr3,a21,tr1,a12)") CP,"""xyz_basin"""
write(unit=lu,fmt="(tr3,a16,tr1,a1,a6,a4,a1)") CD,"""",Ba,"name",""""
write(unit=lu,fmt=*) "#------------------------------------------------------"
!
!group of basin fields
write(unit=lu,fmt=fmta) "Object ""basindata"" class group"
do ibas=1,nbas
  iat=indx_atoms(ibas)
  write(unit=lu,fmt=gmta) M//" """//Ba//trim(lat(iat))//""" """//ba//f//trim(lat(iat))//""""
end do
write(unit=lu,fmt=*) "#------------------------------------------------------------"
write(unit=lu,fmt=*)
write(unit=lu,fmt=*) "Object ""centres"" class group"
write(unit=lu,fmt=gmta) "member 0 value ""basin_text"""
!
write(unit=lu,fmt=*) "Object ""basins_all"" class group"
write(unit=lu,fmt=*) "member 0 value ""centres"""
write(unit=lu,fmt=*) "member 1 value ""basindata"""
!
end subroutine print_basin_result_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_atom_results_for_dx(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer, intent(in) :: lu
!
integer :: ibas
!
do ibas=1,nbas
  iat=indx_atoms(ibas)
!print one field for each atom
  write(unit=lu,fmt=fmta) "Object ""xyz_atom"//trim(lat(iat))//""" class array rank 1 shape 3 items 1 data follows"
  write(unit=lu,fmt="(tr3,3f9.4)") rat(iat,:)!atom position
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) "Object ""label_"//trim(lat(iat))//""" class array type string rank 1 shape 3 items 1 data follows"
  write(unit=lu,fmt="(tr3,a1,a,a1)") """",trim(lat(iat)),""""!atom label
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
!
  write(unit=lu,fmt=fmta) "Object ""color_"//trim(lat(iat))//""" class array type float rank 1 shape 3 items 1 data follows"
  write(unit=lu,fmt="(tr3,3(f4.2,tr1))") atom_color(ibas,:)!atom color
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
!
  write(unit=lu,fmt=fmta) "Object ""data_"//trim(lat(iat))//""" class array type float rank 1 shape 1 items 1 data follows"
  write(unit=lu,fmt="(tr3,f4.2)") atom_size(ibas)	!atom size
  write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) "Object ""atom_field"//trim(lat(iat))//""" class field"
  write(unit=lu,fmt=gmta) "component ""positions""  ""xyz_atom"//trim (lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""data_string"//"""  """//"label_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""colors""     ""color_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "component ""data""     ""data_"//trim(lat(iat))//""""
  write(unit=lu,fmt=gmta) "attribute ""num"" string ""Hello"""!info about atom
!
  write(unit=lu,fmt=*) "#----------------------------------------------"
end do
write(unit=lu,fmt=*)
write(unit=lu,fmt=*) "Object ""atom_group"" class group"
do ibas=1,nbas
  iat=indx_atoms(ibas)
  write(unit=lu,fmt=gmta) "member """//trim(lat(iat))//"""  value ""atom_field"//trim(lat(iat))//""""
end do
write(unit=lu,fmt=*)
end subroutine print_atom_results_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine print_bonds_for_dx(lu,pos1,pos2,dist,at1,at2)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer, intent(in) :: lu
integer(kind=int32), intent(in) :: at1,at2
real(kind=double), intent(in) :: dist !distance between atoms
real(kind=double), dimension(3), intent(in) :: pos1,pos2 !atom positions
!
character(len=5) :: stbond
!
call num2ascii(ibond,stbond)
!
write(unit=lu,fmt=*) "Object ""xyz_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""!!!!!!
write(unit=lu,fmt=*) " class array type float rank 1 shape 3 items 2 data follows"
write(unit=lu,fmt="(tr1,3(f10.5,tr2))") pos1
write(unit=lu,fmt="(tr1,3(f10.5,tr2))") pos2
write(unit=lu,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "Object ""line_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=fmta) "class array type int rank 1 shape 2 items 1 data follows"!!!!!!!!!
write(unit=lu,fmt=gmta) "0 1"
write(unit=lu,fmt=gmta) "attribute ""element type"" string ""lines"""
write(unit=lu,fmt=gmta) "attribute ""ref"" string ""positions"""
!!!
write(unit=lu,fmt=fmta) "Object ""data_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=fmta) "class array type float rank 1 shape 1 items 1 data follows"
write(unit=lu,fmt="(tr1,f10.5)") dist
write(unit=lu,fmt=gmta) "attribute ""dep"" string ""connections"""
!
write(unit=lu,fmt=fmta) "Object ""color_"//trim(lat(at1))//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=fmta) " class array type float rank 1 shape 3 items 1 data follows"
write(unit=lu,fmt=gmta) "0.  1. .5"
write(unit=lu,fmt=gmta) "attribute ""dep"" string ""connections"""
!
write(unit=lu,fmt=fmta) "Object ""bond_field_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""" class field"
write(unit=lu,fmt=gmta) "component ""positions""  ""xyz_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=gmta) "component ""connections"" ""line_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=gmta) "component ""colors""  ""color_"//trim(lat(at1))//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=gmta) "component ""data"" ""data_"//trim(lat(at1))//"---"//trim(lat(at2))//trim(stbond)//""""
write(unit=lu,fmt=fmta) "#----------------------------------------------------------------------------"
!
end subroutine print_bonds_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine print_bonds_group_for_dx(lu,j)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu,j
!
integer(kind=int32) :: i
character(len=7) :: stbond
!
write(unit=lu,fmt=*) "Object ""bonds_group"" class group"
do i=1,j
  call num2ascii(i,stbond)
  write(unit=lu,fmt=gmta) "member """//trim(bonds_name(i))//trim(stbond)//""" value ""bond_field_"//trim(bonds_name(i))// &
trim(stbond)//""""
end do
!
end subroutine print_bonds_group_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine num2ascii(nn,cc)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: nn
character(len=*), intent(out) :: cc
!
integer(kind=int32) :: nm,nc,nd,nu,ns,nl
!
ns=48     !!! shift to transform integer 0 to 9 to ASCII "0" to "9"
if(nn < 10) then
  cc=char(ns+nn)
else if(nn < 100) then
  nd=nn/10
  nu=nn-10*nd
  cc=char(ns+nd)//char(ns+nu)
else if(nn < 100) then
  nc=nn/100
  nd=(nn-100*nc)/10
  nu=nn-100*nc-10*nd
  cc=char(ns+nc)//char(ns+nd)//char(ns+nu)
else if(nn < 1000) then
  nm=nn/1000
  nc=(nn-1000*nm)/100
  nd=(nn-100*nc-1000*nm)/10
  nu=nn-100*nc-10*nd-1000*nm
  cc=char(ns+nm)//char(ns+nc)//char(ns+nd)//char(ns+nu)
else if(nn < 10000) then
  nl=nn/10000
  nm=(nn-nl*10000)/1000
  nc=(nn-1000*nm-nl*10000)/100
  nd=(nn-100*nc-1000*nm-nl*10000)/10
  nu=nn-100*nc-10*nd-1000*nm-nl*10000
cc=char(ns+nl)//char(ns+nm)//char(ns+nc)//char(ns+nd)//char(ns+nu)
else
  write(unit=13,fmt=fmta) "WARNING    number of CP  exceeding ncmax !"
  stop
end if
end subroutine num2ascii

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_box_for_dx(a,b,c)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
real(kind=double), dimension(3), intent(in) :: a,b,c
!
open(35,file=trim(file_path)//"dx/box.dx",status="new",action="write")
!
write(unit=35,fmt=*) "Object ""pos_box"" class array type float rank 1 shape 3 items 8 data follows"
write(unit=35,fmt="(tr1,3f10.5)") zero,zero,zero
write(unit=35,fmt="(tr1,3f10.5)") a(:)
write(unit=35,fmt="(tr1,3f10.5)") a(:)+b(:)
write(unit=35,fmt="(tr1,3f10.5)") b(:)
write(unit=35,fmt="(tr1,3f10.5)") b(:)+c(:)
write(unit=35,fmt="(tr1,3f10.5)") c(:)
write(unit=35,fmt="(tr1,3f10.5)") a(:)+c(:)
write(unit=35,fmt="(tr1,3f10.5)") a(:)+b(:)+c(:)
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
write(unit=35,fmt=*) "Object ""connect_"" class array type int rank 1 shape 2 items 12 data follows"
write(unit=35,fmt=gmta) "0 1"
write(unit=35,fmt=gmta) "1 2"
write(unit=35,fmt=gmta) "2 3"
write(unit=35,fmt=gmta) "0 3"
write(unit=35,fmt=gmta) "4 5"
write(unit=35,fmt=gmta) "5 6"
write(unit=35,fmt=gmta) "6 7"
write(unit=35,fmt=gmta) "4 7"
write(unit=35,fmt=gmta) "1 6"
write(unit=35,fmt=gmta) "0 5"
write(unit=35,fmt=gmta) "3 4"
write(unit=35,fmt=gmta) "2 7"
write(unit=35,fmt=gmta) "attribute ""element type"" string ""lines"""
write(unit=35,fmt=gmta) "attribute ""ref"" string ""positions"""
write(unit=35,fmt=*)
write(unit=35,fmt=fmta) "Object ""avector"" class constantarray type float rank 1 shape 3 items 8"
write(unit=35,fmt=fmta) " data follows"
write(unit=35,fmt="(tr3,3(f10.3))") a(:)
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
!!!!!!
write(unit=35,fmt=fmta) "Object ""bvector"" class constantarray type float rank 1 shape 3 items 8"
write(unit=35,fmt=fmta) " data follows"
write(unit=35,fmt="(tr3,3(f10.3))") b(:)
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
!!!!!!
write(unit=35,fmt=fmta) "Object ""cvector"" class constantarray type float rank 1 shape 3 items 8"
write(unit=35,fmt=fmta) " data follows"
write(unit=35,fmt="(tr3,3(f10.3))") c(:)
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
write(unit=35,fmt=*)
write(unit=35,fmt=fmta) "Object ""box_field"" class field"
write(unit=35,fmt=gmta) "component ""positions"" ""pos_box"""
write(unit=35,fmt=gmta) "component ""connections"" value ""connect_"""
write(unit=35,fmt=gmta) "component ""ta"" ""avector"""
write(unit=35,fmt=gmta) "component ""tb"" ""bvector"""
write(unit=35,fmt=gmta) "component ""tc"" ""cvector"""
!
close(35)
!
end subroutine print_box_for_dx

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_rhointerp_for_dx()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!!nmax=nombre max de points accepted by OpenDx
!!nbx,nby,nbz==nbre de box in each direction
!!np1,np2,np3==nbre of printed points in each direction
!!
integer(kind=int32) :: nbox
integer(kind=int32) :: ng1,ng2,ng3,nn,jx,jy,jz
!
real(kind=single) :: rho_4,lap_4
!
real(kind=double) :: alpha,Gx,Gy,Gz
real(kind=double) :: densi       !!! output density
real(kind=double), dimension(3) :: rdx,shift_ori !!! input position, origin shift
real(kind=double), dimension(3) :: gradi       !!! output gradient
real(kind=double), dimension(3,3) :: hessi
!!!
nbox=nbx*nby*nbz
if(nbox /= 1) then
  shift_ori=-del1*np1-del2*np2-del3*np3
else
  shift_ori(:)=zero
end if
!
nn=nbx*np1*nby*np2*nbz*np3
alpha=(real(nmaxdx,double)/real(nn,double))**over3
if(alpha >= one) alpha=one
ng1=int(nbx*np1*alpha)
ng2=int(nby*np2*alpha)
ng3=int(nbz*np3*alpha)
Gx=real(nbx*np1,double)/real(ng1,double)
Gy=real(nby*np2,double)/real(ng2,double)
Gz=real(nbz*np3,double)/real(ng3,double)
!
write(unit=*,fmt=*)
write(unit=*,fmt=*) "   Output grid for OpenDx"
write(unit=*,fmt="(tr5,a,f12.4)") "alpha = ",alpha
write(unit=*,fmt="(tr5,3(a,i6))") "Nb points OpenDx grid: ",ng1," x ",ng2," x ",ng3
write(unit=*,fmt="(tr5,3(a,f12.4))") "Magnification:  Gx=",Gx,"  Gy=",Gy,"  Gz=",Gz
write(unit=*,fmt=*)
!
open(35,file=trim(file_path) // "dx/rhoascii2.ascii",status="new",action="write")
open(36,file=trim(file_path) // "dx/lapascii2.ascii",status="new",action="write")
!
do jx=1,ng1
  rdx(1)=one+real(Jx-1,double)*Gx
  do jy=1, ng2
    rdx(2)=one+real(Jy-1,double)*Gy
    do jz=1,ng3
      rdx(3)=one+real(Jz-1,double)*Gz
      call interp(rdx,densi,gradi,.true.,hessi)
      rho_4=densi
      hessi=matmul(c2gt,hessi)
      hessi=matmul(hessi,c2g)
      lap_4=hessi(1,1)+hessi(2,2)+hessi(3,3)
      write(unit=35,fmt=*) rho_4
      write(unit=36,fmt=*) lap_4
    end do
  end do
end do
close(35)
close(36)
open(35,file=trim(file_path)//"dx/rhodx2.dx",status="new",action="write")!!!create file for Dx to read density
!
write(unit=35,fmt=fmta) "Object ""rho_data"" class array type float rank 1 shape 1 items"
write(unit=35,fmt="(i12,a)") ng1*ng2*ng3," data file rhoascii2.ascii"!!!!!!!!!!!!!!
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
!write(unit=35,fmt="(a,tr1,3(i3,tr1))") "Object ""position_1"" class gridpositions counts",nboxX*int(np1*Gr2),nboxY*int(np2*Gr2),nboxZ*int(np3*Gr2)
write(unit=35,fmt="(a,tr1,3(i3,tr1))") "Object ""position_1"" class gridpositions counts",ng1,ng2,ng3   !!ng3,ng2,ng1
write(unit=35,fmt="(a,tr1,3(f9.5,tr1))") "origin",origin+shift_ori
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del1*nbx*np1)/ng1
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del2*nby*np2)/ng2
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del3*nbz*np3)/ng3
write(unit=35,fmt=*)
write(unit=35,fmt="(a,tr1,3(i3,tr1))") "Object ""liaison_1"" class gridconnections counts",int(nbx*alpha*np1),int(nby*alpha*np2), &
int(nbz*alpha*np3)
write(unit=35,fmt=gmta) "attribute ""element type"" string ""cubes"""
write(unit=35,fmt=gmta) "attribute ""ref"" string ""positions"""
write(unit=35,fmt=gmta) "attribute ""dep"" string ""connections"""
write(unit=35,fmt=*)
write(unit=35,fmt=fmta) "Object ""rho_field"" class field"
write(unit=35,fmt=gmta) "component ""positions"" ""position_1"""
write(unit=35,fmt=gmta) "component  ""data""  ""rho_data"""
write(unit=35,fmt=gmta) "component ""connections"" ""liaison_1"""
!
close (35)
open(35,file=trim(file_path)//"dx/lapdx2.dx",status="new",action="write")!!!create file for Dx to read density
!
write(unit=35,fmt=fmta) "Object ""rho_data"" class array type float rank 1 shape 1 items"
write(unit=35,fmt="(i12,a)") ng1*ng2*ng3," data file lapascii2.ascii"!!!!!!!!!!!!!!
write(unit=35,fmt=gmta) "attribute ""dep"" string ""positions"""
write(unit=35,fmt=*)
write(unit=35,fmt="(a,tr1,3(i3,tr1))") "Object ""position_1"" class gridpositions counts",ng1,ng2,ng3
write(unit=35,fmt="(a,tr1,3(f9.5,tr1))") "origin",origin+shift_ori
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del1*nbx*np1)/ng1
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del2*nby*np2)/ng2
write(unit=35,fmt="(a,tr1,3(f9.5))") "delta",(del3*nbz*np3)/ng3
write(unit=35,fmt=*)
write(unit=35,fmt="(a,tr1,3(i3,tr1))") "Object ""liaison_1"" class gridconnections counts",int(nbx*alpha*np1),int(nby*alpha*np2), &
int(nbz*alpha*np3)
write(unit=35,fmt=gmta) "attribute ""element type"" string ""cubes"""
write(unit=35,fmt=gmta) "attribute ""ref"" string ""positions"""
write(unit=35,fmt=gmta) "attribute ""dep"" string ""connections"""
write(unit=35,fmt=*)
write(unit=35,fmt=fmta) "Object ""rho_field"" class field"
write(unit=35,fmt=gmta) "component ""positions"" ""position_1"""
write(unit=35,fmt=gmta) "component  ""data""  ""rho_data"""
write(unit=35,fmt=gmta) "component ""connections"" ""liaison_1"""
!
close(35)
end subroutine print_rhointerp_for_dx

end module integrity_dx_mod
