module integrity_mod

use atom_module
use basin_module
use constants_module
use filename_module
use file_units_module
use grid_module
use interpolation
use kind_numbers
use math_module
use plane_module
use results_module
use rho_module
use romberg_module

implicit none
private

public :: InteGriTy_initialize,ini_convergence_test,struc_initialize,get_atom_par,grid_initialize,rho_initialize,rhov_initialize, &
valid_atom,print_parameters,print_grid,print_atoms,plane_rho,print_atom_header,init_basin_search,basin_theta_phi_loops, &
print_coarse_basin_stat,plane_basin,qromb_theta,rlim_and_dmin_adjust,qromb_rho,qromb_phi,date_time_to_sec,integration_time, &
print_atom_results,print_cell_results

private :: get_one_par,cross_product,get_atoms,cell_integration,inout_box_check,inout_basin_check,basin_coarse_bound, &
basin_fine_bound,rlim_interp,ini_convergence_test_0,polint,trapz_rho,trapz_phi,trapz_theta,wo_blk

intrinsic :: char,cos,date_and_time,dot_product,exp,floor,index,len,len_trim,log,matmul,max,min,sin,transpose,trim

real(kind=double), parameter, public :: max_bond_length=3.5_double !!! maximum bondlength to consider output bonds

contains

!>>>>>
!>
!>     InteGriTy PARAMETERS" INITIALIZATION ROUTINES
!>
!>>>>>
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine InteGriTy_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32) :: io_error,i_get,j,ifb,ilb,ib,ie
real(kind=double) :: x_get
character(len=1) :: par_type
character(len=20) :: key_wrd
character(len=160) :: c_get
character(len=256) :: one_line
!
open(unit=MIU,file="integrity.in",iostat=io_error,status="old",action="read",position="rewind")
!
!!!  check if "InteGriTy.in" file exists or can be opened
if(io_error /= 0) then
  close(MIU)
  write(unit=SOU,fmt=*) " file integrity.in not found or unreadable - program stops"
  stop
else
!
!!!  check file extension for total and valence densities
!
  key_wrd="ext"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,"c")
  if(index(c_get,"no_occ") /= 0) then
    extension_tot="-tot.data"
    extension_val="-val.data"
  else
    if(index(c_get,"|") > 1) then
      j=index(c_get,"|")
      extension_tot=trim(c_get(1:j-1))
      extension_val=trim(c_get(j+1:))
      j=1
      do
        if(extension_tot(j:j) == " ") then
          j=j+1
        else
          exit
        end if
      end do
      extension_tot=extension_tot(j:len_trim(extension_tot))
      j=1
      do
        if(extension_val(j:j) == " ") then
          j=j+1
        else
          exit
        end if
      end do
      extension_val=extension_val(j:len_trim(extension_val))
    else
      close(MIU)
      write(unit=SOU,fmt=*) " data filename extensions badly specified - program stops"
      stop
    end if
  end if
!
!!!  check pathway  and (data & atoms) specifications" file
!
  c_get=""
  key_wrd="file"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"no_occ") /= 0 .or. len_trim(c_get) <= 0) then
    close(MIU)
    write(unit=SOU,fmt=*) " data and atoms files not found - program stops"
    stop
  else
    ifb=index(trim(c_get)," ")
    ilb=len_trim(c_get)
    if(ilb == 0 .or. ilb > len(filename_2)) then
      close(MIU)
      open(unit=MIU,file="InteGriTy.error",status="new",action="write")
      write(unit=MIU,fmt=*) " data filename too long - program stops"
      write(unit=SOU,fmt=*) " data filename too long - program stops"
      close(MIU)
      stop
    end if
    filename_2=c_get(ifb+1:ilb)
  end if
!
  key_wrd="path"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  file_path=trim(c_get)
  if(index(file_path,"no_occ") /= 0) file_path=""
  if(len_trim(file_path) > 0) then
    one_line=trim(file_path)//trim(filename_2)
    call wo_blk(one_line,ib,ie)
    filename_2=one_line(ib:ie)
  end if
!
!!!  check grid type  : periodic or isolated system and van der waals surface check
!
  key_wrd="grid"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"iso") /= 0) then
    iso_test=.true.
  else
    iso_test=.false.
  end if
  if(index(c_get,"vdv") /= 0 .or. index(c_get,"van") /= 0) then
    vdv_test=.true.
  else
    vdv_test=.false.
  end if
!
!  Initial values for many variables set in basin_module and romberg_module
!
!!!    modification of parameters if specified by key words in InteGriTy.in
!
  key_wrd="tot_use"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"val") /= 0) tot_use=.false.
  key_wrd="conv"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"no_occ") /= 0) then
    convergence_test="val_vol"
  else
    convergence_test=trim(c_get)
  end if
  key_wrd="rho_vdv"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,rho_vdv,par_type)
  key_wrd="dr_c"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,dr_coarse,par_type)
  key_wrd="dr_f"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,dr_fine,par_type)
  key_wrd="ed_c"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,ed_coarse,par_type)
  key_wrd="ed_f"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,ed_fine,par_type)
  key_wrd="ea_c"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,ea_coarse,par_type)
  key_wrd="ea_f"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,ea_fine,par_type)
  key_wrd="dtol_c"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,dtol_coarse,par_type)
  key_wrd="dtol_f"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,dtol_fine,par_type)
  key_wrd="fa_c"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,fa_coarse,par_type)
  key_wrd="fa_f"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,fa_fine,par_type)
  key_wrd="exp_dg"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,exp_dg,par_type)
  key_wrd="coef_d"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,coef_dmin,par_type)
  key_wrd="nth"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nth,x_get,par_type)
  key_wrd="nph"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nphi,x_get,par_type)
  key_wrd="nmax"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nmax,x_get,par_type)
  key_wrd="jmax"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,jmax,x_get,par_type)
  key_wrd="kmin"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,kmin,x_get,par_type)
  key_wrd="kmax"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,kmax,x_get,par_type)
  key_wrd="eps_r"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,eps_r,par_type)
  key_wrd="eps_phi"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,eps_phi,par_type)
  key_wrd="eps_theta"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,eps_theta,par_type)
  key_wrd="eps_theta"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,eps_theta,par_type)
  key_wrd="out"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
!
  key_wrd="nmaxdx"
  par_type="x"
  call get_one_par(MIU,key_wrd,c_get,i_get,nmaxdx,par_type)
  key_wrd="nbx"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nbx,x_get,par_type)
  key_wrd="nby"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nby,x_get,par_type)
  key_wrd="nbz"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nbz,x_get,par_type)
!
  if(index(c_get,"basi") /= 0) read_out_basin=.true.
  if(index(c_get,"romb") /= 0) out_romberg=.true.
  if(index(c_get,"mult") /= 0) out_mul=.true.
  if(index(c_get,"no_occ") /= 0) then
    read_out_basin=.true.
    out_mul=.true.
    long_output=.false.
    out_romberg=.false.
  end if
  key_wrd="stat"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"romb") /= 0) then
    romb_stat=.true.
  else
    romb_stat=.false.
  end if
  key_wrd="work"
  par_type="c"
  call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
  if(index(c_get,"no_occ") /= 0) running_mode="auto"
  if(index(c_get,"auto") /= 0) running_mode="auto"
  if(index(c_get,"grid") /= 0) running_mode="grid"
  if(index(c_get,"man") /= 0) running_mode="manual"
  if(index(c_get,"bas") /= 0) running_mode="basin"
  if(index(c_get,"plane") /= 0) running_mode="plane"
  if(index(c_get,"line") /= 0) running_mode="line"
  key_wrd="max_step_fwd"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,max_step_fwd,x_get,par_type)
  key_wrd="min_step_dwd"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,min_step_dwd,x_get,par_type)
  key_wrd="ntg"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,ntg,x_get,par_type)
  key_wrd="nrg"
  par_type="i"
  call get_one_par(MIU,key_wrd,c_get,nrg,x_get,par_type)
!
  jmaxp=jmax+1
  km1=kmax-1
  kint=max(kmin,kmax)
!!!!
!     plane definition
!
  middle_23="n"
  perppl="parallel"
  incx=0.1_double
  incy=0.1_double
  nx=40
  ny=40
  nz=1
  dxo=zero
  dyo=zero
  dzo=zero
  nal=180
  npcp=5
  if(index(running_mode,"plane") /= 0 .or. index(running_mode,"line") /= 0) then
    key_wrd="plane_def"
    par_type="c"
    call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
    if(index(c_get,"no_occ") /= 0) c_get="interactive"
    if(index(c_get,"interactive") == 0) then
      plane_def="plane definition given in integrity.in file"
      key_wrd="atom1"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
      if(index(c_get,"no_occ") /= 0) then
        lbl1=""
      else
        call wo_blk(c_get,ib,ie)
        lbl1=c_get(ib:ie)
      end if
      key_wrd="atom2"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
      if(index(c_get,"no_occ") /= 0) then
        lbl2=""
      else
        call wo_blk(c_get,ib,ie)
        lbl2=c_get(ib:ie)
      end if
      key_wrd="atom3"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
      if(index(c_get,"no_occ") /= 0) then
        lbl3=""
      else
        call wo_blk(c_get,ib,ie)
        lbl3=c_get(ib:ie)
      end if
      key_wrd="middle_23"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
      if(index(c_get,"no_occ") /= 0) then
        middle_23="no"
      else
        call wo_blk(c_get,ib,ie)
        middle_23=c_get(ib:ie)
      end if
      key_wrd="perp_plane"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,"c")
      if(index(c_get,"no_occ") /= 0) then
        perppl="parallel"
        perpxy="XY"
      else
        call wo_blk(c_get,ib,ie)
        perppl=c_get(ib:ie)
        if(index(perppl,"y") /= 0 .or. index(perppl,"x") /= 0) then
          if(index(perppl,"y") /= 0) perpxy="YZ"
          if(index(perppl,"x") /= 0) perpxy="XZ"
          perppl="perpendicular"
        else
          perppl="parallel"
          perpxy="XY"
        end if
      end if
      key_wrd="incx"
      par_type="x"
      call get_one_par(MIU,key_wrd,c_get,i_get,incx,par_type)
      key_wrd="incy"
      par_type="x"
      call get_one_par(MIU,key_wrd,c_get,i_get,incy,par_type)
      key_wrd="nx"
      par_type="i"
      call get_one_par(MIU,key_wrd,c_get,nx,x_get,par_type)
      key_wrd="ny"
      par_type="i"
      call get_one_par(MIU,key_wrd,c_get,ny,x_get,par_type)
      key_wrd="centering"
      par_type="c"
      call get_one_par(MIU,key_wrd,c_get,i_get,x_get,par_type)
      if(index(c_get,"molec") == 0 .or. index(c_get,"no_occ") /= 0) then
        molor="manual offset"
        key_wrd="dxo"
        par_type="x"
        call get_one_par(MIU,key_wrd,c_get,i_get,dxo,par_type)
        key_wrd="dyo"
        par_type="x"
        call get_one_par(MIU,key_wrd,c_get,i_get,dyo,par_type)
      else
        molor="molecular centering"
      end if
      key_wrd="dzo"
      par_type="x"
      call get_one_par(MIU,key_wrd,c_get,i_get,dzo,par_type)
      key_wrd="nz"
      par_type="i"
      call get_one_par(MIU,key_wrd,c_get,nz,x_get,par_type)
      key_wrd="nal"
      par_type="i"
      call get_one_par(MIU,key_wrd,c_get,nal,x_get,par_type)
      key_wrd="npcp"
      par_type="i"
      call get_one_par(MIU,key_wrd,c_get,npcp,x_get,par_type)
      key_wrd="tol2plane"
      par_type="x"
      call get_one_par(MIU,key_wrd,c_get,i_get,tol2plane,par_type)
!etc.
    else
      plane_def="plane definition given interactively"
    end if
  end if
!
end if
close(MIU)
return
!
end subroutine InteGriTy_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine wo_blk(car_str,i_begin,i_end)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!----------------------------------------------------------------------------
!     this subroutine search for the position "ib" of the first non blank
!     character and the position "ie+1" of the next blank occurence
!     in string "car_string"
!
integer(kind=int32), intent(out) :: i_begin,i_end
character(len=*), intent(in) :: car_str
!
integer(kind=int32) :: i,j
integer(kind=int32) :: len_str
!
len_str=len(car_str)
i_begin=1
i_end=len_str
do i=1,len_str
  if(car_str(i:i) == " ") then
    cycle
  else
    i_begin=i
    exit
  end if
end do
!
do j=min(i+1,len_str),len_str
  if(car_str(j:j) == " ") then
    i_end=j-1
    exit
  else
    cycle
  end if
end do
!
end subroutine wo_blk

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine get_one_par(i_unt,kwrd,c_io,i_io,x_io,cix_flag)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! enables initialization of parameters character, integer or real
! reading such "key_word = value !" expressions in "InteGriTy.in" file
!
integer(kind=int32), intent(in) :: i_unt
integer(kind=int32), intent(inout) :: i_io
real(kind=double), intent(inout) :: x_io
character(len=*), intent(inout) :: c_io
character(len=*), intent(in) :: cix_flag
character(len=*), intent(in) :: kwrd
!
integer(kind=int32) :: eol,jkw,i1,i2,io_err,i_line
character(len=256) :: o_line
!
rewind(unit=i_unt)
i_line=0
do
  i_line=i_line+1
  read(unit=i_unt,fmt=fmta,iostat=io_err) o_line
  if(io_err == 0) then
    if(index(o_line(1:2),"!!") /= 0) cycle
    if(index(o_line,"EOB!") /= 0 .or. index(o_line,"END!") /= 0) exit
    jkw=index(o_line,trim(kwrd))
    if(jkw /= 0) then
      eol=len_trim(o_line)
      i1=index(o_line(jkw:eol),"=")+jkw
      if(i1 == jkw) then
        cycle        ! empty line
      else
        i2=index(o_line(jkw:eol),"!")
        if(i2 == 0) then
          i2=eol
        else
          i2=i2+jkw-2
        end if
        select case(cix_flag)
        case("c")
          c_io=o_line(i1:i2)
        case("i")
          read(unit=o_line(i1:i2),fmt=*) i_io
        case("x")
          read(unit=o_line(i1:i2),fmt=*) x_io
        case default
          write(unit=6,fmt=*) "ERROR in get_one_par: wrong cix_flag => STOP"
          stop
        end select
        exit
      end if
    else
      c_io="no_occ"
      cycle
    end if
  else
    c_io="error"
    write(unit=*,fmt="(/,a,i2,2a,/,a,/)") " error reading o_line at line ",i_line," for keyword ",kwrd,trim(o_line)
    exit
  end if
end do
!
return
end subroutine get_one_par

!------------------------------------------------------------------------------
!
!>>>>>
!>
!>     GRID & ATOMS READ-OUT ROUTINES
!>
!>>>>>
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine grid_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Search for all informations concerning the grid and determines
! transformations matrices (example: g2c= from grid to cartesian
!
real(kind=double) :: d11,d12,d13,d21,d22,d23,d31,d32,d33,detg2c
!
read(unit=MIU) np1,np2,np3
read(unit=MIU) origin
read(unit=MIU) del1
read(unit=MIU) del2
read(unit=MIU) del3
!
if(iso_test) then
  r_box(1,1)=2
  r_box(2,1)=2
  r_box(3,1)=2
  r_box(1,2)=np1-2
  r_box(2,2)=np2-2
  r_box(3,2)=np3-2
else
  r_box(1,1)=1
  r_box(2,1)=1
  r_box(3,1)=1
  r_box(1,2)=np1
  r_box(2,2)=np2
  r_box(3,2)=np3
end if
!
inc1=sqrt(dot_product(del1,del1))
inc2=sqrt(dot_product(del2,del2))
inc3=sqrt(dot_product(del3,del3))
!
!  transformation matrix from grid to cartesian
!
g2c(1,1)=del1(1)
g2c(2,1)=del1(2)
g2c(3,1)=del1(3)
g2c(1,2)=del2(1)
g2c(2,2)=del2(2)
g2c(3,2)=del2(3)
g2c(1,3)=del3(1)
g2c(2,3)=del3(2)
g2c(3,3)=del3(3)
!
!  transformation matrix from cartesian to grid :c2g
!  c2gt=Transpose(c2g)=Transpose(Inv(g2c))
!
d11=del2(2)*del3(3)-del2(3)*del3(2)
d12=-del1(2)*del3(3)+del1(3)*del3(2)
d13=del1(2)*del2(3)-del1(3)*del2(2)
d21=-del2(1)*del3(3)+del2(3)*del3(1)
d22=del1(1)*del3(3)-del1(3)*del3(1)
d23=-del1(1)*del2(3)+del1(3)*del2(1)
d31=del2(1)*del3(2)-del2(2)*del3(1)
d32=-del1(1)*del3(2)+del1(2)*del3(1)
d33=del1(1)*del2(2)-del1(2)*del2(1)
!
detg2c=del1(1)*d11+del2(1)*d12+del3(1)*d13
delv=abs(detg2c)         !!Elementary grid volume  del1 * (del2 x del3)
c2gt(1,1)=d11/detg2c
c2gt(2,1)=d21/detg2c
c2gt(3,1)=d31/detg2c
c2gt(1,2)=d12/detg2c
c2gt(2,2)=d22/detg2c
c2gt(3,2)=d32/detg2c
c2gt(1,3)=d13/detg2c
c2gt(2,3)=d23/detg2c
c2gt(3,3)=d33/detg2c
!
c2g=transpose(c2gt)
!
end subroutine grid_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine rho_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! READS the total density on the grid
!
integer(kind=int32) :: i,j,k
!
read(unit=10)(((rho(i,j,k),k=1,np3),j=1,np2),i=1,np1)
!
end subroutine rho_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine rhov_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! READS the VALENCE density on the grid
!
integer(kind=int32) :: i,j,k,n1,n2,n3
!
read(unit=MIU) n1,n2,n3
if(n1 /= np1 .or. n2 /= np2 .or. n3 /= np3) then
  write(unit=MOU,fmt=fmta) " Fatal Error ... Valence Grid does not match Total Density one !!!"
  close(MOU)
  stop
else
  read(unit=MIU)
  read(unit=MIU)
  read(unit=MIU)
  read(unit=MIU)
!
  allocate(rhov(np1,np2,np3))
  read(unit=MIU)(((rhov(i,j,k),k=1,np3),j=1,np2),i=1,np1)
end if
!
end subroutine rhov_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine plane_rho()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   subroutine to initialize a plane with regular grid spacing
!   and to compute the density and its gradient on that meshgrid
!
integer :: i,j,k,ibass
integer :: imolp,iz,i1,i2,j1,j2
real(kind=double) :: la,lb,lc
real(kind=double) :: densj,lplj
real(kind=double) :: gn
real(kind=double) :: z_offset,d_12
real(kind=double), dimension(3) :: rx,rj,gradx,gradj,r_1,r_2,r_12,r_2t
real(kind=double), dimension(3) :: ta,tb,tc
real(kind=double), dimension(3,3) :: hessj
logical :: lgc_1,lgc_2,lgc_3,lgc_123,lgc_blk
character(len=6) :: lbl
!
if(index(plane_def,"interactively") /= 0) then
!!! plane specifications are not given in the integrity.in file, ask from user
  write(unit=SOU,fmt=fmta) "   # Plane definition"
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta) "      grid orientation matrix :"
  do i=1,3
    write(unit=SOU,fmt="(tr1,3f12.6)") real(np1,double)*del1(i),real(np2,double)*del2(i),real(np3,double)*del3(i)
  end do
  write(unit=SOU,fmt="(/,a,3f10.4,/)") "      grid origin : ",origin
!
!     origin atom , 2nd  and 3rd atoms defining Oxy plane
!
  write(unit=SOU,fmt=fmta) ">>> atoms list :"
  do i=1,mol_number
    write(unit=SOU,fmt="(2/,tr2,a,i2)") " molecule ",i
    k=0
    do j=1,nat
      if(mol_index(j) == i) then
        k=k+1
        write(unit=SOU,fmt="(tr1,a6)",advance="no") trim(lat(j))
        if(modulo(k,10) == 0) write(unit=SOU,fmt="(a,/)") " ..."
      end if
    end do
  end do
!
  write(unit=SOU,fmt=fmta) ""
  write(unit=SOU,fmt=fmta,advance="no") "      enter atom #1 defining plane origin  : "
  read(unit=SOU,fmt=fmta) lbl1
  write(unit=SOU,fmt=fmta,advance="no") "      enter atom #2 defining ""x"" axis from atom 1 : "
  read(unit=SOU,fmt=fmta) lbl2
  write(unit=SOU,fmt=fmta,advance="no") "      enter atom #3 defining ""xy"" plane from atoms #1 & #2 : "
  read(unit=SOU,fmt=fmta) lbl3
  write(unit=SOU,fmt=fmta,advance="no") "      move ""x"" direction to middle of atoms #2 & #3 [y/n] ? "
  read(unit=SOU,fmt=fmta) middle_23
  write(unit=SOU,fmt=fmta,advance="no") "      switch to perpendicular ""zx"" [x] or ""zy"" [y] plane ? "
  read(unit=SOU,fmt=fmta) perppl
  if(index(perppl,"y") /= 0 .or. index(perppl,"x") /= 0) then
    if(index(perppl,"y") /= 0) perpxy="YZ"
    if(index(perppl,"x") /= 0) perpxy="XZ"
    perppl="perpendicular"
  else
    perppl="parallel"
    perpxy="XY"
  end if
  dist=zero
  write(unit=SOU,fmt=fmta,advance="no") "      give dX et dY step increments : "
  read(unit=SOU,fmt=*) incx,incy
  write(unit=SOU,fmt=fmta,advance="no") "      give numbers of points in ""x"" and ""y"" : "
  read(unit=SOU,fmt=*) nx,ny
  write(unit=SOU,fmt=fmta,advance="no") "      center  plane offset on molecular origin projection [y/n] ? "
  read(unit=SOU,fmt=fmta) molor
  if(index(molor,"y") /= 0) then
    molor="molecular centring"
  else
    molor="manual offset"
    write(unit=SOU,fmt=fmta,advance="no") "      give ""x"" and ""y""  offsets : "
    read(unit=SOU,fmt=*) dxo,dyo
  end if
!
  write(unit=SOU,fmt=fmta,advance="no") "      give perpendicular ""z"" offset or increment : "
  read(unit=SOU,fmt=*) dzo
!
  if(dzo /= zero) then
    write(unit=SOU,fmt=fmta,advance="no") "      give number of ""z"" plane : "
    read(unit=SOU,fmt=*) nz
  else
    nz=1
  end if
!
  write(unit=SOU,fmt=fmta,advance="no") "      give number of angular steps for basin contour search : "
  read(unit=SOU,fmt=*) nal
end if
!
extension=".plane"
nfil=trim(file_path)//trim(filename_1)//trim(extension)
open(unit=BOU,file=nfil,status="new",action="write")
!
if(len_trim(lbl1) < 1 .or. len_trim(lbl2) < 1 .or. len_trim(lbl3) < 1) then
  lgc_blk=.true.
else
  lgc_blk=.false.
end if
lgc_1=.false.
lgc_2=.false.
lgc_3=.false.
do i=1,nat
  lbl=lat(i)
  if(lbl1 == lbl) lgc_1=.true.
  if(lbl2 == lbl) lgc_2=.true.
  if(lbl3 == lbl) lgc_3=.true.
end do
lgc_123=.not.(lgc_1 .and. lgc_2 .and. lgc_3)
if(lgc_blk .or. lgc_123) then
  lbl1="O"
  lbl2="i"
  lbl3="j"
  p1=half*(real(np1,double)*del1+real(np2,double)*del2)
  p2=real(np1,double)*del1+half*real(np2,double)*del2
  p3=half*real(np1,double)*del1+real(np2,double)*del2
else
  do i=1,nat
    lbl=lat(i)
    if(lbl1 == lbl) then
      p1(:)=rat(i,:)
      imolp=mol_index(i)
    end if
    if(lbl2 == lbl) then
      p2(:)=rat(i,:)
    end if
    if(lbl3 == lbl) then
      p3(:)=rat(i,:)
    end if
  end do
end if
!
ra1(:)=p1(:)-origin(:)
ra2(:)=p2(:)-p1(:)
ra3(:)=p3(:)-p1(:)
!
if(index(middle_23,"y") /= 0) p2(:)=half*(p2(:)+p3(:))
!
v1(:)=p2(:)-p1(:)
v2(:)=p3(:)-p1(:)
u1(:)=v1(:)/sqrt(dot_product(v1,v1))
uu(:)=u1(:)
vv(:)=v2(:)
call cross_product()
v3(:)=ww(:)
!
u3(:)=v3(:)/sqrt(dot_product(v3,v3))
uu(:)=u3(:)
vv(:)=u1(:)
call cross_product()
u2(:)=ww(:)
!
if(perppl == "perpendicular") then
  uu(:)=u1(:)
  vv(:)=u2(:)
  ww(:)=u3(:)
  if(perpxy == "YZ") then
    u1(:)=vv(:)
    u2(:)=ww(:)
    u3(:)=uu(:)
  end if
  if(perpxy == "XZ") then
    u1(:)=uu(:)
    u2(:)=ww(:)
    u3(:)=-vv(:)
  end if
end if
!
uu(1)=dot_product(ra2,u1)
uu(2)=dot_product(ra2,u2)
uu(3)=dot_product(ra2,u3)
vv(1)=dot_product(ra3,u1)
vv(2)=dot_product(ra3,u2)
vv(3)=dot_product(ra3,u3)
!
v1(:)=incx*u1(:)
v2(:)=incy*u2(:)
!!!!!
write(unit=BOU,fmt=fmta) filename_1
!!!!! output box informations
ta(:)=real(np1,double)*del1(:)
tb(:)=real(np2,double)*del2(:)
tc(:)=real(np3,double)*del3(:)
la=sqrt(dot_product(ta,ta))/real(np1,double)
lb=sqrt(dot_product(tb,tb))/real(np2,double)
lc=sqrt(dot_product(tc,tc))/real(np3,double)
write(unit=BOU,fmt="(a,3(i4,a2),a,3f8.3,a)") "___grid:  ",np1," x",np2," x",np3," x","  (",la,lb,lc,")"
write(unit=BOU,fmt="(a,3f10.5)") "___grid_origin@ ",origin(:)
write(unit=BOU,fmt=fmta)"___grid_vertices:"
write(unit=BOU,fmt="(a,3f10.5)") "   (1)  ",zero,zero,zero
write(unit=BOU,fmt="(a,3f10.5)") "   (2)  ",ta(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (3)  ",ta(:)+tb(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (4)  ",tb(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (5)  ",tb(:)+tc(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (6)  ",tc(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (7)  ",ta(:)+tc(:)
write(unit=BOU,fmt="(a,3f10.5)") "   (8)  ",ta(:)+tb(:)+tc(:)
!
write(unit=BOU,fmt="(a,i5)") "___number_of_input_atoms:",nbas
write(unit=BOU,fmt=fmta) "___atomic_positions:"
do ibass=1,nat !nbas
  iat=ibass  !iat=indx_atoms(ibass)
  write(unit=BOU,fmt="(i3,tr2,a6,tr1,3f12.6)") ibass,trim(lat(iat)),rat(iat,:)
end do
write(unit=BOU,fmt=fmta) "___bonds:"
do i1=1,nat !nbas
  i2=i1 !indx_atoms(i1)
  r_1=rat(i2,:)
  do j1=i1+1,nat !nbas
    j2=j1 !indx_atoms(j1)
    r_2=rat(j2,:)
    do i=-1,1
      ta=real(i*np1,double)*del1(:)
      do j=-1,1
        tb=real(j*np2,double)*del2(:)
        do k=-1,1
          tc=real(k*np3,double)*del3(:)
          r_2t=r_2+ta+tb+tc
          r_12=r_2t-r_1
          d_12=sqrt(dot_product(r_12,r_12))
          if(d_12 > zero .and. d_12 <= max_bond_length) then
            write(unit=BOU,fmt="(i3,a3,i3,2(tr5,3f12.6))") i1," ---",j1,r_1,r_2t
          end if
        end do
      end do
    end do
  end do
end do
write(unit=BOU,fmt=fmta) "___end_of_bonds"
!
write(unit=BOU,fmt="(a,4(a,tr3))") "___atoms_defining_plane: ",lbl1,lbl2,lbl3,perppl
write(unit=BOU,fmt="(a,2i4,2f15.7)") "___plane_mesh: ",nx,ny,incx,incy
write(unit=BOU,fmt="(a,2f15.7)") "___in_plane_offset: ",dxo,dyo
write(unit=BOU,fmt="(a,i4,f15.7)") "___out_of_plane_offsets: ",nz,dzo
write(unit=BOU,fmt="(a,i4)") "___number_of_angular_steps: ",nal
!
!!!!   loop on the "nz" planes
!
do iz=1,nz
  if(nz == 1) then
    z_offset=dzo
  else
    z_offset=real(iz-1,double)*dzo
  end if
  v3(:)=z_offset*u3(:)
  o2plane=z_offset+dot_product(ra1,u3)
  write(unit=BOU,fmt="(a,f15.7,a)") "___plane_density_@offset: ",z_offset, "  x y z rho grad(rho) Lap(rho)"
  write(unit=*,fmt="(a,i2,a)") "___plane_density@offset: ",iz-1  ,"   write density etc."
!
!      plane offset, centred on molecular origin or manually input
!
  if(index(molor ,"molecular") /= 0) then
    vv(:)=ori_mol(imolp,:)-p1(:)
    uu(:)=vv(:)- dot_product(vv,u3)*u3(:)
    udec(:)=uu(:)+v3(:)
    write(unit=*,fmt="(/,a)") "molecular centering"
    write(unit=*,fmt="(tr2,a,i2)") "imolp=",imolp
    write(unit=*,fmt="(tr2,a,3f9.4)") "ori_mol",ori_mol(imolp,:)
    write(unit=*,fmt="(tr2,a,3f9.4)") "p1=",p1(:)
    write(unit=*,fmt="(tr2,a,3f9.4)") "vv=",vv(:)
    write(unit=*,fmt="(tr2,a,3f9.4)") "uu=",uu(:)
    write(unit=*,fmt="(tr2,a,3f9.4,a,f9.4)") "udec=",udec(:)," | ",sqrt(udec(1)**2+udec(2)**2+udec(3)**2)
  else
    udec(:)=dxo*u1(:)+dyo*u2(:)+v3(:)
  end if
!     GRADIENT INTERPOLATION IN THE PLANE
!     ===================================
!
  uu(:)=udec(:)+ra1(:)-half*(real(nx,double)*v1(:)+real(ny,double)*v2(:))
  do i=0,nx
    do j=0,ny
      rx(:)=real(i,double)*v1(:)+real(j,double)*v2(:)+uu(:)
      rj=matmul(c2g,rx)
!     call interp0(rj,densj,gradj,.true.)
!     call interp(rj,densj,gradj,hessj,.true.)
      call interp(rj,densj,gradj,.true.)
      call interp(rj,densj,gradj,.true.,hessj)
      hessj=matmul(c2gt,hessj)
      hessj=matmul(hessj,c2g)
      lplj=hessj(1,1)+hessj(2,2)+hessj(3,3)
      gradx=matmul(c2gt,gradj)
      gn=sqrt(dot_product(gradx,gradx))
      write(unit=BOU,fmt="(3f12.5,3es15.5)") rx(:)+origin(:),densj,gn,lplj
    end do
  end do
end do         !! loop over iz=1 to nz planes
return
!
end subroutine plane_rho

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine plane_basin()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer :: i,j,iz
integer :: npl
real(kind=double) :: mean_xpl,mean_ypl,alpha,dal,datba,wght,sumden
real(kind=double) :: min_rpl,max_rpl,rpl,rpld,dd,rpl1,rpl2,rplo
real(kind=double) :: at2plane,r2plane
real(kind=double) :: xc,yc,z_offset
real(kind=double), dimension(3) :: aa,bb,oo,r_unit
real(kind=double), dimension(5000) :: xpl,ypl
!
!!!!   loop on the "nz" planes
!
write(unit=*,fmt="(a,a6,2(tr2,3f12.7))") "___Basin_Contour@",trim(lat(iat)),rat(iat,:),oo(:)+origin(:)
write(unit=BOU,fmt="(a,a6,2(tr2,3f12.7))") "___Basin_Contour@",trim(lat(iat)),rat(iat,:),oo(:)+origin(:)
write(unit=*,fmt=*) "nb of planes",nz
do iz=1,nz
  if(nz == 1) then
    z_offset=dzo
  else
    z_offset=real(iz-1,double)*dzo
  end if
  v3(:)=z_offset*u3(:)
  o2plane=z_offset+dot_product(ra1,u3)
!
  at2plane=dot_product(rag,u3)-o2plane
  dtheta=pi/real(nth,double)
  dphi=dpi/real(nphi,double)
  theta=zero
  xpl(1:1000)=zero
  ypl(1:1000)=zero
  npl=0
  do i=1,nth+1
    sinth=sin(theta)
    phi=zero
    v_unit(3)=cos(theta)
    do j=1,nphi+1
      v_unit(1)=sinth*cos(phi)
      v_unit(2)=sinth*sin(phi)
      aa(:)=rag(:)+rlim_thph(ibas,i,j)*v_unit(:)
      r2plane=dot_product(aa,u3)- o2plane
      if(abs(r2plane) <= tol2plane) then
        bb(:)=aa(:)-ra1(:)
        npl=npl+1
        xpl(npl)=dot_product(bb,u1)
        ypl(npl)=dot_product(bb,u2)
      end if
      phi=phi+dphi
    end do
    theta=theta+dtheta
  end do
!
  write(unit=SOU,fmt="(tr2,a,f9.5)") "Distance atom to plane : ",at2plane
  write(unit=SOU,fmt="(tr2,a,f6.2,a,i4)") "Number of points below ",tol2plane," a.u. from plane : ",npl
!
  if(npl > npcp) then
    write(unit=BOU,fmt="(a,i3)") "...plane_contour@offset: ",iz-1
    write(unit=*,fmt="(a,i3)") "...plane_contour@offset: ",iz-1
    min_rpl=100.0_double
    max_rpl=zero
    mean_xpl=zero
    mean_ypl=zero
    sumden=zero
    do i=1,npl
      bb=ra1+v3+xpl(i)*u1+ypl(i)*u2-rag
      wght=one
      mean_xpl=mean_xpl+wght*xpl(i)
      mean_ypl=mean_ypl+wght*ypl(i)
      sumden=sumden+wght
    end do
    mean_xpl=mean_xpl/sumden
    mean_ypl=mean_ypl/sumden
    bb(:)=rag(:)-ra1(:)
    xc=dot_product(bb,u1)
    yc=dot_product(bb,u2)
    oo(:)=ra1(:)+v3(:)+mean_xpl*u1(:)+mean_ypl*u2(:)
    datba=sqrt((xc-mean_xpl)**2+(yc-mean_ypl)**2)
    write(unit=*,fmt="(a,f8.3)") "  Distance atom proj. to search center = ",datba
    do i=1,npl
      rpl=sqrt((xpl(i)-mean_xpl)**2+(ypl(i)-mean_ypl)**2)
      min_rpl=min(min_rpl,rpl)
      max_rpl=max(max_rpl,rpl)
    end do
!!!
!!! ERREUR SANS DOUTE ICI
!!!
    write(unit=SOU,fmt="(tr2,a,2f12.6)") "min & max r_contour = ",min_rpl,max_rpl
!
    call init_basin_search("fine")
    dal=dpi/real(nal,double)
    alpha=zero
    rpld=half*(min_rpl+max_rpl)
    do i=1,nal+1
      r_unit=cos(alpha)*u1(:)+sin(alpha)*u2(:)
      aa(:)=oo(:)+rpld*r_unit(:)-rag(:)
      rplo=sqrt(dot_product(aa,aa))
      v_unit(:)=aa(:)/rplo
      call inout_basin_check(rplo)
      if(inout_basin) then
        rpl1=rpld
        rpl2=max_dist*1.5_double   !!!!!modif pour etre sur d'avoir point a l'interieur!!!
      else
        rpl1=zero
        rpl2=rpld
      end if
!
      do
        rpld=half*(rpl1+rpl2)
        dd=rpl2-rpl1
        if(dd < dtol) then
          rpl=rpld
          exit
        else
          aa(:)=oo(:)+rpld*r_unit(:)-rag(:)
          rplo=sqrt(dot_product(aa,aa))
          v_unit(:)=aa(:)/rplo
          call inout_basin_check(rplo)
          if(inout_basin) rpl1=rpld
          if(.not.inout_basin) rpl2=rpld
        end if
      end do
      bb(:)=oo(:)+rpl*r_unit(:)
      write(unit=BOU,fmt="(3f12.7,tr2,2f12.7)") bb(:)+origin(:),mean_xpl+rpl*cos(alpha),mean_ypl+rpl*sin(alpha)
      alpha=alpha+dal
    end do
  end if
end do  !! loop over iz=1 to "nz" planes
!
end subroutine plane_basin

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine cross_product()
!
ww(1)=uu(2)*vv(3)-uu(3)*vv(2)
ww(2)=uu(3)*vv(1)-uu(1)*vv(3)
ww(3)=uu(1)*vv(2)-uu(2)*vv(1)
!
end subroutine cross_product

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine struc_initialize()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Reads atoms' number, allocate concerned arrays and calls GET_ATOMS routine
!
integer(kind=int32) :: ieob,ikw
character(len=256) :: linecar,kword
!
kword="ATOM"
!
rewind(unit=MIU)
read(unit=MIU,fmt=*) filename_1 !! generic name of density data-files
read(unit=MIU,fmt=*) lunit      !! coef. to transform read coordinates to a.u.
rewind(unit=MIU)
ieob=0
nat=0
! check number of atoms
do
  if(ieob /= 0) exit
  read(unit=MIU,fmt=fmta) linecar
  ieob=index(linecar,"!EOB")
  if(index(linecar,trim(KWORD)) /= 0) nat=nat+1
end do
write(unit=SOU,fmt="(a,i3)") "        number of atoms =",nat
if(nat == 0) then
  open(unit=TOU,file=trim(file_path)//"InteGriTy.error",status="new",action="write")
  write(unit=TOU,fmt=*) trim(kword)," not found, program stopped"
  close(TOU)
  stop
end if
!
! allocate arrays concerning atoms" properties
allocate(lat(nat))
allocate(rat(nat,3))
allocate(mol_index(nat))
allocate(nb_e_atom(nat))
allocate(nb_eval_atom(nat))
allocate(mult_factor(nat))
allocate(rstart_atom(nat))
allocate(rmin_atom(nat))
allocate(atom_size(nat))
allocate(atom_color(nat,3))
!
! call routine to read effectively atomic properties
call get_atoms()
do ikw=1,nat
  rat(ikw,:)=rat(ikw,:)*lunit
end do
!
end subroutine struc_initialize

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine get_atoms()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! read ATOM PROPERTIES
!
integer(kind=int32) :: i1,i2,i,j
character(len=256) :: linecar
logical :: check_name,check_pos,check_mult,check_rmin,check_mol
logical :: check_size,check_color
!
!     number of atoms "nat" known from "STRUC_INITIALIZE" subroutine
!     unit=MIU corresponds to "data.in" file
!
rewind(unit=MIU)
i=0
do
  read(unit=MIU,fmt=fmta) linecar
  if(index(linecar,"!EOB") /= 0 .or. i > nat) then
    exit
  else
    if(index(linecar,"!ATOM") /= 0) then
      i=i+1
      check_name=.false.
      check_pos=.false.
      check_mult=.false.
      check_rmin=.false.
      check_mol=.false.
      check_size=.false.
      check_color=.false.
! maximum number of lines := parameter (dflt=4) in atom_module
      do j=1,max_line
! check name of atom
        if(.not. check_name) then
          i1=index(linecar,"NAME=")
          if(i1 /= 0) then
            check_name=.true.
            i1=i1+len_trim("NAME=")+1
            i2=i1+index(linecar(i1:)," ")-3
            lat(i)=linecar(i1:i2)
          else
            if(j == max_line .or. index(linecar,"!ATOM") /= 0 .or. index(linecar,"!EOB") /= 0) then
              open(unit=TOU,file=trim(file_path)//"InteGriTy.error",status="new",action="write")
              write(unit=TOU,fmt=*) "NAME= not found after ATOM, program stopped"
              close(TOU)
              stop
            end if
          end if
        end if
! check position of atom
        if(.not. check_pos) then
          i1=index(linecar,"R=")
          if(i1 /= 0) then
            check_pos=.true.
            i1=i1+len_trim("R=")
            read(unit=linecar(i1:),fmt=*) rat(i,:)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
              open(unit=TOU,file=trim(file_path)//"InteGriTy.error",status="new",action="write")
              write(unit=TOU,fmt=*) "R= not found after ATOM, program stopped"
              close(TOU)
              stop
            end if
          end if
        end if
! check multiplicity of atom
        if(.not. check_mult) then
          i1=index(linecar,"MULT=")
          if(i1 /= 0) then
            check_mult=.true.
            i1=i1+len_trim("MULT=")
            read(unit=linecar(i1:),fmt=*) mult_factor(i)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
! if not specified in input file, default multiplicity := 1
              mult_factor(i)=one
              check_mult=.true.
            end if
          end if
        end if
! check minimum atomic basin radius
        if(.not. check_rmin) then
          i1=index(linecar,"RMIN=")
          if(i1 /= 0) then
            check_rmin=.true.
            i1=i1+len_trim("RMIN=")
            read(unit=linecar(i1:),fmt=*) rmin_atom(i)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
! if not specified in input file, rmin_atom read from "ATOM_PAR.inp"
              rmin_atom(i)=zero
              check_rmin=.true.
            end if
          end if
        end if
!
! check molecular indices
        if(.not. check_mol) then
          i1=index(linecar,"MOL=")
          if(i1 /= 0) then
            check_mol=.true.
            i1=i1+len_trim("MOL=")
            read(unit=linecar(i1:),fmt=*) mol_index(i)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
! if not specified in input file, default mol_index := 1
              mol_index(i)=1
              check_mol=.true.
            end if
          end if
        end if
!
! check atomic size
        if(.not. check_size) then
          i1=index(linecar,"SIZE=")
          if(i1 /= 0) then
            check_size=.true.
            i1=i1+len_trim("SIZE=")
            read(linecar(i1:),"(f3.2)") atom_size(i)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
! if not specified in input file, default atom_size := 1.
              atom_size(i)=zero
              check_size=.true.
            end if
          end if
        end if
!
!check atom color
        if(.not. check_color) then
          i1=index(linecar,"COLOR=")
          if(i1 /= 0) then
            check_color=.true.
            i1=i1+len_trim("COLOR=")
            write(unit=*,fmt=*) linecar(i1:)
            read(unit=linecar(i1:),fmt=*) atom_color(i,:)
            write(unit=*,fmt=*) atom_color(i,:)
          else
            if(j == max_line .or. index(linecar,"!END") /= 0 .or. index(linecar,"!EOB") /= 0) then
! if not specified in input file, default atom_color := 0.5,0.5,0.5  (grey)
              atom_color(i,:)=-one
              check_color=.true.
            end if
          end if
        end if
!
! exit "j" loop before max_line if all parameters found
        if(check_name.and.check_pos.and.check_mult.and.check_rmin.and.check_mol) then
          exit
        else
          if(index(linecar,"!EOB") == 0) then
            read(unit=MIU,fmt=fmta) linecar
          else
            open(unit=TOU,file=trim(file_path)//"InteGriTy.error",status="new",action="write")
            write(unit=TOU,fmt=*) "!EOB found and still atomic parameter missing !!!"
            close(TOU)
            stop
          end if
        end if
      end do
    end if
  end if
end do
return
end subroutine get_atoms

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine get_atom_par()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   default atomic parameters such as number of total & valence electrons,
!   starting and default minimum atomic basins' radii are stored in the
!   "ATOM_PAR.inp" file. Are used only if not individually stated in the
!   "data.in" file
!
integer(kind=int32) :: nbe,nbeval       !! number of total and valence electrons
integer(kind=int32) :: nspec,ispec,jat  !! dummy variables
integer(kind=int32) :: i_err            !! iostat value
real(kind=double) :: rstart,rmin !! starting and minimum atomic basin's radii
real(kind=double) :: size_at	 !! size for Dx
real(kind=double), dimension(3) :: color_at         !!color for Dx
character(len=2) :: spec         !! atom species
character(len=6) :: atom         !! name of considered atom
logical :: test_at
!
character(len=*), parameter :: fmt0="(tr1,a2,2i4,6f5.1)"
!
open(unit=TIU,file="ATOM_PAR.inp",status="old",action="read",position="rewind")
nspec=0
do
  read(unit=TIU,fmt=*,iostat=i_err)
  if(i_err == -1) exit
  nspec=nspec+1
end do
rewind(unit=TIU)
do jat=1,nat
  nb_e_atom(jat)=0
  nb_eval_atom(jat)=0
  rstart_atom(jat)=two !!!!!!!!!!!!modify default value
end do
!
!!! ici si nspec == 0 mettre un message d'erreur ...
do ispec=1,nspec
  read(unit=TIU,fmt=fmt0) spec,nbe,nbeval,rstart,rmin,size_at,color_at
  do jat=1,nat
    atom=lat(jat)
    call valid_atom(atom(1:2),test_at)
    if(test_at) then
      if(atom(1:2) == spec) then
        nb_e_atom(jat)=nbe
        nb_eval_atom(jat)=nbeval
        rstart_atom(jat)=rstart
        if(atom_size(jat) == zero) atom_size(jat)=size_at
        if(atom_color(jat,1) == -one) atom_color(jat,:)=color_at(:)
        if(rmin_atom(jat) == zero) rmin_atom(jat)=rmin
      end if
    end if
  end do
end do
close(TIU)
!
end subroutine get_atom_par

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine valid_atom(at_name,at_flag)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
character(len=*), intent(in) :: at_name
logical, intent(out) :: at_flag
!
integer(kind=int32) :: i
!
at_flag=.true.
do i=1,9
  if(index(at_name(1:2),trim(at_check_list(i))) /= 0) then
    at_flag=.false.
    return
  end if
end do
!
return
end subroutine valid_atom

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine cell_integration()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!     computation of direct integration over the unit cell
!     either by discrete summation based on elementary volume "delv"
!     either using integral expression of tricubic interpolation
!
integer :: i,j,l,k,m,n,ll,mm,nn,i1,i2,j1,j2,k1,k2
real(kind=double) :: c_lmn
real(kind=double), dimension(4) :: cc
!
!      initialization of coefficients for integration
!      from tricubic interpolation on regular grid
!
cc(1)=-one/24.0_double
cc(2)=13.0_double/24.0_double
cc(3)=13.0_double/24.0_double
cc(4)=-one/24.0_double
!
tot_grid_charge=zero
val_grid_charge=zero
tot_interp_charge=zero
val_interp_charge=zero
!
i1=r_box(1,1)
j1=r_box(2,1)
k1=r_box(3,1)
i2=r_box(1,2)
j2=r_box(2,2)
k2=r_box(3,2)
!
do i=i1,i2
  do j=j1,j2
    do k=k1,k2
!       direct integration from regular grid
      tot_grid_charge=tot_grid_charge+rho(i,j,k)
      val_grid_charge=val_grid_charge+rhov(i,j,k)
!       integration from tricubic interpolation on regular grid
      do l=1,4
        ll=(l-2)+i
        ll=modulo(ll,np1)
        if(ll <= 0) ll=ll+np1
        do m=1,4
          mm=(m-2)+j
          mm=modulo(mm,np2)
          if(mm <= 0) mm=mm+np2
          do n=1,4
            nn=(n-2)+k
            nn=modulo(nn,np3)
            if(nn <= 0) nn=nn+np3
            c_lmn=cc(l)*cc(m)*cc(n)
            tot_interp_charge=tot_interp_charge+rho(ll,mm,nn)*c_lmn
            val_interp_charge=val_interp_charge+rhov(ll,mm,nn)*c_lmn
          end do
        end do
      end do
    end do
  end do
end do
tot_grid_charge=tot_grid_charge*delv
val_grid_charge=val_grid_charge*delv
tot_interp_charge=tot_interp_charge*delv
val_interp_charge=val_interp_charge*delv
grid_volume=delv*real(i2-i1+1,double)*real(j2-j1+1,double)*real(k2-k1+1,double)
!
end subroutine cell_integration

!>>>>>
!>
!>     ATOMIC BASINS" SEARCH ROUTINES
!>
!>>>>>
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine inout_box_check(r)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!         check if the running point is inside the box in case of isolated system
!
!           r :: running point in grid coordinates
!           out_box :: (in grid_module) = .true. if r out of box
!           r_box :: (in grid_module) define box boundaries
!                          in case of non periodicity the box is defined
!                          in the direction "i" by :
!                               r_box(i,1) = 2   &  r_box(i,2) = np_i - 2
!
real(kind=double), dimension(3), intent(in) :: r
!
integer(kind=int32) :: i
!
out_box=.false.
do i=1,3
  if(r(i) < r_box(i,1) .or. r(i) > r_box(i,2)) then
    out_box=.true.
    return
  end if
end do
!
end subroutine inout_box_check

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine inout_basin_check(r0)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
real(kind=double), intent(in) :: r0   !!! radial position at theta,phi
!
integer(kind=int32) :: jd,jf
real(kind=double) :: modgrad        !!! Gradient modulus
real(kind=double) :: densg          !!! Interpolated density in GC
real(kind=double) :: dbefore,dafter !!! distance from nucleus to running point
real(kind=double) :: fa,cos_dg
real(kind=double), dimension(3) :: ri !!! running position in cartesian coordinates (CC)
real(kind=double), dimension(3) :: rg !!! running position in grid coordinates (GC)
real(kind=double), dimension(3) :: r,d
real(kind=double), dimension(3) :: gradg  !!! Interpolated gradient in CC
real(kind=double), dimension(3) :: grad   !!! Interpolated gradient in CC
!
! transforms spherical (r0,theta,phi) to cartesian coordinates, then to gc
ri(:)=rag(:)+r0*v_unit(:)
rg(:)=matmul(c2g,ri(:))
if(iso_test) then
  call inout_box_check(rg)
  if(out_box) then
    inout_basin=.false.
    return
  end if
end if
! interpolates rho and grad(rho) at running position, then transforms back to cc
call interp(rg,densg,gradg,.true.)
!
if(vdv_test .and. densg < rho_vdv) then
  inout_basin=.false.
  return
end if
!
grad=matmul(c2gt,gradg)
modgrad=sqrt(dot_product(grad,grad))
!
! case where gradient modulus is zero
! a bond critical point is assumed, and belongs to basin ...
if(modgrad < ModGradMin) then
  write(unit=MIU,fmt=fmta) "WARNING! ... modgrad < ModGradMin"
  inout_basin=.true.
  return
else
  r(:)=ri(:)
  d(:)=r(:)-rag(:)
  dafter=sqrt(dot_product(d,d))
!     if coarse search, performs statistics
  if(cofi) then
    min_grad=min(min_grad,modgrad)
    max_grad=max(max_grad,modgrad)
    min_dist=min(min_dist,dafter)
    max_dist=max(max_dist,dafter)
    min_rho=min(min_rho,densg)
    max_rho=max(max_rho,densg)
  end if
  jd=0
  jf=0
  do
    fa=one/dafter
    d_unit(:)=d(:)*fa
    if(dafter < dmin) then
      inout_basin=.true.
      exit
    else
      dbefore=dafter
      rg(:)=matmul(c2g,r(:))
      if(iso_test) then
        call inout_box_check(rg)
        if(out_box) then
          if(jd > min_step_dwd) then
            inout_basin=.true.
          else
            inout_basin=.false.
          end if
          exit
        end if
      end if
      call interp(rg,densg,gradg,.true.)
      grad=matmul(c2gt,gradg)
      modgrad=sqrt(dot_product(grad,grad))
      fa=one/modgrad
      g_unit(:)=grad(:)*fa
! computes accelerating factor for increment along gradient path downward nucleus
      cos_dg=abs(dot_product(g_unit,d_unit))
      fa=exp(cos_dg**exp_dg*lnfa)
      r(:)=r(:)+g_unit(:)*fa*dr
      d(:)=r(:)-rag(:)
      dafter=sqrt(dot_product(d,d))
      if(dafter < dbefore) then
! keep on following gradient path downward attractor
        jd=jd+1
        cycle
      else
! run away from attractor, exit after several steps !!!
        jf=jf+1
!!
!!  here add test for neighbouring atoms
!!  i.e. if close enough to a neighbouring atom then inout = .false. and exit
!!
        if((vdv_test .and. densg < rho_vdv) .or. jf > max_step_fwd) then
          inout_basin=.false.
          exit
        end if
      end if
    end if
  end do
end if
return
end subroutine inout_basin_check

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine basin_coarse_bound()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!    looks for coarse limits [rinf, rsup] of basin radius at theta & phi
!
!    # if starting point is inside basin, looks "forward" to higher radial values
!    # "      "      "   "  outside   "      "   "backward"
!
real(kind=double) :: rr     !! running position to be in the interval [rinf, rsup] at the end
real(kind=double) :: r1     !! dummy
real(kind=double) :: e      !! forward or backward coefficient
logical :: lback  !! flag forward / backward search
!
!      initializes running point to starting radius (from ATOM_PAR.inp file or former search)
!      check if it belongs or not to basin and set flag and coef. for forward/backward search
!
rr=r_start
call inout_basin_check(rr)
if(inout_basin) then
  e=ea
  lback=.false.
else
  e=-ed
  lback=.true.
end if
!
!      perform coarse searching, after each step rr is replaced by (1+e)*rr
!      when either lower or upper bound is found, min and max of (r1,rr)
!      are stored in rinf and rsup
!
do
  r1=rr
  rr=(one+e)*rr
  call inout_basin_check(rr)
  if(lback) then
    if(inout_basin) exit
  else
    if(.not. inout_basin) exit
  end if
end do
rinf=min(r1,rr)
rsup=max(r1,rr)
return
end subroutine basin_coarse_bound

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine basin_fine_bound()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   performs dichotomy refinement of basin radius at theta & phi
!   starts from "rinf" and "rsup" values and ends when
!   distance d = rsup - rinf is below tolerance "dtol"
!
real(kind=double) :: d,rra,rrb,rrr
!
rrb=rsup
rra=rinf
do
  rrr=half*(rrb+rra)
  d=rrb-rra
  if(d < dtol) then
    rlim=rrr
    exit
  else
    call inout_basin_check(rrr)
    if(inout_basin) rra=rrr
    if(.not.inout_basin) rrb=rrr
  end if
end do
end subroutine basin_fine_bound

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine rlim_and_dmin_adjust(lw)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!    When coarse search has been done for one atomic basin
!    the minimum found radius is set as "dmin" for fine radial
!    search during integration process (saving time).
!    If then basin radius is found equal to this minimum value,
!    the "dmin" value is reset to the one fixed by either
!    "ATOM_PAR.in" file or "data.in" file, the basin radius
!    at running theta & phi is re-computed and set as new "dmin"
!
logical, intent(out) :: lw !! return flag ".true." if rlim <= dmin is persistent
!
lw=.false.
call basin_coarse_bound()
call basin_fine_bound()
if(out_box .and. boundary_check) boundary_check=.false.
if(rlim <= dmin) then
  dmin=rmin_atom(iat)
  rlim_min=min(rlim,rlim_min)
  call basin_coarse_bound()
  call basin_fine_bound()
  if(out_box .and. boundary_check) boundary_check=.false.
  if(rlim <= dmin) then
    lw=.true.
  end if
  dmin=rlim
end if
return
!
end subroutine rlim_and_dmin_adjust

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine basin_theta_phi_loops(r2)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!   theta,phi loops to compute basin boundary
!
real(kind=double), intent(inout) :: r2
!
integer :: i,j
!
theta=zero
do i=1,nth+1
  sinth=sin(theta)
  phi=zero
  v_unit(3)=cos(theta)
  do j=1,nphi+1
    v_unit(1)=sinth*cos(phi)
    v_unit(2)=sinth*sin(phi)
    call basin_coarse_bound()
    call basin_fine_bound()
    if(out_box .and. boundary_check) boundary_check=.false.
!
    rlim_min=min(rlim_min,rlim)
    rlim_max=max(rlim_max,rlim)
! output basin boundary properties for comparisons & graphics
    rlim_thph(ibas,i,j)=rlim
    atom_surface=atom_surface+rlim*rlim*sinth
    rlim_mean=rlim_mean+rlim
    r2=r2+rlim*rlim
    phi=phi+dphi
  end do
  theta=theta+dtheta
end do
!
end subroutine basin_theta_phi_loops

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine init_basin_search(toto)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! initializes basin search parameters to "coarse" or "fine" search
!
character(len=*), intent(in) :: toto  !!dummy
!
if(index(toto,"coarse") /= 0) then
  ea=ea_coarse
  ed=ed_coarse
  lnfa=log(fa_coarse)
  dr=dr_coarse
  dtol=dtol_coarse
  cofi=.true.
else
  ea=ea_fine
  ed=ed_fine
  lnfa=log(fa_fine)
  dr=dr_fine
  dtol=dtol_fine
  cofi=.false.
end if
return
!
end subroutine init_basin_search

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine rlim_interp()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!   Linear interpolation of basin limit knowing coarse rlim(theta,phi)
!
integer(kind=int32) :: it,jp
real(kind=double) :: A,B,C,D,t,p,dt,dp
!
t=real(nth,double)*theta/pi
p=real(nphi,double)*phi/dpi
it=floor(t)
jp=floor(p)
dt=t-real(it,double)
dp=p-real(jp,double)
it=it+1
jp=jp+1
C=rlim_thph(ibas,it,jp)
A=rlim_thph(ibas,it+1,jp)-C
B=rlim_thph(ibas,it,jp+1)-C
D=rlim_thph(ibas,it+1,jp+1)-(A+B+C)
r_start=C+A*dt+B*dp+D*dt*dp
return
!
end subroutine rlim_interp

!>>>>>
!>
!>     GRID & ROMBERG INTEGRATION and ASSOCIATED ROUTINES
!>
!>>>>>
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine ini_convergence_test()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  initializes convergence criteria for romberg procedure
!
integer(kind=int32) :: i
logical :: lgcla
!
conv_test(:)=.false.
conv_string=" "
if(index(convergence_test,"all") /= 0) then
  conv_test(:)=.true.
  conv_string=" volume - surface - q-total - q-valence - laplacian - dipole - quadrupole - octupole"
else
  if(index(convergence_test,"vol") /= 0) then
    conv_test(1)=.true.
    conv_string=trim(conv_string)//" volume -"
  end if
  if(index(convergence_test,"sur") /= 0) then
    conv_test(2)=.true.
    conv_string=trim(conv_string)//" surface -"
  end if
  if(index(convergence_test,"tot") /= 0) then
    conv_test(3)=.true.
    conv_string=trim(conv_string)//" q-total -"
  end if
  if(index(convergence_test,"val") /= 0) then
    conv_test(4)=.true.
    conv_string=trim(conv_string)//" q-valence -"
  end if
  if(index(convergence_test,"lap") /= 0) then
    conv_test(5)=.true.
    conv_string=trim(conv_string)//" laplacian -"
  end if
  if(index(convergence_test,"dip") /= 0) then
    conv_test(6:8)=.true.
    conv_string=trim(conv_string)//" dipole -"
  end if
  if(index(convergence_test,"qua") /= 0) then
    conv_test(9:14)=.true.
    conv_string=trim(conv_string)//" quadrupole -"
  end if
  if(index(convergence_test,"oct") /= 0) then
    conv_test(15:24)=.true.
    conv_string=trim(conv_string)//" octupole -"
  end if
end if
!
lgcla=.false.
do i=1,7
  lgcla=lgcla .or. conv_test(i)
end do
if(.not. lgcla) then
  conv_test(4)=.true.
  conv_string=" q-valence by default"
end if
return
end subroutine ini_convergence_test

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine ini_convergence_test_0()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  initializes convergence criteria for romberg procedure
!
logical :: lgcla
!
lgcla=index(convergence_test,"all") /= 0
!
if(index(convergence_test,"tot") /= 0 .or. lgcla) then
  conv_tot=.true.
  yn_tot="yes"
else
  conv_tot=.false.
  yn_tot="no"
end if
if(index(convergence_test,"val") /= 0 .or. lgcla) then
  conv_val=.true.
  yn_val="yes"
else
  conv_val=.false.
  yn_val="no"
end if
if(index(convergence_test,"vol") /= 0 .or. lgcla) then
  conv_vol=.true.
  yn_vol="yes"
else
  conv_vol=.false.
  yn_vol="no"
end if
if(index(convergence_test,"lap") /= 0 .or. lgcla) then
  conv_lap=.true.
  yn_lap="yes"
else
  conv_lap=.false.
  yn_lap="no"
end if
!
if(conv_tot .or. conv_val .or. conv_lap) then
  conv_rad=.false.
  dflt_val=" "
else
  conv_rad=.true.
  dflt_val="but used by default for radial integration "
end if
return
end subroutine ini_convergence_test_0

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine polint(xa,ya0,nvar,npol,y,dy)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  polynomial interpolation of order "n" to estimate
!  integration value at each step of romberg procedure
!
integer(kind=int32), intent(in) :: nvar,npol
real(kind=double), dimension(nvar), intent(out) :: y,dy
real(kind=double), dimension(npol), intent(in) :: xa
real(kind=double), dimension(nvar,npol), intent(in) :: ya0
!
integer(kind=int32) :: i,ivar,m,ns
real(kind=double) :: den,dif,dift,ho,hp,w
real(kind=double), dimension(nmax) :: c,d
real(kind=double), dimension(npol) :: ya
!
do ivar=1,nvar
  ya(:)=ya0(ivar,:)
  ns=1
  dif=abs(xa(1))
  do i=1,npol
    dift=abs(xa(i))
    if(dift < dif) then
      ns=i
      dif=dift
    end if
    c(i)=ya(i)
    d(i)=ya(i)
  end do
  y(ivar)=ya(ns)
  ns=ns-1
  do m=1,npol-1
    do i=1,npol-m
      ho=xa(i)
      hp=xa(i+m)
      w=c(i+1)-d(i)
      den=ho-hp
      if(den == zero) then
        write(unit=SOU,fmt=fmta) " Fatal error ... failure in polint; program stopped !"
        stop
      end if
      den=w/den
      d(i)=hp*den
      c(i)=ho*den
    end do
    if(2*ns <= npol-m) then
      dy(ivar)=c(ns+1)
    else
      dy(ivar)=d(ns)
      ns=ns-1
    end if
    y(ivar)=y(ivar)+dy(ivar)
  end do
end do
return
end subroutine polint

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine trapz_rho(s_r,n_r)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  trapezoidal integration at stage "n"  cf. "NUMERICAL RECIPIES"
!
!  radial integrated quantities from 0 to r_lim(theta,phi) :
!  s_r(1:2) ::  r^2 * rho(r)  total & valence
!  s_r(3) ::  r^2 * Lap(r)  laplacian
!  s_r(4) ::  r^3 * rho(r)  dipole
!  s_r(5) ::  r^4 * rho(r)  quadrupole
!  s_r(6) ::  r^5 * rho(r)  octupole
!
!  at theta & phi & known rlim(theta,phi)
!
!  v_unit :: unitary vector in direction theta,phi
!  rag :: atomic position in cartesian grid coordinates system
!
integer(kind=int32), intent(in) :: n_r
real(kind=double), dimension(6,jmaxp), intent(out) :: s_r
!
integer(kind=int32) :: it,i,j
real(kind=double) :: r,r2,rs2,fbt,fbv,fr2
real(kind=double) :: tnm_r
real(kind=double) :: lpl_r
real(kind=double), dimension(3) :: ri,rg,gr
real(kind=double), dimension(6) :: sum_r
real(kind=double), dimension(3,3) :: hgr,hh,hsx
!
character(len=*), parameter :: fmt0="(f10.5,2es15.5,tr2,es13.4)"
!
if(n_r == 1) then
!!! if "manual" "radial" running mode then output result in a file
  if(rh_study) then
    r=0
    ri(:)=r*v_unit(:)+rag(:)
    rg(:)=matmul(c2g,ri(:))
    if(tot_use) then
      call interp(rg,fbt,gr,.true.,hgr)
      call interp(rg,fbv,gr,.false.)
    else
      call interp(rg,fbt,gr,.true.)
      call interp(rg,fbv,gr,.false.,hgr)
    end if
    hh=matmul(c2gt,hgr)
    hsx=matmul(hh,c2g)
    lpl_r=hsx(1,1)+hsx(2,2)+hsx(3,3)
    write(unit=R4U,fmt=fmt0) r,fbt,fbv,lpl_r
  end if
  r2=rlim*rlim
  ri(:)=rlim*v_unit(:)+rag(:)
  rg(:)=matmul(c2g,ri(:))
  if(tot_use) then
    call interp(rg,fbt,gr,.true.,hgr)
    call interp(rg,fbv,gr,.false.)
  else
    call interp(rg,fbt,gr,.true.)
    call interp(rg,fbv,gr,.false.,hgr)
  end if
  hh=matmul(c2gt,hgr)
  hsx=matmul(hh,c2g)
  lpl_r=hsx(1,1)+hsx(2,2)+hsx(3,3)
  if(rh_study) write(unit=R4U,fmt=fmt0) rlim,fbt,fbv,lpl_r
!
  rs2=half*rlim*r2
  s_r(1,1)=rs2*fbt
  s_r(2,1)=rs2*fbv
  s_r(3,1)=rs2*lpl_r
  if(tot_use) then
    s_r(4,1)=s_r(1,1)*rlim
    s_r(5,1)=s_r(1,1)*r2
    s_r(6,1)=s_r(1,1)*r2*rlim
  else
    s_r(4,1)=s_r(2,1)*rlim
    s_r(5,1)=s_r(2,1)*r2
    s_r(6,1)=s_r(2,1)*r2*rlim
  end if
else
  it=2**(n_r-2)
  tnm_r=real(it,double)
  delr=rlim/tnm_r
  r=half*delr
  sum_r(1:6)=zero
  do j=1,it
    r2=r*r
    ri(:)=r*v_unit(:)+rag(:)
    rg(:)=matmul(c2g,ri(:))
    if(tot_use) then
      call interp(rg,fbt,gr,.true.,hgr)
      call interp(rg,fbv,gr,.false.)
    else
      call interp(rg,fbt,gr,.true.)
      call interp(rg,fbv,gr,.false.,hgr)
    end if
    hh=matmul(c2gt,hgr)
    hsx=matmul(hh,c2g)
    lpl_r=hsx(1,1)+hsx(2,2)+hsx(3,3)
!!! if "manual" "radial" running mode then output result in a file
    if(rh_study) write(unit=R4U,fmt=fmt0) r,fbt,fbv,lpl_r
!
    sum_r(1)=sum_r(1)+fbt*r2
    sum_r(2)=sum_r(2)+fbv*r2
    sum_r(3)=sum_r(3)+lpl_r*r2
    if(tot_use) then
      fr2=fbt*r2
    else
      fr2=fbv*r2
    end if
    sum_r(4)=sum_r(4)+fr2*r
    sum_r(5)=sum_r(5)+fr2*r2
    sum_r(6)=sum_r(6)+fr2*r2*r
    r=r+delr
  end do
  do i=1,6
    s_r(i,n_r)=half*(s_r(i,n_r)+delr*sum_r(i))
  end do
end if
return
end subroutine trapz_rho

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine qromb_rho()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! performs romberg procedure for radial integration
! cf. "NUMERICAL RECIPIES"
!
integer(kind=int32) :: i,ir0,j,jr,nd,nu
real(kind=double), dimension(jmaxp) :: h_r
real(kind=double), dimension(6,jmaxp) :: sq_r
logical :: lgr
character(len=3) :: count_ext
!
h_r(1)=one
if(rh_study) then
  nd=count_radial/10
  nu=count_radial-10*nd
  count_ext="_"//char(48+nd)//char(48+nu)
  open(unit=R1U,file=trim(file_path)//trim(lat(iat))//count_ext//".radial.tot",status="new",action="write")
  write(unit=R1U,fmt=*) trim(lat(iat)),theta,phi
  open(unit=R2U,file=trim(file_path)//trim(lat(iat))//count_ext//".radial.val",status="new",action="write")
  write(unit=R2U,fmt=*) trim(lat(iat)),theta,phi
  open(unit=R3U,file=trim(file_path)//trim(lat(iat))//count_ext//".radial.lap",status="new",action="write")
  write(unit=R3U,fmt=*) trim(lat(iat)),theta,phi
  open(unit=R4U,file=trim(file_path)//trim(lat(iat))//count_ext//".radial.rho",status="new",action="write")
  write(unit=R4U,fmt=*) trim(lat(iat)),theta,phi
end if
sq_r(1:6,1:jmaxp)=zero
!
do j=1,jmax
  call trapz_rho(sq_r,j)
  if(j >= kint) then
    jr=j-km1
    call polint(h_r(jr:j),sq_r(:,jr:j),6,kmax,rad_prop,rad_dprop)
    if(rh_study) then
      write(unit=R1U,fmt=*) j,sq_r(1,j),rad_prop(1)
      write(unit=R2U,fmt=*) j,sq_r(2,j),rad_prop(2)
      write(unit=R3U,fmt=*) j,sq_r(3,j),rad_prop(3)
    end if
  else
    if(rh_study) then
      write(unit=R1U,fmt=*) j,sq_r(1,j),sq_r(1,j)
      write(unit=R2U,fmt=*) j,sq_r(2,j),sq_r(2,j)
      write(unit=R3U,fmt=*) j,sq_r(3,j),sq_r(3,j)
    end if
  end if
!
  lgr=.true.
  do i=1,6
    select case(i)
    case(5)
      ir0=4
    case(6)
      ir0=9
    case default
      ir0=2
    end select
    if(conv_test(i+ir0)) then
      if(j >= kint) then
        lgr=lgr .and. abs(rad_dprop(i)) <= eps_r*abs(rad_prop(i))
      else
        lgr=.false.
        exit
      end if
    end if
  end do
!
  if(lgr) then
    if(romb_stat) then
      nloop_r_min=min(nloop_r_min,j)
      nloop_r_max=max(nloop_r_max,j)
      nloop_tot=nloop_tot+1+2**(j-1)
      if(phi == zero) nloop_tot=nloop_tot+1+2**(j-1)
    end if
    if(rh_study) then
      close(R1U)
      close(R2U)
      close(R3U)
      close(R4U)
    end if
    return
  end if
!
  do i=1,6
    sq_r(i,j+1)=sq_r(i,j)
  end do
  h_r(j+1)=0.25_double*h_r(j)
end do
write(unit=SOU,fmt="(a,i3,a)") "WARNING ::   convergence not reached after ",jmax," cycles !"
write(unit=SOU,fmt="(2(a,f8.2))") "    in qromb_rho at THETA = ",theta*deg," and PHI = ",phi*deg
end subroutine qromb_rho

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine trapz_phi(s_phi,n_phi)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  trapezoidal integration at order "n"
!
!  integrated quantities with "phi" from 0 to 2 pi :
!  s_phi(1) ::   volume
!  s_phi(2) ::   surface
!  s_phi(3:4) ::   TOTAL & VALENCE
!  s_phi(5) ::   laplacian
!  s_phi(6:8) ::   dipole
!  s_phi(9:14) ::   quadrupole
!  s_phi(15:24) ::   octupole
!
!  at theta & over phi & known rlim(theta,phi)
!
!  v_unit :: unitary vector in direction theta,phi
!  rag :: atomic position in cartesian grid coordinates system
!
integer(kind=int32), intent(in) :: n_phi
real(kind=double), dimension(24,jmaxp), intent(out) :: s_phi
!
integer(kind=int32) :: it,i,j
real(kind=double) :: tnm_phi,csp,snp,x,y,z,w
real(kind=double) :: rlim2
real(kind=double), dimension(24) :: sum_phi
logical :: lwrng   !! return flag ".true." if rlim <= dmin is persistent
!
if(n_phi == 1) then
  phi=zero
  v_unit(1)=sinth
  v_unit(2)=zero
  call rlim_interp()
  call rlim_and_dmin_adjust(lwrng)
  if(lwrng) write(unit=SOU,fmt="(a,f8.3,2a)") "WARNING !!!   rmin=",dmin," too high for atom ",lat(iat)
  rlim_min=min(rlim,rlim_min)
  rlim_max=max(rlim,rlim_max)
  if(out_basin) write(unit=BOU,fmt=*) theta,zero,rlim
  if(out_basin) write(unit=BOU,fmt=*) theta,dpi,rlim
  rlim2=rlim*rlim
  call qromb_rho()
! surface, volume, q-tot, q-val, laplacian
  s_phi(2,1)=dpi*rlim2
  s_phi(1,1)=s_phi(2,1)*rlim
  s_phi(3,1)=dpi*rad_prop(1)
  s_phi(4,1)=dpi*rad_prop(2)
  s_phi(5,1)=dpi*rad_prop(3)
! dipole
  s_phi(6,1)=dpi*rad_prop(4)
  s_phi(7,1)=zero
  s_phi(8,1)=s_phi(6,1)
! quadrupole
  w=dpi*rad_prop(5)
  s_phi(9:14,1)=zero
  s_phi(9,1)=w
  s_phi(11,1)=w
  s_phi(13,1)=w
! octupole
  x=v_unit(1)
  y=v_unit(2)
  z=v_unit(3)
  w=dpi*rad_prop(6)
  s_phi(15,1)=w*x*(five*x*x-three)   !! xxx
  s_phi(16,1)=w*y*(five*x*x-one)     !! xxy
  s_phi(17,1)=w*z*(five*x*x-one)     !! xxz
  s_phi(18,1)=w*x*(five*y*y-one)     !! xyy
  s_phi(19,1)=w*z*(five*x*y    )     !! xyz
  s_phi(20,1)=w*x*(five*z*z-one)     !! xzz
  s_phi(21,1)=w*y*(five*y*y-three)   !! yyy
  s_phi(22,1)=w*z*(five*y*y-one)     !! yyz
  s_phi(23,1)=w*y*(five*z*z-one)     !! yzz
  s_phi(24,1)=w*z*(five*z*z-three)   !! zzz
else
  it=2**(n_phi-2)
  tnm_phi=real(it,double)
  dphi=dpi/tnm_phi
  phi=half*dphi
  sum_phi(1:24)=zero
  do j=1,it
    csp=cos(phi)
    snp=sin(phi)
    v_unit(1)=sinth*csp
    v_unit(2)=sinth*snp
    call rlim_interp()
    call rlim_and_dmin_adjust(lwrng)
    if(lwrng) write(unit=SOU,fmt="(a,f8.3,2a)") "WARNING !!!   rmin=",dmin," too high for atom ",lat(iat)
    rlim_min=min(rlim,rlim_min)
    rlim_max=max(rlim,rlim_max)
    if(out_basin) write(unit=10,fmt=*) theta,phi,rlim
    rlim2=rlim*rlim
    call qromb_rho()
! volume, surface
    sum_phi(1)=sum_phi(1)+rlim2*rlim
    sum_phi(2)=sum_phi(2)+rlim2
! q-tot, q-val, lap.
    sum_phi(3)=sum_phi(3)+rad_prop(1)
    sum_phi(4)=sum_phi(4)+rad_prop(2)
    sum_phi(5)=sum_phi(5)+rad_prop(3)
! dipole
    sum_phi(6)=sum_phi(6)+rad_prop(4)*csp
    sum_phi(7)=sum_phi(7)+rad_prop(4)*snp
    sum_phi(8)=sum_phi(8)+rad_prop(4)
! quadrupole
    x=v_unit(1)
    y=v_unit(2)
    z=v_unit(3)
    w=rad_prop(5)
    sum_phi(9)=sum_phi(9)+w*(three*x*x-one)   !!xx
    sum_phi(10)=sum_phi(10)+w*(three*y*y-one) !!yy
    sum_phi(11)=sum_phi(11)+w*(three*z*z-one) !!zz
    sum_phi(12)=sum_phi(12)+w*(three*x*y)     !!xy
    sum_phi(13)=sum_phi(13)+w*(three*x*z)     !!xz
    sum_phi(14)=sum_phi(14)+w*(three*y*z)     !!yz
! octupole
    w=rad_prop(6)
    sum_phi(15)=sum_phi(15)+w*x*(five*x*x-three)   !! xxx
    sum_phi(16)=sum_phi(16)+w*y*(five*x*x-one)     !! xxy
    sum_phi(17)=sum_phi(17)+w*z*(five*x*x-one)     !! xxz
    sum_phi(18)=sum_phi(18)+w*x*(five*y*y-one)     !! xyy
    sum_phi(19)=sum_phi(19)+w*z*(five*x*y    )     !! xyz
    sum_phi(20)=sum_phi(20)+w*x*(five*z*z-one)     !! xzz
    sum_phi(21)=sum_phi(21)+w*y*(five*y*y-three)   !! yyy
    sum_phi(22)=sum_phi(22)+w*z*(five*y*y-one)     !! yyz
    sum_phi(23)=sum_phi(23)+w*y*(five*z*z-one)     !! yzz
    sum_phi(24)=sum_phi(24)+w*z*(five*z*z-three)   !! zzz
!
    phi=phi+dphi
  end do
  do i=1,24
    s_phi(i,n_phi)=half*(s_phi(i,n_phi)+dphi*sum_phi(i))
  end do
end if
!
return
end subroutine trapz_phi

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine qromb_phi()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! performs romberg procedure for PHI integration
! cf. "NUMERICAL RECIPIES"
!
integer(kind=int32) :: i,j,jp,nd,nu
real(kind=double), dimension(jmaxp) :: h_phi
real(kind=double), dimension(24,jmaxp) :: sq_phi
logical :: lgp
character(len=3) :: count_ext
!
h_phi(1)=one
if(ph_study) then
  nd=count_phi/10
  nu=count_phi-10*nd
  count_ext="_"//char(48+nd)//char(48+nu)
  open(unit=R1U,file=trim(file_path)//trim(lat(iat))//count_ext//".phi.tot",status="new",action="write")
  write(unit=R1U,fmt=*) trim(lat(iat)),theta
  open(unit=R2U,file=trim(file_path)//trim(lat(iat))//count_ext//".phi.val",status="new",action="write")
  write(unit=R2U,fmt=*) trim(lat(iat)),theta
  open(unit=R3U,file=trim(file_path)//trim(lat(iat))//count_ext//".phi.vol",status="new",action="write")
  write(unit=R3U,fmt=*) trim(lat(iat)),theta
  open(unit=R4U,file=trim(file_path)//trim(lat(iat))//count_ext//".phi.lap",status="new",action="write")
  write(unit=R4U,fmt=*) trim(lat(iat)),theta
end if
sq_phi(1:24,1:jmaxp)=zero
!
do j=1,jmax
  call trapz_phi(sq_phi,j)
  if(j >= kint) then
    jp=j-km1
    call polint(h_phi(jp:j),sq_phi(:,jp:j),24,kmax,phi_prop,phi_dprop)
    if(ph_study) then
      write(unit=R1U,fmt=*) j,sq_phi(3,j),phi_prop(3)
      write(unit=R2U,fmt=*) j,sq_phi(4,j),phi_prop(4)
      write(unit=R3U,fmt=*) j,sq_phi(1,j),phi_prop(1)
      write(unit=R4U,fmt=*) j,sq_phi(5,j),phi_prop(5)
    end if
  else
    if(ph_study) then
      write(unit=R1U,fmt=*) j,sq_phi(3,j),sq_phi(3,j)
      write(unit=R2U,fmt=*) j,sq_phi(4,j),sq_phi(4,j)
      write(unit=R3U,fmt=*) j,sq_phi(1,j),sq_phi(1,j)
      write(unit=R4U,fmt=*) j,sq_phi(5,j),sq_phi(5,j)
    end if
  end if
!
  lgp=.true.
  do i=1,24
    if(conv_test(i)) then
      if(j >= kint) then
        lgp=lgp .and. abs(phi_dprop(i)) <= eps_phi*abs(phi_prop(i))
      else
        lgp=.false.
        exit
      end if
    end if
  end do
!
  if(lgp) then
    if(romb_stat) then
      nloop_phi_min=min(nloop_phi_min,j)
      nloop_phi_max=max(nloop_phi_max,j)
      nloop_thph=nloop_thph+1+2**(j-1)
    end if
    if(ph_study) then
      close(R1U)
      close(R2U)
      close(R3U)
      close(R4U)
    end if
    return
  end if
!
  do i=1,24
    sq_phi(i,j+1)=sq_phi(i,j)
  end do
  h_phi(j+1)=0.25_double*h_phi(j)
end do
write(unit=SOU,fmt="(a,i3,a)") "WARNING ::   convergence not reached after ",jmax," cycles !"
write(unit=SOU,fmt="(a,f8.2)") "    in qromb_phi at THETA = ",theta*deg
!
end subroutine qromb_phi

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine trapz_theta(s_theta,n_theta)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! trapezoidal integration at order "n"
!
!  integrated quantities with "theta" from 0 to pi :
!  s_theta(1) ::   volume
!  s_theta(2) ::   surface
!  s_theta(3:4) ::   TOTAL & VALENCE
!  s_theta(5) ::   laplacian
!  s_theta(6:8) ::   dipole
!  s_theta(9:14) ::   quadrupole
!  s_theta(15:24) ::   octupole
!
!  over theta & phi & known rlim(theta,phi)
!
!  v_unit :: unitary vector in direction theta,phi
!  rag :: atomic position in cartesian grid coordinates system
!
integer(kind=int32), intent(in) :: n_theta
real(kind=double), dimension(24,jmaxp), intent(out) :: s_theta
!
integer(kind=int32) :: i,it,j
real(kind=double) :: tnm_theta,sct,s2t
real(kind=double), dimension(24) :: sum_theta
logical :: lwrng   !! return flag ".true." if rlim <= dmin is persistent
!
if(n_theta == 1) then
  s_theta(1:24,1)=zero
  theta=zero
  v_unit(:)=zero
  phi=zero
  v_unit(3)=one
  call rlim_interp()
  call rlim_and_dmin_adjust(lwrng)
  if(lwrng) write(unit=SOU,fmt="(a,f8.3,2a)") "WARNING !!!   dmin=",dmin," too high for atom ",lat(iat)
  rlim_min=min(rlim,rlim_min)
  rlim_max=max(rlim,rlim_max)
  if(out_basin) write(unit=BOU,fmt=*) theta,phi,rlim
  theta=pi
  v_unit(3)=-one
  call rlim_and_dmin_adjust(lwrng)
  if(lwrng) write(unit=SOU,fmt="(a,f8.3,2a)") "WARNING !!!   dmin=",dmin," too high for atom ",lat(iat)
  rlim_min=min(rlim,rlim_min)
  rlim_max=max(rlim,rlim_max)
  if(out_basin) write(unit=BOU,fmt=*) theta,phi,rlim
else
  it=2**(n_theta-2)
  tnm_theta=real(it,double)
  dtheta=pi/tnm_theta
  theta=half*dtheta
  sum_theta(1:24)=zero
  do j=1,it
    sinth=sin(theta)
    costh=cos(theta)
    sct=sinth*costh
    s2t=sinth*sinth
    v_unit(3)=costh
    call qromb_phi()
    do i=1,5
      sum_theta(i)=sum_theta(i)+phi_prop(i)*sinth
    end do
! dipole
    sum_theta(6)=sum_theta(6)+phi_prop(6)*s2t
    sum_theta(7)=sum_theta(7)+phi_prop(7)*s2t
    sum_theta(8)=sum_theta(8)+phi_prop(8)*sct
! quadrupole
    do i=9,14
      sum_theta(i)=sum_theta(i)+phi_prop(i)*sinth  !! Jackson chap.4  equ. 4.9
    end do
! octupole
    do i=15,24
      sum_theta(i)=sum_theta(i)+phi_prop(i)*sinth
    end do
!
    theta=theta+dtheta
  end do
  do i=1,24
    s_theta(i,n_theta)=half*(s_theta(i,n_theta)+dtheta*sum_theta(i))
  end do
end if
return
end subroutine trapz_theta

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine qromb_theta()
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! performs romberg procedure for THETA integration
! cf. "NUMERICAL RECIPIES"
!
integer(kind=int32) :: i,j,jt
real(kind=double), dimension(jmaxp) :: h_theta
real(kind=double), dimension(24,jmaxp) :: sq_theta
logical :: lgth
!
h_theta(1)=one
if(th_study) then
  open(unit=R1U,file=trim(file_path)//trim(lat(iat))//".theta.tot",status="new",action="write")
  write(unit=R1U,fmt=fmta) trim(lat(iat))
  open(unit=R2U,file=trim(file_path)//trim(lat(iat))//".theta.val",status="new",action="write")
  write(unit=R2U,fmt=fmta) trim(lat(iat))
  open(unit=R3U,file=trim(file_path)//trim(lat(iat))//".theta.vol",status="new",action="write")
  write(unit=R3U,fmt=fmta) trim(lat(iat))
  open(unit=R4U,file=trim(file_path)//trim(lat(iat))//".theta.lap",status="new",action="write")
  write(unit=R4U,fmt=fmta) trim(lat(iat))
end if
!
sq_theta(1:24,1:jmaxp)=zero
!
do j=1,jmax
  call trapz_theta(sq_theta,j)
  if(j >= kint) then
    jt=j-km1
    call polint(h_theta(jt:j),sq_theta(:,jt:j),24,kmax,theta_prop,theta_dprop)
    if(th_study) then
      write(unit=R1U,fmt=*) j,sq_theta(3,j),theta_prop(3)
      write(unit=R2U,fmt=*) j,sq_theta(4,j),theta_prop(4)
      write(unit=R3U,fmt=*) j,sq_theta(2,j),theta_prop(2)
      write(unit=R4U,fmt=*) j,sq_theta(5,j),theta_prop(5)
    end if
  else
    if(th_study) then
      write(unit=R1U,fmt=*) j,sq_theta(3,j),sq_theta(3,j)
      write(unit=R2U,fmt=*) j,sq_theta(4,j),sq_theta(4,j)
      write(unit=R3U,fmt=*) j,sq_theta(2,j),sq_theta(2,j)
      write(unit=R4U,fmt=*) j,sq_theta(5,j),sq_theta(5,j)
    end if
  end if
!
  lgth=.true.
  do i=1,24
    if(conv_test(i)) then
      if(j >= kint) then
        lgth=lgth .and. abs(theta_dprop(i)) <= eps_theta*abs(theta_prop(i))
      else
        lgth=.false.
        exit
      end if
    end if
  end do
!
  if(lgth) then
    if(romb_stat) nloop_theta=j
    if(th_study) then
      close(R1U)
      close(R2U)
      close(R3U)
      close(R4U)
    end if
    return
  end if
!
  do i=1,24
    sq_theta(i,j+1)=sq_theta(i,j)
  end do
  h_theta(j+1)=0.25_double*h_theta(j)
end do
write(unit=SOU,fmt="(a,i3,a)") "WARNING ::   convergence not reached after ",jmax," cycles !"
end subroutine qromb_theta

!>>>>>
!>
!>     OUTPUT ROUTINES
!>
!>>>>>
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_parameters(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu  !!! logical unit number
!
character(len=*), parameter :: fmt0="(a,f7.2)"
character(len=*), parameter :: fmt1="(a,f7.4)"
character(len=*), parameter :: fmt2="(a,f15.7,a)"
!
write(unit=lu,fmt=fmta) " ___________________________________________________________________________"
write(unit=lu,fmt=*)
write(unit=lu,fmt="(tr1,a)") trim(file_path)//trim(filename_1)
write(unit=lu,fmt=fmta) " ___________________________________________________________________________"
write(unit=lu,fmt=*)
write(unit=lu,fmt="(a,2i3,i5)") " DATE :: ",datetime(3:1:-1)
write(unit=lu,fmt=*)
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) " Criteria for atomic basin determination :"
write(unit=lu,fmt=fmta) " =========================================="
write(unit=lu,fmt="(a,i3)") " # maximum run away steps           : ",max_step_fwd
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt0) " # exponent for step variation      : ",exp_dg
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) " # coarse search : "
write(unit=lu,fmt="(a,2i4)") "  - theta & phi steps numbers       : ",nth,nphi
write(unit=lu,fmt=fmt0) "  - forward coefficient             : ",ea_coarse
write(unit=lu,fmt=fmt0) "  - backward coefficient            : ",ed_coarse
write(unit=lu,fmt=fmt1) "  - dichotomy tolerance stop        : ",dtol_coarse
write(unit=lu,fmt=fmt1) "  - minimum step following gradient : ",dr_coarse
write(unit=lu,fmt=fmt0) "  - maximum magnification of step   : ",fa_coarse
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) " # fine search : "
write(unit=lu,fmt=fmt0) "  - forward coefficient             : ",ea_fine
write(unit=lu,fmt=fmt0) "  - backward coefficient            : ",ed_fine
write(unit=lu,fmt=fmt1) "  - dichotomy tolerance stop        : ",dtol_fine
write(unit=lu,fmt=fmt1) "  - minimum step following gradient : ",dr_fine
write(unit=lu,fmt=fmt0) "  - maximum magnification of step   : ",fa_fine
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt0) " # Coef. applied to coarse minimum radius : ",coef_dmin
write(unit=lu,fmt=*)
if(index(running_mode,"grid") /= 0) then
  write(unit=lu,fmt=fmta) " Regular Grid Integration :"
  write(unit=lu,fmt=fmta) " ====================="
  write(unit=lu,fmt=fmta) " # Convergence levels  :"
  write(unit=lu,fmt="(a,i4)") "  - Theta steps        : ",ntg
  write(unit=lu,fmt="(a,i4)") "  - Radial steps       : ",nrg
  write(unit=lu,fmt=*)
else
  write(unit=lu,fmt=fmta) " Romberg Parameters :"
  write(unit=lu,fmt=fmta) " ====================="
  write(unit=lu,fmt=fmta) ""
  write(unit=lu,fmt=fmta) "    Romberg Integration Method"
  write(unit=lu,fmt=fmta) "    Press et al. Numerical Recipies ch.4, Cambrige Univ. Press"
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) " # Convergence levels  :"
  write(unit=lu,fmt="(a,i6)") "  - Maximum Iteration  : ",jmax
  write(unit=lu,fmt=fmt2) "  - Theta angle        : ",eps_theta
  write(unit=lu,fmt=fmt2) "  - Phi angle          : ",eps_phi
  write(unit=lu,fmt=fmt2) "  - Radius             : ",eps_r
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) " # Interpolation       :"
  write(unit=lu,fmt="(a,i3)") "  - Polynomial order   : ",kmax
  write(unit=lu,fmt="(a,i3)") "  - Min. # iterations  : ",kmin
  write(unit=lu,fmt=*)
  write(unit=lu,fmt=fmta) " # Used criterion for convergence :"
  write(unit=lu,fmt="(2a)") "  - ",conv_string
  write(unit=lu,fmt=*)
end if
if(tot_use) then
  write(unit=lu,fmt=fmta) " # TOTAL density used for Laplacian sum and Multipoles"
else
  write(unit=lu,fmt=fmta) " # VALENCE density used for Laplacian sum and Multipoles"
end if
write(unit=lu,fmt=*)
!
end subroutine print_parameters

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_grid(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu  !!! logical unit number
!
character(len=*), parameter :: fmt0="(tr3,a,3f10.5)"
character(len=*), parameter :: fmt1="(tr3,a,es15.7)"
character(len=*), parameter :: fmt2="(tr5,i3,a,3f12.7,a)"
character(len=*), parameter :: fmt3="(tr5,a,es15.8)"
character(len=*), parameter :: fmt4="(tr5,2(a,es15.8))"
!
write(unit=lu,fmt=fmta) " --------"
write(unit=lu,fmt=fmta) "| GRID   |"
write(unit=lu,fmt=fmta) " --------"
write(unit=lu,fmt=fmt0) "# Origin             (au)   : ",origin
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt0) "# Mesh grid :"
write(unit=lu,fmt=fmt2) np1," x { del1 = ",del1,"}"
write(unit=lu,fmt=fmt2) np2," x { del2 = ",del2,"}"
write(unit=lu,fmt=fmt2) np3," x { del3 = ",del3,"}"
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt1) "# Elementary Volume  (au^3) : ",delv
write(unit=lu,fmt="(tr3,a,f12.5)") "  Cell Volume (au^3)        : ",grid_volume
write(unit=lu,fmt=*)
if(iso_test) then
  write(unit=lu,fmt=fmt0) "# Non periodic conditions applied"
else
  write(unit=lu,fmt=fmt0) "# Periodic conditions applied"
end if
if(vdv_test) then
  write(unit=lu,fmt=fmt1) "# V.d.W. Surface at rho (e-/au^3) : ",rho_vdv
else
  write(unit=lu,fmt=fmt1) "# No surface density condition"
end if
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt0) "# Integration over all box (restriction applied in case of non periodicity) :"
write(unit=lu,fmt=fmt4) " TOTAL electron number  : ",tot_grid_charge," --> ",tot_grid_charge-nb_e_tot
write(unit=lu,fmt=fmt3) "            minimum rho : ",rho_tot_min
write(unit=lu,fmt=fmt3) "            maximum rho : ",rho_tot_max
write(unit=lu,fmt=fmt4) " VALENCE electron number: ",val_grid_charge," --> ",val_grid_charge-nb_e_val
write(unit=lu,fmt=fmt3) "            minimum rho : ",rho_val_min
write(unit=lu,fmt=fmt3) "            maximum rho : ",rho_val_max
write(unit=lu,fmt=*)
!
if(index(running_mode,"plane") /= 0) then
  write(unit=lu,fmt=fmta) ""
  write(unit=lu,fmt=fmta) " -------"
  write(unit=lu,fmt=fmta) "| PLANE |"
  write(unit=lu,fmt=fmta) " -------"
  write(unit=lu,fmt="(2a)") "   # ",trim(plane_def)
  if(len_trim(lbl1) <= 0 .or.len_trim(lbl2) <= 0 .or. len_trim(lbl2) <= 0) then
    write(unit=lu,fmt=fmta) "   # default basal grid plane (i,j)"
  else
    write(unit=lu,fmt="(a,3a6)") "   # atoms 1,2&3 defining plane : ",lbl1,lbl2,lbl3
  end if
  if(index(middle_23,"y") /= 0) write(unit=lu,fmt=fmta) "   # Ox axis defined from atom 1 and middle of atoms 2&3"
  write(unit=lu,fmt="(4a)") "   # plane ",trim(perppl)," ",trim(perpxy)
  write(unit=lu,fmt="(2(a,i2,a,f6.3),a)") "   # dimensions : (",nx,"*",incx,") x (",ny,"*",incy,")"
  if(index(molor,"molecular") /= 0) then
    write(unit=lu,fmt="(3a)") "   # ",molor," derived from atom 1"
  else
    write(unit=lu,fmt="(a,2f8.3)") "   # X&Y offset : ",dxo,dyo
  end if
  write(unit=lu,fmt="(a,f8.3)") "   # Z offset : ",dzo
  write(unit=lu,fmt="(a,f8.3)") "   # distance of selected surface points to plane : ",tol2plane
  write(unit=lu,fmt="(a,i3)") "     minimum number of points for atom slection   : ",npcp
  write(unit=lu,fmt="(a,i3)") "   # number of angular steps : ",nal
end if
write(unit=lu,fmt=*)
!
call date_and_time(values=datetime)
start_time(:)=datetime(:)
!
end subroutine print_grid

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_atoms(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu  !!! logical unit number
!
integer(kind=int32) :: i
!
character(len=*), parameter :: fmt0="(tr6,a6,tr2,3f10.5,tr1,2i4,tr4,f7.3,tr2,i2,tr2,f4.1)"
character(len=*), parameter :: fmt1="(tr5,3(a,tr9),a,tr8,a,tr2,a,tr4,2(a,tr3),a)"
character(len=*), parameter :: fmt2="(tr3,a)"
!
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta)              " ------------"
write(unit=lu,fmt="(a,i3,2a,i2,a)") "| ",nbas," ATOMS  |","     among ",mol_number,"  molecules"
write(unit=lu,fmt=fmta)              " ------------"
!
write(unit=lu,fmt=fmt2) "# Molecules origin :"
do i=1,mol_number
  write(unit=lu,fmt="(tr5,a,i2,a,3f10.5)") "No",i," at ",ori_mol(i,:)
end do
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmt2) "# Atoms :"
!
write(unit=lu,fmt=fmt1) "name","X","Y","Z","#e","#val","rmin","mol#","mult"
write(unit=lu,fmt=fmt2) "___________________________________________________________________________"
do i=1,nbas
  iat=indx_atoms(i)
  checked_atom=lat(iat)
! is current atom true atom or dummy, or Critical Point ?
  write(unit=lu,fmt=fmt0) lat(iat),rat(iat,:),nb_e_atom(iat),nb_eval_atom(iat),rmin_atom(iat),mol_index(iat),mult_factor(iat)
end do
write(unit=lu,fmt=fmt2) "___________________________________________________________________________"
write(unit=lu,fmt=fmta) ""
!
end subroutine print_atoms

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_atom_header(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu  !!! logical unit number
!
rag(:)=rat(iat,:)-origin(:)
rgg(:)=matmul(c2g,rag)
r_start=rstart_atom(iat)
dmin=rmin_atom(iat)
!
write(unit=lu,fmt=*)
call date_and_time(values=datetime)
write(unit=lu,fmt=fmta) "==================================================================================="
write(unit=lu,fmt="(a6,i2,tr2,a6,tr2,a,f4.1,tr5,a,f4.2,tr6,a,i2,tr8,2i3,i5,tr2,3(i2,a1))") " ATOM_",iat,lat(iat),"MULT=", &
mult_factor(iat),"RMIN=",rmin_atom(iat),"Mol#",mol_index(iat),datetime(3:1:-1),datetime(5),"h",datetime(6),"m",datetime(7),"s"
write(unit=lu,fmt=fmta) "-----------------------------------------------------------------------------------"
write(unit=lu,fmt="(tr2,a4,3f9.5,tr5,a5,3f9.3)") "XYZ=",rat(iat,:)," IJK=",rgg(:)
write(unit=lu,fmt="(tr2,a,tr2,2f10.5)") "Start and min. radii for coarse search    : ",r_start,dmin
!
end subroutine print_atom_header

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_coarse_basin_stat(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu  !! logical unit number
!
character(len=*), parameter :: fmt0="(tr2,a,tr2,2es12.5)"
!
write(unit=lu,fmt=fmt0) "Coarse min & max of limit radii           : ",rlim_min,rlim_max
write(unit=lu,fmt=fmt0) "Gradient minimum  & maximum               : ",min_grad,max_grad
write(unit=lu,fmt=fmt0) "Explored distance  minimum  & maximum     : ",min_dist,max_dist
write(unit=lu,fmt=fmt0) "Rho minimum  & maximum                    : ",min_rho,max_rho
write(unit=lu,fmt=fmt0) "A priori minimum radius for fine search   : ",dmin
!
end subroutine print_coarse_basin_stat

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_atom_results(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu
!
integer(kind=int32) :: nj,nh,nm,ns,i,j,k,m
real(kind=double) :: dip
character(len=1), dimension(3) :: oc
character(len=6) :: octu
!
character(len=*), parameter :: fmt0="(tr2,a,tr2,2f12.5)"
character(len=*), parameter :: fmt1="(tr2,a,3i12)"
character(len=*), parameter :: fmt2="(tr2,2(a,f12.5))"
character(len=*), parameter :: fmt3="(tr2,a,3f9.5,a)"
!
write(unit=lu,fmt=fmt0) "Min & max rlim                            : ",integ_prop(ibas,10:11)
write(unit=lu,fmt=fmt0) "Rlim mean & std. dev                      : ",integ_prop(ibas,12:13)
if(.not. boundary_check .and. vdv_test) then
  write(unit=lu,fmt="(tr2,a)") " V.d.W Surface partially limited by box boundary !!!"
end if
if(integ_prop(iat,58)== one) then
  write(unit=lu,fmt="(tr2,a)") " Input minimum radius too high for this atom !!!"
end if
write(unit=lu,fmt=*)
!
if(romb_stat) then
  write(unit=lu,fmt="(tr2,a,i12)") "Total integration loops #        : ",int(integ_prop(ibas,14))
  write(unit=lu,fmt="(tr2,a,i12)") "Theta loops #                    : ",int(integ_prop(ibas,21))
  write(unit=lu,fmt="(tr2,a,2i12)") "Theta+phi loops #                : ",int(integ_prop(ibas,22))
  write(unit=lu,fmt=fmt1) "Min, mean & max phi steps #      : ",int(integ_prop(ibas,18:20))
  write(unit=lu,fmt=fmt1) "Min, mean & max radial steps #   : ",int(integ_prop(ibas,15:17))
end if
!
atom_vol_err=integ_prop(ibas,24)*dtol
tot_atom_diff=integ_prop(ibas,26)-nb_e_atom(iat)
val_atom_diff=integ_prop(ibas,27)-nb_eval_atom(iat)
write(unit=lu,fmt=*)
write(unit=lu,fmt="(tr2,a,f12.5)") "Atomic Surface (au^2)            : ",integ_prop(ibas,24)
write(unit=lu,fmt=fmt2) "Atomic Volume (au^3)             : ",integ_prop(ibas,25), " +/-",atom_vol_err
write(unit=lu,fmt=fmt2) "Atomic TOTAL   e- number         : ",integ_prop(ibas,26), " -->",tot_atom_diff
write(unit=lu,fmt=fmt2) "Atomic VALENCE e- number         : ",integ_prop(ibas,27), " -->",val_atom_diff
write(unit=lu,fmt="(tr2,a,f12.5)") "Integrated Laplacian (e- / au^2) : ",integ_prop(ibas,28)
!
dip=sqrt(dot_product(integ_prop(iat,29:31),integ_prop(iat,29:31)))
!
write(unit=lu,fmt=*)
write(unit=lu,fmt="(tr2,a,f12.5)") "Dipole (e- * au)                 : ",dip
if(dip /= zero) then
  write(unit=lu,fmt=fmt3) "      | ex ey ez |                    |",integ_prop(iat,29:31)/dip," |"
end if
!
quad_mat(1,1)=integ_prop(ibas,32)
quad_mat(2,2)=integ_prop(ibas,33)
quad_mat(3,3)=integ_prop(ibas,34)
quad_mat(1,2)=integ_prop(ibas,35)
quad_mat(1,3)=integ_prop(ibas,36)
quad_mat(2,3)=integ_prop(ibas,37)
quad_mat(2,1)=quad_mat(1,2)
quad_mat(3,1)=quad_mat(1,3)
quad_mat(3,2)=quad_mat(2,3)
call eigen(quad_mat,quad_eval,quad_evec)
write(unit=lu,fmt=*)
write(unit=lu,fmt="(tr2,a,3f12.5)") "Quadrupole (e- * au^2)           : ",quad_eval
write(unit=lu,fmt=fmt3) "      | e1x e2x e3x |                 |",(quad_evec(1,j),j=1,3)," |"
write(unit=lu,fmt=fmt3) "      | e1y e2y e3y |                 |",(quad_evec(2,j),j=1,3)," |"
write(unit=lu,fmt=fmt3) "      | e1z e2z e3z |                 |",(quad_evec(3,j),j=1,3)," |"
write(unit=lu,fmt=*)
write(unit=lu,fmt="(tr2,a)") "Octupole (e- * au^3):"
write(unit=lu,fmt=*)
oc(1)="x"
oc(2)="y"
oc(3)="z"
m=37
do i=1,3
  do j=i,3
    do k=j,3
      octu="O"//oc(i)//oc(j)//oc(k)//": "
      m=m+1
      write(unit=lu,fmt="(tr24,a6,f10.5)") octu,integ_prop(iat,m)   !38:47
    end do
  end do
end do
write(unit=lu,fmt=*)
nj=int(integ_prop(iat,54))
nh=int(integ_prop(iat,55))
nm=int(integ_prop(iat,56))
ns=int(integ_prop(iat,57))
write(unit=lu,fmt="(tr2,a,4(i3,a))") "elapsed time    : ",nj,"j ",nh,"h ",nm,"m ",ns,"s"
!
end subroutine print_atom_results

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine date_time_to_sec(dt,s)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! convert date and time to absolute time in sec
!
integer(kind=int32), intent(out) :: s
integer(kind=int32), dimension(8), intent(in) :: dt
!
integer(kind=int32) :: mj
integer(kind=int32), parameter :: aj=365
integer(kind=int32), parameter :: jh=24
integer(kind=int32), parameter :: hm=60
integer(kind=int32), parameter :: ms=60
!
select case(dt(2))
case(4,6,9,11)
  mj=30
case(2)
  if(modulo(dt(1),4) == 0 .and. modulo(dt(1),100) /= 0) then
    mj=29
  else
    mj=28
  end if
case default
  mj=31
end select
!
s=(((dt(1)*aj+dt(2)*mj+dt(3))*jh+dt(5))*hm+dt(6))*ms+dt(7)
!
end subroutine date_time_to_sec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine integration_time(t1,t2,jj,hh,mm,ss)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! conversion from date & time to seconds
!
integer(kind=int32), intent(in) :: t1,t2
integer(kind=int32), intent(out) :: jj,hh,mm,ss
!
integer(kind=int32) :: dt
integer(kind=int32), parameter :: js=86400
integer(kind=int32), parameter :: hs=3600
integer(kind=int32), parameter :: ms=60
!
dt=t2-t1
jj=dt/js
hh=(dt-jj*js)/hs
mm=(dt-jj*js-hh*hs)/ms
ss=(dt-jj*js-hh*hs-mm*ms)
end subroutine integration_time

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
subroutine print_cell_results(lu)
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
integer(kind=int32), intent(in) :: lu
!
integer(kind=int32) :: imol,st,et
real(kind=double) :: dip
!
character(len=*), parameter :: fmt0="(3(a,es14.7,tr2))"
character(len=*), parameter :: fmt1="(2(a,es14.7,tr2))"
character(len=*), parameter :: fmt2="(a,es14.7)"
!
call date_time_to_sec(end_time,et)
call date_time_to_sec(start_time,st)
call integration_time(st,et,j_time,h_time,m_time,s_time)
!
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) "-----------------------------------------------------------------------------"
write(unit=lu,fmt=*)
write(unit=lu,fmt="(a,i3,a1,3(i2,a1))") " Integration time :",j_time,"j",h_time,"h",m_time,"m",s_time,"s"
write(unit=lu,fmt=*)
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) " -----------------------------------"
write(unit=lu,fmt=fmta) "| SUMMATION OVER BASINS & MOLECULES |"
write(unit=lu,fmt=fmta) " -----------------------------------"
write(unit=lu,fmt=*)
write(unit=lu,fmt=fmta) " All Cell"
write(unit=lu,fmt=fmta) "============="
write(unit=lu,fmt=fmt0) "  Volume (au^3)                   : ",cell_volume," +/-", tot_vol_err," -->",cell_volume-grid_volume
write(unit=lu,fmt=fmt1) "  TOTAL   e- number               : ",tot_cell_charge," -->", tot_cell_charge-nb_e_tot
write(unit=lu,fmt=fmt1) "  VALENCE e- number               : ",val_cell_charge," -->", val_cell_charge-nb_e_val
write(unit=lu,fmt=fmt2) "  Integrated Laplacian (e- /au^2) : ",cell_laplacian
write(unit=lu,fmt=*)
!
do imol=1,mol_number
  dip=sqrt(dot_product(dip_mol(imol,:),dip_mol(imol,:)))
  write(unit=lu,fmt="(a,i2)") " Molecule #",imol
  write(unit=lu,fmt=fmta) "============="
  write(unit=lu,fmt=fmt1) "  Volume (au^3)                    : ",mol_volume(imol)
  write(unit=lu,fmt=fmt1) "  TOTAL   e- number                : ",tot_mol_charge(imol),"->", &
    tot_mol_charge(imol)-real(nb_e_mol(imol),double)
  write(unit=lu,fmt=fmt1) "  VALENCE e- number                : ",val_mol_charge(imol),"->", &
    val_mol_charge(imol)-real(nb_eval_mol(imol),double)
  write(unit=lu,fmt=fmt2) "  Integrated Laplacian (e- /au^2)  : ",mol_laplacian(imol)
  write(unit=lu,fmt="(tr2,a,3f12.5)") "Molecular Dipole (Debye)           : ",dip*2.541748
  if(dip /= zero) then
    write(unit=lu,fmt="(tr2,a,3f9.5,a)") "      | ex ey ez |                    |",dip_mol(imol,:)/dip," |"
  end if
  write(unit=lu,fmt=*)
end do
!
end subroutine print_cell_results

end module integrity_mod
