module basin_module

use kind_numbers
implicit none
private

integer(kind=int32), public :: nth=30 !! # theta points for coarse basin limits search
integer(kind=int32), public :: nphi=60 !! # phi points for coarse basin limits search
integer(kind=int32), public :: max_step_fwd=3 !! max steps # following gradient outward attractor
integer(kind=int32), public :: min_step_dwd=5 !! min steps # following gradient downward attractor

real(kind=double), public :: dr_coarse=5.0e-2_double !! coarse increment along gradient path
real(kind=double), public :: dr_fine=5.0e-4_double !! fine increment along gradient path
real(kind=double), public :: ea_coarse=0.25_double !! coarse forward coefficient for basin limit search
real(kind=double), public :: ed_coarse=0.5_double  !! coarse backward coefficient for basin limit search
real(kind=double), public :: ea_fine=0.03_double !! fine forward coefficient for basin limit search
real(kind=double), public :: ed_fine=0.03_double !! fine backward coefficient for basin limit search
real(kind=double), public :: dtol_coarse=5.0e-2_double !! coarse tolerance for dicothomy stop
real(kind=double), public :: dtol_fine=5.0e-4_double !! fine tolerance for dicothomy stop
real(kind=double), public :: fa_coarse=5.0_double !! ==> dr_max = fa_coarse x dr
real(kind=double), public :: fa_fine=200.0_double !! ==> dr_max = fa_fine x dr
real(kind=double), public :: exp_dg=1.5_double !! exponent for accelerating increment downward nucleus 
real(kind=double), public :: coef_dmin=0.98_double !! coef_dmin x rlim_min, security factor after coarse search
real(kind=double), public :: dr                !! running increment along gradient path
real(kind=double), public :: ea,ed             !! running coefficient for coarse search
real(kind=double), public :: lnfa              !!
real(kind=double), public :: dtol              !! running tolerance for dichotomy stop
real(kind=double), public :: dmin              !! running atomic minimum basin radius
real(kind=double), public :: rinf              !! coarse lower bound for basin limit
real(kind=double), public :: rsup              !! coarse upper bound for basin limit
real(kind=double), public :: rlim              !! running basin limit
real(kind=double), public :: rlim_mean         !! mean atomic basin radius
real(kind=double), public :: rlim_sig          !! atomic basin radius std. dev.
real(kind=double), public :: theta,phi         !! spherical coordinates
real(kind=double), public :: dtheta,dphi,delr  !! running increments for spherical coordinates
real(kind=double), public :: sinth,costh      
real(kind=double), public :: min_grad, max_grad !! gradient moduli statistics
real(kind=double), public :: min_dist, max_dist !! explored distances statistics
real(kind=double), public :: min_rho, max_rho   !! density statistics
real(kind=double), public :: dens_tot,dens_val  !! density at rlim(theta,phi)
real(kind=double), dimension(3), public :: v_unit !! unit vector along theta & phi direction
real(kind=double), dimension(3), public :: d_unit !! unit vector along running position
real(kind=double), dimension(3), public :: g_unit !! unit vector along gradient at running position
real(kind=double), dimension(3), public :: grad_tot,grad_val  !!! gradient at rlim(theta,phi)
real(kind=double), dimension(3), public :: grad1,grad2        !!! grid_gradient at running point or basin boundary
real(kind=double), dimension(:,:,:), allocatable, public :: rlim_thph

logical, public :: inout_basin       !! boolean "belongs to atomic basin"
logical, public :: read_out_basin=.false. !! do not a priori write atomic basins output files
logical, public :: long_output=.false. !! do not a priori write "long" atomic basins output files
logical, public :: romb_stat=.true.
logical, public :: out_basin,cofi
logical, public :: out_box          !! flag for out of box running point

end module basin_module
